<?php

use SHD\Smview\Connect\Service\Aeneis\Exporter;
use SHD\Smview\Connect\Service\Aeneis\Importer;

/**
 * i-doit
 *
 * Aeneis API for SM-VIEW Connect.
 *
 * @package     i-doit
 * @subpackage  smview_connect
 * @author      Steve Groeber <steve.groeber@shd-online.de>
 * @copyright   2023 SHD System-Haus-Dresden GmbH
 * @license     http://www.i-doit.com/license
 */
class connect_api_model_aeneis extends isys_api_model_cmdb implements isys_api_model_interface
{

    /**
     * Read importable objects.
     *
     * @param  array $params
     * @return array
     * @throws isys_exception_api
     */
    public function read($params)
    {
        if (! isset($params['type'])) {
            throw new isys_exception_api("Missing 'type' param. It must be the aeneis type.");
        }

        /** @var Importer */
        $importer = isys_application::instance()->container->get('smview_connect.aeneis.importer');
        $type = $params['type'];
        $resolveValues = $params['resolve_values'] ?? false;
        $objects = $importer->getImportObjects($type, $resolveValues);

        // convert object properties to simple array
        $newObjects = [];
        foreach ($objects['objects'] as $o) {
            $o['properties'] = array_values($o['properties']);
            $newObjects[] = $o;
        }
        return [
            'objects'               => $newObjects,
            'property_descriptions' => array_values($objects['properties']),
        ];
    }

    /**
     * Read exportable objects.
     *
     * @param  array $params
     * @return array
     * @throws isys_exception_api
     */
    public function read_export(array $params): array
    {
        if (! isset($params['type'])) {
            throw new isys_exception_api("Missing 'type' param. It must be the aeneis type.");
        }

        /** @var Exporter */
        $exporter = isys_application::instance()->container->get('smview_connect.aeneis.exporter');
        $type = $params['type'];
        return $exporter->getExportObjects($type);
    }

    /**
     * Import selected objects.
     *
     * @param  array $params
     * @return array
     * @throws isys_exception_api
     * @throws Exception
     */
    public function import(array $params): array
    {
        $lang = isys_application::instance()->container->get('language');
        if (! isset($params['type'])) {
            throw new isys_exception_api("Missing 'type' param. It must be the aeneis type.");
        }

        if (empty($params['items'])) {
            throw new isys_exception_api("Missing 'items' param. It must be a comma-separated list of items.");
        }

        /** @var Importer $importer */
        $importer = isys_application::instance()->container->get('smview_connect.aeneis.importer');
        $type = $params['type'];
        $objectGuids = json_decode($params['items'], true);
        $resolveObjectValues = $params['resolve_values'] ?? false;
        try {
            $importer->saveObjectsToCmdb($type, $objectGuids, $resolveObjectValues);
        } catch (isys_exception_api $e) {
            return [
                'success' => false,
                'error' => $lang->get('LC__CMDB__SMVIEW_CONNECT__AENEIS_ERROR_IMPORT'),
            ];
        }

        return [
            'success' => true,
            'message' => $lang->get('LC__CMDB__SMVIEW_CONNECT__AENEIS_IMPORT_SUCCESSFUL'),
        ];
    }

    /**
     * Export selected objects.
     *
     * @param array $params
     *
     * @return array
     * @throws isys_exception_api
     */
    public function export(array $params): array
    {
        $lang = isys_application::instance()->container->get('language');
        if (! isset($params['type'])) {
            throw new isys_exception_api("Missing 'type' param. It must be the aeneis type.");
        }

        if (empty($params['items'])) {
            throw new isys_exception_api("Missing 'items' param. It must be a comma-separated list of items.");
        }

        /** @var Exporter */
        $exporter = isys_application::instance()->container->get('smview_connect.aeneis.exporter');
        $type = $params['type'];
        $objectIds = json_decode($params['items'], true);
        try {
            $exporter->saveObjectsToAeneis($type, $objectIds);
        } catch (isys_exception_api $e) {
            return [
                'success' => false,
                'error'   => $lang->get('LC__CMDB__SMVIEW_CONNECT__AENEIS_ERROR_EXPORT') . ': ' . $e->getMessage(),
            ];
        }

        return [
            'success' => true,
            'message' => $lang->get('LC__CMDB__SMVIEW_CONNECT__AENEIS_EXPORT_SUCCESSFUL'),
        ];
    }
}
