<?php

use SHD\Smview\Connect\Domain\Baramundi\Endpoint;
use SHD\Smview\Connect\Service\Baramundi\EndpointImporter;

/**
 * i-doit
 *
 * Baramundi Applications API for SM-VIEW Connect.
 *
 * @package     i-doit
 * @subpackage  smview_connect
 * @author      Florian Weber <florian.weber@shd-online.de>
 * @copyright   2023 SHD System-Haus-Dresden GmbH
 * @license     http://www.i-doit.com/license
 */
class connect_api_model_baramundi_applications extends shd_api_model_cmdb_cachable
{
    const CACHE_KEY = 'baramundi_applications';

    /**
     * @param $p_params
     * @return array|null
     * @throws Exception
     */
    public function read($p_params)
    {
        if ($this->useAuth) {
            isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');
        }

        $key = self::getCacheKey($p_params);

        if($this->hasCache($key) && $this->isValid($key)) {
            return $this->retreive($key);
        }

        return [];
    }

    /**
     * @param array $parameters
     * @return array
     * @throws \Seld\JsonLint\ParsingException
     * @throws \Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface
     */
    public static function retreiveData(array $parameters): array
    {
        /** @var EndpointImporter $importer */
        $importer = isys_application::instance()->container->get('smview_connect.baramundi.endpoint_importer');

        if(!$importer->status($parameters['profile_id'])) {
            return [];
        }

        $apps = $importer->getApplicationsFromApi($parameters['profile_id'], null);

        $applications = [];

        foreach ($apps as $app) {
            if(!in_array($app->Name, $applications)) {
                $applications[] = $app->Name;
            }
        }

        return $applications;
    }

    /**
     * @param $p_params
     * @return array
     * @throws Exception
     */
    public function status($p_params): array
    {
        try {
            /** @var EndpointImporter $importer */
            $importer = isys_application::instance()->container->get('smview_connect.baramundi.endpoint_importer');

            return [
                'online' => $importer->status($p_params['profile_id']),
            ];
        } catch (Throwable $t) {
            /** @var \SHD\Smview\Connect\Service\ConnectLogger $logger */
            $logger = isys_application::instance()->container->get('smview_connect.logger');
            $logger->log($t->getMessage(), 'Baramundi Status', \Monolog\Logger::ERROR);

            return [
                'online' => false,
            ];
        }
    }

    public static function getCacheKey(array $parameters): string
    {
        return self::CACHE_KEY.$parameters['profile_id'];
    }

    public function isBasedOnRemoteData(): bool
    {
        return true;
    }

    public function isRemoteAccessible(array $parameters): bool
    {
        $status = $this->status($parameters);

        return $status['online'];
    }
}
