<?php

use SHD\Smview\Connect\DAO\ProfileAssignmentDAO;
use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\Domain\Gcache\GcacheEntryMapping;
use SHD\Smview\Connect\Service\ConnectLogger;
use SHD\Smview\Connect\Service\CsvImporterInterface;
use SHD\Smview\Connect\Service\DB\DaoUtil;

class connect_api_model_csv extends isys_api_model_cmdb implements isys_api_model_interface
{
    /**
     * @param $p_params
     * @return array
     * @throws isys_exception_database
     * @throws isys_exception_general
     */
    public function read($p_params)
    {
        if ($this->useAuth) {
            isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');
        }

        $id = $p_params['id'] ?? null;
        $profile = DaoUtil::allRows(ProfileDAO::getInstance()->get_data($id))[0] ?? null;

        if(!$profile) {
            throw new isys_exception_general('profile not found');
        }

        list($importer, $mapping) = $this->getImporter($profile);

        $temp = $this->makeTmp($p_params['text']);
        $path = stream_get_meta_data($temp)['uri'];

        try {
            $entries = $importer->read($path, $mapping);
        } catch (Throwable $e) {
            $entries = [];
        }

        fclose($temp);

        return intval($p_params['limit']) > 0 ? array_slice($entries, 0, intval($p_params['limit'])) : $entries;
    }

    /**
     * @param $p_params
     * @return array
     * @throws isys_exception_database
     * @throws isys_exception_general
     */
    public function import($p_params): array
    {
        if ($this->useAuth) {
            isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');
        }

        /** @var ConnectLogger $logger */
        $logger = isys_application::instance()->container->get('smview_connect.logger');

        $id = $p_params['id'] ?? null;
        $profile = DaoUtil::allRows(ProfileDAO::getInstance()->get_data($id))[0] ?? null;

        if(!$profile) {
            throw new isys_exception_general('profile not found');
        }

        /** @var $importer CsvImporterInterface */
        list($importer, $mapping) = $this->getImporter($profile);

        $temp = null;
        if($p_params['text'] != '') {
            $temp = $this->makeTmp($p_params['text']);
            $path = stream_get_meta_data($temp)['uri'];
        } else {
            $path = $importer->getConfig()->getSetting($profile['id'], 'csv.path') ?? null;
        }

        $time_pre = microtime(true);

        try {
            $importer->import($path, $mapping);
        } catch (Throwable $e) {
            $logger->log($e->getMessage(), 'connect_api_model_csv::import', \Monolog\Logger::ERROR);
        }

        if(!(!$temp)) {
            fclose($temp);
        }

        $time_post = microtime(true);
        $exec_time = $time_post - $time_pre;

        return [
            'time' => $exec_time,
            'time_m' => $exec_time / 60,
        ];
    }

    private function makeTmp(string $content)
    {
        $temp = tmpfile();
        fwrite($temp, $content);

        return $temp;
    }

    /**
     * @param array $profile
     * @return array
     * @throws isys_exception_database
     * @throws isys_exception_general
     */
    private function getImporter(array $profile): array
    {
        /** @var CsvImporterInterface $importer */
        $importer = null;

        switch ($profile['connect_module']) {
            case 'GCACHE':
                /** @var \SHD\Smview\Connect\Service\Gcache\Importer $importer */
                $importer = \isys_application::instance()->container->get('smview_connect.gcache.importer');

                $_mapping = DaoUtil::allRows(ProfileAssignmentDAO::getInstance()->get_data($profile['id'], ProfileAssignmentDAO::ASSIGNMENT_TYPE_CSV)) ?? [];
                $mapping = new GcacheEntryMapping();

                foreach ($_mapping as $_map) {
                    $mapping->{$_map['source_field']} = $_map['meta1'];
                }

                break;
            default:
                throw new isys_exception_general(sprintf('module %s has no csv importer', $profile['connect_module']));
        }

        return [$importer, $mapping];
    }
}
