<?php

use SHD\Smview\Connect\DAO\CustomFunctionDAO;
use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\Domain\idoit\DynamicValueAssignment;
use SHD\Smview\Connect\Service\AbstractConfiguration;
use SHD\Smview\Connect\Service\ConnectLogger;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Import\ValuePipeline;

class connect_api_model_custom_functions extends isys_api_model_cmdb implements isys_api_model_interface
{
    /**
     * @param $p_params
     * @return array
     * @throws isys_exception_database
     */
    public function read($p_params): array
    {
        if ($this->useAuth) {
            isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');
        }

        $dao = CustomFunctionDAO::getInstance();

        $fns = DaoUtil::allRows($dao->get_data(@$p_params['fn'] ?? null));

        if($p_params['include_builtin']) {
            foreach (\SHD\Smview\Connect\Service\Import\ValuePipeline::BUILTIN_FUNCTIONS as $fn) {
                $fns[] = [
                    'id' => null,
                    'key' => $fn,
                    'value' => null,
                ];
            }
        }

        return $fns;
    }

    public function save($params)
    {
        if ($this->useAuth) {
            isys_auth_cmdb::instance()->is_allowed_to(isys_auth::EDIT, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');
        }

        $dao = CustomFunctionDAO::getInstance();

        return [
            'status' => $dao->saveDefinitions($params['definitions']),
        ];
    }

    /**
     * @param $p_params
     * @return array
     * @throws Exception
     */
    public function calculate_value($p_params): array
    {
        /** @var ConnectLogger $logger */
        $logger = isys_application::instance()->container->get('smview_connect.logger');

        try {
            $id = $p_params['profile_id'] ?? null;

            if(!$id) {
                throw new Exception('profile id must not be null');
            }

            $profile = DaoUtil::allRows(ProfileDAO::getInstance()->get_data($id))[0] ?? null;

            if(!$profile) {
                throw new Exception('profile not found');
            }

            if (!isys_application::instance()->container->has('smview_connect.profile_'.$id.'.config')) {
                throw new Exception('config not found');
            }

            /** @var AbstractConfiguration $config */
            $config = isys_application::instance()->container->get('smview_connect.profile_'.$id.'.config');

            $sourceKey = $p_params['source_key'];

            $attributes = $config->getAttributes($id, true);
            $attribute = array_values(array_filter($attributes, function($a) use ($sourceKey) {
                return $a['key'] == $sourceKey;
            }))[0] ?? null;

            if(!$attribute) {
                return [
                    'value' => null,
                ];
            }

            $value = $attribute['first_line'];

            $pipeline = $p_params['pipeline'];

            $data = new DynamicValueAssignment();
            $data->meta1 = $pipeline;

            /** @var ValuePipeline $pipeline */
            $pipeline = isys_application::instance()->container->get('smview_connect.value_pipeline');

            return [
                'value' => $pipeline->load($data)->transform($value, []) ?? null
            ];
        } catch (Throwable $e) {
            $logger->log($e->getMessage(), 'connect_api_model_custom_functions::calculate_value', \Monolog\Logger::ERROR);

            return [
                'value' => null,
            ];
        }
    }
}
