<?php

use SHD\Smview\Connect\Service\DB\DaoUtil;

/**
 * i-doit
 *
 * Manufacturers API for SM-VIEW Connect.
 *
 * @package     i-doit
 * @subpackage  smview_connect
 * @author      Florian Weber <florian.weber@shd-online.de>
 * @copyright   2023 SHD System-Haus-Dresden GmbH
 * @license     http://www.i-doit.com/license
 */
class connect_api_model_manufacturers extends isys_api_model_cmdb implements isys_api_model_interface
{
    /**
     * @param $p_params
     * @return array
     * @throws isys_exception_database
     */
    public function read($p_params)
    {
        if ($this->useAuth) {
            isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');
        }

        /** @var isys_component_database $db */
        $db = isys_application::instance()->container->get('database');
        $dao = isys_cmdb_dao_object::instance($db);

        $result = DaoUtil::allRows($dao->retrieve(<<<SQL
select  `isys_model_manufacturer__id` `id`,
        `isys_model_manufacturer__title` `title`,
        `isys_model_manufacturer__description` `description`,
        `isys_model_manufacturer__sort` `sort`,
        `isys_model_manufacturer__const` `const`,
        `isys_model_manufacturer__status` `status`,
        `isys_model_manufacturer__property` `property`
from `isys_model_manufacturer`
where `isys_model_manufacturer__const` <> {$dao->convert_sql_text('C__MODEL_NOT_SPECIFIED')} or isys_model_manufacturer__const is null
SQL
)) ?? [];

        return array_values(array_map(function($r) {
            return [
                'id' => $r['id'],
                'value' => $r['title'],
            ];
        }, $result));
    }
}
