<?php

use SHD\Smview\Connect\DAO\ProfileAssignmentDAO;

/**
 * i-doit
 *
 * Profile Assignment API for SM-VIEW Connect.
 *
 * @package     i-doit
 * @subpackage  smview_connect
 * @author      Steve Groeber <steve.groeber@shd-online.de>
 * @copyright   2022 SHD System-Haus-Dresden GmbH
 * @license     http://www.i-doit.com/license
 */
class connect_api_model_profile_assignment extends isys_api_model_cmdb implements isys_api_model_interface
{
    /**
     * Get profile assignments.
     *
     * @param  array $p_params
     *
     * @return array
     * @throws isys_exception_database
     */
    public function read($p_params)
    {
        if ($this->useAuth) {
            isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');
        }

        $profile_id = $p_params['profile_id'] ?? null;
        return ProfileAssignmentDAO::getInstance()->get_data($profile_id)->__as_array();
    }

    /**
     * Create profile assignments.
     *
     * @param  array $p_params
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_database
     * @throws isys_exception_general
     * @throws isys_exception_dao
     */
    public function create($p_params)
    {
        if (!isset($p_params['data'])) {
            throw new isys_exception_api('Please specify object data by setting \'data\' parameter.');
        }

        if ($this->useAuth) {
            isys_auth_cmdb::instance()->is_allowed_to(isys_auth::EDIT, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');
        }

        $data = $p_params['data'];
        $required_properties = ['profile_id', 'type', 'assignments'];
        foreach ($required_properties as $prop) {
            if (! isset($data[$prop])) {
                throw new isys_exception_api("Please specify assignment data by setting '$prop' parameter.");
            }
        }

        $success = ProfileAssignmentDAO::getInstance()->createProfileAssignments(
            $data['profile_id'],
            $data['type'],
            $data['assignments']
        );

        return [
            'id'      => $data['profile_id'],
            'message' => 'Created profile assignments',
            'success' => $success,
        ];
    }

    /**
     * Update profile assignments.
     *
     * @param  array $p_params
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_database
     * @throws isys_exception_general
     * @throws isys_exception_dao
     */
    public function update($p_params)
    {
        if (!isset($p_params['data'])) {
            throw new isys_exception_api('Please specify object data by setting \'data\' parameter.');
        }

        if ($this->useAuth) {
            isys_auth_cmdb::instance()->is_allowed_to(isys_auth::EDIT, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');
        }

        $data = $p_params['data'];
        $required_properties = ['profile_id', 'type', 'assignments'];
        foreach ($required_properties as $prop) {
            if (! isset($data[$prop])) {
                throw new isys_exception_api("Please specify assignment data by setting '$prop' parameter.");
            }
        }

        $success = ProfileAssignmentDAO::getInstance()->updateProfileAssignments(
            $data['profile_id'],
            $data['type'],
            $data['assignments']
        );

        return [
            'id'      => $data['profile_id'],
            'message' => 'Updated profile assignments',
            'success' => $success,
        ];
    }

    /**
     * Delete profile assignments.
     *
     * @param  array $p_params
     * @return array
     * @throws isys_exception_api
     * @throws isys_exception_dao
     */
    public function delete($p_params)
    {
        if (!isset($p_params['profile_id'])) {
            throw new isys_exception_api('Please specify profile ID by setting \'profile_id\' parameter.');
        }

        if (!isset($p_params['type'])) {
            throw new isys_exception_api('Please specify assignment type by setting \'type\' parameter.');
        }

        $success = ProfileAssignmentDAO::getInstance()->deleteProfileAssignmentsByType($p_params['profile_id'], $p_params['type']);
        return [
            'id'      => null,
            'message' => 'Deleted profile assignments',
            'success' => $success,
        ];
    }
}
