<?php

use SHD\Smview\Connect\Service\Telekom\ContractImporter;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;

/**
 * i-doit
 *
 * Telekom Status API for SM-VIEW Connect.
 *
 * @package     i-doit
 * @subpackage  smview_connect
 * @author      Florian Weber <florian.weber@shd-online.de>
 * @copyright   2023 SHD System-Haus-Dresden GmbH
 * @license     http://www.i-doit.com/license
 */
class connect_api_model_telekom_status extends isys_api_model_cmdb implements isys_api_model_interface
{
    public function read($p_params)
    {
        return [];
    }

    /**
     * @param $p_params
     * @return array
     * @throws Exception|TransportExceptionInterface
     */
    public function status($p_params)
    {
        if ($this->useAuth) {
            isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');
        }

        try {
            /** @var ContractImporter $importer */
            $importer = isys_application::instance()->container->get('smview_connect.telekom.contract_importer');

            return [
                'online' => $importer->status($p_params['profile_id']),
            ];
        } catch (Throwable $t) {
            /** @var \SHD\Smview\Connect\Service\ConnectLogger $logger */
            $logger = isys_application::instance()->container->get('smview_connect.logger');
            $logger->log($t->getMessage(), 'Telekom Status', \Monolog\Logger::ERROR);

            return [
                'online' => false,
            ];
        }
    }
}
