<?php

use SHD\Smview\Connect\Service\Telekom\ContractImporter;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;

class connect_api_model_telekom_telephone_rates extends shd_api_model_cmdb_cachable
{
    const CACHE_KEY = 'telekom_phone_rates';

    public function read($p_params)
    {
        if ($this->useAuth) {
            isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');
        }

        $key = self::getCacheKey($p_params);

        if($this->hasCache($key) && $this->isValid($key)) {
            return $this->retreive($key);
        }

        $rates = self::retreiveData($p_params);
        $this->cache($key, $rates, (new DateTime())->modify('+1 day'));

        return $rates;
    }

    /**
     * @param array $parameters
     * @return array
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     * @throws isys_exception_database
     */
    public static function retreiveData(array $parameters): array
    {
        /** @var ContractImporter $importer */
        $importer = isys_application::instance()->container->get('smview_connect.telekom.contract_importer');

        if(!$importer->status($parameters['profile_id'])) {
            return [];
        }

        $contractsList = $importer->fetchContractsList($parameters['profile_id']);

        return array_values(array_unique(array_map(function ($contract) {
            return $contract->name;
        }, $contractsList)));
    }

    public static function getCacheKey(array $parameters): string
    {
        return self::CACHE_KEY.$parameters['profile_id'];
    }

    public function isBasedOnRemoteData(): bool
    {
        return true;
    }
}
