<?php

use SHD\Smview\Connect\Service\DB\DaoUtil;
use Webmozart\Assert\Assert;

/**
 * i-doit
 *
 * Telephone Rate API for SM-VIEW Connect.
 *
 * @package     i-doit
 * @subpackage  smview_connect
 * @author      Florian Weber <florian.weber@shd-online.de>
 * @copyright   2023 SHD System-Haus-Dresden GmbH
 * @license     http://www.i-doit.com/license
 */
class connect_api_model_telephone_rates extends isys_api_model_cmdb implements isys_api_model_interface
{
    public function read($p_params)
    {
        if ($this->useAuth) {
            isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');
        }

        /** @var isys_component_database $db */
        $db = isys_application::instance()->container->get('database');
        $dao = isys_cmdb_dao_object::instance($db);

        if(count($p_params['custom'] ?? []) != 0) {
            Assert::notEmpty($p_params['custom']['identifier'] ?? null, 'no dialog+ custom-identifier provided');

            $rates = DaoUtil::allRows($dao->retrieve(
                <<<SQL
SELECT `isys_dialog_plus_custom__id` `isys_telephone_rate__id`, `isys_dialog_plus_custom__title` `isys_telephone_rate__title`
FROM `isys_dialog_plus_custom` where `isys_dialog_plus_custom__status` = 2 and `isys_dialog_plus_custom__identifier` = {$dao->convert_sql_text($p_params['custom']['identifier'])}
SQL
            )) ?? [];
        } else {
            $rates = DaoUtil::allRows($dao->retrieve(
                <<<SQL
SELECT * FROM `isys_telephone_rate` where `isys_telephone_rate__status` = 2
SQL
            )) ?? [];
        }

        return array_values(array_map(function($r) {
            return [
                'id' => $r['isys_telephone_rate__id'],
                'value' => $r['isys_telephone_rate__title'],
            ];
        }, $rates));
    }
}
