<?php

use SHD\Smview\Connect\Service\Vsphere\Import\VmImporter;
use SHD\Smview\Connect\Service\Vsphere\VsphereConfiguration;
use Webmozart\Assert\Assert;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class connect_api_model_vsphere extends isys_api_model_cmdb implements isys_api_model_interface, shd_api_model_cmdb_runnable_interface
{
    public function read($p_params): array
    {
        return [];
    }

    /**
     * @param array $params
     * @return array
     * @throws \Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function run(array $params): array
    {
        $profileId = $params['profile_id'];
        Assert::notNull($profileId);

        /** @var VmImporter $importer */
        $importer = isys_application::instance()->container->get('smview_connect.vsphere.profile_' . $profileId . '.vm_importer');

        return [
            'data' => $importer->import(),
            'message' => 'vmware import erfolgreich',
        ];
    }

    /**
     * @param array $params
     * @return array
     * @throws \Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface
     * @throws \Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface
     * @throws isys_exception_database
     * @throws Exception
     */
    public function dryrun(array $params): array
    {
        /** @var VsphereConfiguration $config */
        $config = isys_application::instance()->container->get('smview_connect.vsphere.config');

        $profileId = $params['profile_id'];
        Assert::notNull($profileId);

        $limit = $params['limit'] ?? $config->getSetting($profileId, $config::SETTING_LIMIT) ?? 0;

        /** @var VmImporter $importer */
        $importer = isys_application::instance()->container->get('smview_connect.vsphere.profile_' . $profileId . '.vm_importer');

        $vms = $importer->fetch($limit);
        return $importer->prepareImportData($vms)->export();
    }

    public function status($p_params)
    {
        if ($this->useAuth) {
            isys_auth_cmdb::instance()->is_allowed_to(isys_auth::VIEW, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');
        }

        try {
            $profileId = $p_params['profile_id'];
            Assert::notNull($profileId);

            if (!isys_application::instance()->container->has('smview_connect.vsphere.profile_' . $profileId . '.vm_importer')) {
                return [
                    'online' => false,
                ];
            }

            /** @var VmImporter $importer */
            $importer = isys_application::instance()->container->get('smview_connect.vsphere.profile_' . $profileId . '.vm_importer');

            return [
                'online' => $importer->status($profileId),
            ];
        } catch (Throwable $t) {
            /** @var \SHD\Smview\Connect\Service\ConnectLogger $logger */
            $logger = isys_application::instance()->container->get('smview_connect.logger');
            $logger->log($t->getMessage(), 'Vsphere Status', \Monolog\Logger::ERROR);

            return [
                'online' => false,
            ];
        }
    }
}
