<?php

/**
 * i-doit
 *
 * API model for SM-VIEW Connect.
 *
 * @package     i-doit
 * @subpackage  smview_connect
 * @author      Steve Groeber <steve.groeber@shd-online.de>
 * @copyright   2022 SHD System-Haus-Dresden GmbH
 * @license     http://www.i-doit.com/license
 */
class isys_api_model_connect extends isys_api_model
{
    /**
     * @param $p_method
     * @param $p_params
     * @return $this|isys_api_model
     * @throws Exception
     */
    public function route($p_method, $p_params)
    {
        if ($p_params["debug"]) {
            ini_set('display_errors', 1);
            ini_set('display_startup_errors', 1);
            error_reporting(E_ERROR);
        } else {
            ini_set('display_errors', 0);
            ini_set('display_startup_errors', 0);
            error_reporting(0);
        }

        // Build model class
        $l_modelclass = 'connect_api_model_' . $p_method;

        //session update
        isys_application::instance()->container->get('session')->start_dbsession();

        // Call data method and format data:
        if (class_exists($l_modelclass)) {
            $l_model = new $l_modelclass();

            if($l_model instanceof shd_api_model_cmdb_runnable_interface && in_array($p_params['option'], ['run'])) {
                try {
                    $result = call_user_func(
                        array($l_model, $p_params['option']),
                        $p_params
                    );

                    if(!array_key_exists('message', $result)) {
                        throw new Exception('the result must have a "message" key');
                    }

                    if(!array_key_exists('data', $result)) {
                        throw new Exception('the result must have a "data" key');
                    }

                    $this->m_data = [
                        'success' => true,
                        'message' => $result['message'],
                        'data' => $result['data'],
                    ];
                } catch (Exception $e) {
                    $this->m_data = [
                        'success' => false,
                        'message' => $e->getMessage(),
                        'data' => null,
                    ];
                }
            } else {
                if (!isset($p_params['option']) && method_exists($l_model, 'read')) {
                    $this->m_data = $l_model->read($p_params);

                    //dynamic loading of function
                } elseif (method_exists($l_model, $p_params['option']) && is_callable(array($l_model, $p_params['option']))) {
                    $this->m_data = call_user_func(
                        array($l_model, $p_params['option']),
                        $p_params
                    );
                } else {
                    throw new Exception('Could not read from model (' . $p_method . ') and function (' . $p_params['option'] . ')');
                }
            }
        } else {
            throw new Exception('Model \'connect.' . $p_method . '\' does not exist');
        }
        return $this;
    }
}
