<?php

use SHD\Smview\Connect\Service\Cache\CacheArray;
use SHD\Smview\Connect\Service\Cache\CacheInterface;
use SHD\Smview\Connect\Service\CacheManager;

/**
 * i-doit
 *
 * SHD Cachable Api Model Base Class
 *
 * @package     i-doit
 * @subpackage  smview_connect
 * @author      Florian Weber <florian.weber@shd-online.de>
 * @copyright   2023 SHD System-Haus-Dresden GmbH
 * @license     http://www.i-doit.com/license
 */
abstract class shd_api_model_cmdb_cachable extends isys_api_model_cmdb implements isys_api_model_interface, shd_api_model_cmdb_cachable_interface
{
    /**
     * @var string
     */
    protected $apiCacheDir;

    /**
     * @var CacheManager
     */
    protected $cacheManager;

    public function __construct()
    {
        parent::__construct();

        $this->apiCacheDir = isys_module_shd_smview_connect::getParameter('smview_connect.api_cache_dir');
        $this->cacheManager = isys_application::instance()->container->get('smview_connect.cache');
    }

    /**
     * @param string $key
     * @param array $data
     * @param DateTime|null $invalidateBy
     * @return CacheArray
     */
    public function cache(string $key, array $data, ?DateTime $invalidateBy = null): CacheArray
    {
        $cache = new CacheArray($key, $this->apiCacheDir);
        $cache->set($data, $invalidateBy);

        $this->cacheManager->persist($cache);

        return $cache;
    }

    /**
     * @param string $key
     * @return array|null
     */
    public function retreive(string $key): ?array
    {
        $cache = $this->getCache($key);

        return $cache->get() ?? null;
    }

    public function getCache(string $key): ?CacheInterface
    {
        $cache = $this->cacheManager->retreive($key, $this->apiCacheDir);

        if(!$cache || !$cache->hasCache()) {
            return null;
        }

        return $cache;
    }

    /**
     * @param string $key
     * @return bool
     */
    public function isValid(string $key): bool
    {
        $cache = $this->cacheManager->retreive($key, $this->apiCacheDir);

        if(!$cache) {
            return false;
        }

        return $cache->isValid();
    }

    /**
     * @param string $key
     * @return bool
     */
    public function hasCache(string $key): bool
    {
        $cache = $this->cacheManager->retreive($key, $this->apiCacheDir);

        if(!$cache) {
            return false;
        }

        return $cache->hasCache();
    }

    public static function retreiveData(array $parameters): array
    {
        return [];
    }

    public function isBasedOnRemoteData(): bool
    {
        return false;
    }

    public function isRemoteAccessible(array $parameters): bool
    {
        return false;
    }
}
