import * as React from "react";
import {ApiProps} from "../api";

interface PropertyDescription {
    id: string,
    label: string,
    guid: string,
    name: string,
    type: string,
}

export interface ListingProps {
    api_props: ApiProps,
    image_url: string,
    types: {[key: string]: string},
    language: {[key: string]: string},
    mode: 'import'|'export',
}

interface ListingState<T> {
    items: T[],
    viewable_items: T[],
    selected_items: T[],
    selected_type: string,
    is_loading: boolean,
    sort_column: string,
    sort_asc: boolean,
    search?: string,
    message?: string,

    property_descriptions?: PropertyDescription[],
    resolve_values?: boolean,
    display_property_descriptions?: boolean,
}

export abstract class GenericListingCompose<T> extends React.Component<ListingProps, ListingState<T>> {

    protected constructor(props: ListingProps) {
        super(props);
    }

    abstract loadData()

    _updateValue(property, value, callback = null) {
        this.setState((prevState) => ({
            ...prevState,
            [property]: value
        }), () => {
            if (callback) {
                callback()
            }
        })
    }

    handleSorting(e: React.MouseEvent<HTMLButtonElement>) {
        const sort_column = e.currentTarget.value
        if (sort_column == this.state.sort_column) {
            // change direction
            this.setState((prevState) => ({
                ...prevState,
                sort_asc: ! this.state.sort_asc
            }))
        } else {
            // change sort column
            this.setState((prevState) => ({
                ...prevState,
                sort_column: sort_column,
                sort_asc: true
            }))
        }
    }

    handleTypeSelection(e: React.ChangeEvent<HTMLSelectElement>) {
        this._updateValue('selected_type', e.target.value, this.loadData)
    }

    getTypeSelector(callbackResolveLinks?: React.ChangeEventHandler<HTMLInputElement>) {
        let options: JSX.Element[] = []
        for (const t in this.props.types) {
            const title = this.props.types[t]
            options.push(<option key={t} value={t}>{title}</option>)
        }

        let resolveLinkHtml
        if (callbackResolveLinks) {
            resolveLinkHtml = <>
                <input className="input mr5" type="checkbox" onChange={callbackResolveLinks} checked={this.state.resolve_values} disabled={this.state.is_loading} />
                {this.props.language.LC__CMDB__SMVIEW_CONNECT__AENEIS_RESOLVE_LINKS}
            </>
        }

        const spinner = this.state.is_loading ? <img src={this.props.image_url + 'ajax-loading.gif'} alt="Loading..." /> : null
        return <table className="contentTable">
            <tbody>
            <tr>
                <td className="key">{this.props.language.LC__UNIVERSAL__TYPE}:</td>
                <td className="value">
                    <select className="input ml5 mr5" onChange={this.handleTypeSelection} value={this.state.selected_type} disabled={this.state.is_loading}>
                        {options}
                    </select>
                    {resolveLinkHtml}
                    {spinner}
                </td>
            </tr>
            </tbody>
        </table>
    }

    getHeaderColumn(value: string, label: string) {
        const arrow = this.state.sort_asc ? <span>&#8595;</span> : <span>&#8593;</span>
        return <th>
            <button type="button" value={value} onClick={this.handleSorting}>
                {label} {this.state.sort_column == value ? arrow : null}
            </button>
        </th>
    }
}
