
export interface ApiProps {
    json_rpc_url: string,
    api_key: string,
    session: string,
    lang: string,
}

export interface ApiMethodBaseConfig {
    section: string,
    base: string,
}

export function getApiMethodBase(bases: ApiMethodBaseConfig[], section: string) {
    return bases.filter((x, i) => {
        return x.section === section
    })[0] ?? null
}

export function hasApiMethodBase(bases: ApiMethodBaseConfig[], section: string) {
    return getApiMethodBase(bases, section) !== null
}

export function api(method, params, api_props: ApiProps) {
    params = params || {}

    const defaultParams = {
        apikey:   api_props.api_key, // api checks key-sensitive parameter
        language: api_props.lang,
    }

    return fetch(api_props.json_rpc_url, {
        method:      'POST',
        headers:     {
            'Accept':             'application/json',
            'Content-Type':       'application/json',
            'X-RPC-Auth-Session': api_props.session,
        },
        credentials: 'same-origin',
        body:        JSON.stringify({
            jsonrpc: '2.0',
            version: '2.0',
            id:      1,
            method,
            params:  {
                ...defaultParams,
                ...params,
            },
        }),
    })
    .then(res => res.json())
}