import * as React from 'react'
import {Profile, ProfileSettingKey} from "../../Profile";
import {Fragment} from "react";
import {Cron} from "../../Cron";

interface CsvProfileSettingsContainerProps {
    language: {[key: string]: string},
    image_url: string,
    is_editable: boolean,
    is_loading: boolean,
    profile: Profile,
    settings_keys: ProfileSettingKey[],
    handleSettingEdit: Function,
    handleProfileDirectOptionChange: Function,
}

interface CsvProfileSettingsContainerState {
}

export class CsvProfileSettingsContainer extends React.Component<CsvProfileSettingsContainerProps, CsvProfileSettingsContainerState> {
    constructor(props: CsvProfileSettingsContainerProps) {
        super(props);

        this.state = {
        }

        this.onHandleSettingsEdit = this.onHandleSettingsEdit.bind(this)
        this.onHandleSettingsSelect = this.onHandleSettingsSelect.bind(this)
        this.onHandleOptionSelection = this.onHandleOptionSelection.bind(this)
    }

    onHandleSettingsEdit (e: React.ChangeEvent<HTMLInputElement>) {
        let sk: ProfileSettingKey = this.props.settings_keys.find(k => k.key === e.target.name) ?? null

        if(sk === null) return;

        this.props.handleSettingEdit(this.props.profile.id, sk, (sk.typeName === 'bool') ? e.target.checked : e.target.value)
    }

    onHandleSettingsSelect (e: React.ChangeEvent<HTMLSelectElement>) {
        let sk: ProfileSettingKey = this.props.settings_keys.find(k => k.key === e.target.name) ?? null

        if(sk === null) return;

        this.props.handleSettingEdit(this.props.profile.id, sk, e.target.value)
    }

    onHandleOptionSelection(e: React.ChangeEvent<HTMLInputElement>) {
        this.props.handleProfileDirectOptionChange(this.props.profile.id, e.target.name, e.target.checked)
    }

    render() {
        const {is_editable, language, is_loading} = this.props

        return <div>
            <table id="mapper_option_table" className="contentTable m10">
                <thead key="cpsc1"></thead>
                <tbody key="cpsc2">

                <tr key="check-active">
                    <td>
                        {language.LC__CMDB__SMVIEW_CONNECT__PROFILE_ACTIVE}
                        <img src={this.props.image_url + 'information.png'} className="vam mr5" alt=""
                             title={language.LC__CMDB__SMVIEW_CONNECT__PROFILE_ACTIVE_HINT}/>
                    </td>
                    <td><input type="checkbox"
                               name="active"
                               onChange={this.onHandleOptionSelection}
                               checked={this.props.profile.active}
                               disabled={is_loading || !is_editable}/>
                    </td>
                </tr>

                {this.props.settings_keys.map((sk, i) => {
                    if (!sk.visible) {
                        return <Fragment></Fragment>
                    }

                    let setting = this.props.profile.settings.find(x => x.key == sk.key) ?? null

                    if (sk.typeName == 'text') {
                        return <tr key={sk.key}>
                            <td>{sk.label}</td>
                            <td>
                                {(is_editable)
                                    ? <input className="input"
                                             type={sk.typeName}
                                             name={sk.key}
                                             onChange={this.onHandleSettingsEdit}
                                             value={(setting === null) ? '' : setting.value}/>
                                    : <span>{(setting === null) ? '-' : setting.value}</span>
                                }
                            </td>
                        </tr>
                    }

                    if (sk.typeName == 'cron') {
                        return <Fragment key={sk.key + '_container'}>
                            <tr key={sk.key}>
                                <td>{sk.label}</td>
                                <td>
                                    {(is_editable)
                                        ? <input className="input"
                                                 type="text"
                                                 name={sk.key}
                                                 onChange={this.onHandleSettingsEdit}
                                                 value={(setting === null) ? '' : setting.value}/>
                                        : <span>{(setting === null) ? '-' : setting.value}</span>
                                    }
                                </td>
                            </tr>
                            <tr key={sk.key + '_addon'}>
                                <td></td>
                                <td>
                                    <Cron value={(setting === null) ? null : setting.value} language={this.props.language}/>
                                </td>
                            </tr>
                        </Fragment>
                    }

                    if (sk.typeName == 'password') {
                        return <tr key={sk.key}>
                            <td>{sk.label}</td>
                            <td>
                                {(is_editable)
                                    ? <input className="input"
                                             type={sk.typeName}
                                             name={sk.key}
                                             onChange={this.onHandleSettingsEdit}
                                             value={(setting === null) ? '' : setting.value}/>
                                    : <span>{(setting === null) ? '-' : '*********'}</span>
                                }
                            </td>
                        </tr>
                    }

                    if (sk.typeName == 'bool') {
                        let checked = false;

                        if (setting !== null) {
                            if (setting.value) {
                                checked = true;
                            }
                        }

                        return <tr key={sk.key}>
                            <td>{sk.label}</td>
                            <td>
                                {(is_editable)
                                    ? <input type="checkbox"
                                             name={sk.key}
                                             onChange={this.onHandleSettingsEdit}
                                             checked={(checked)}/>
                                    :
                                    <span>{(checked) ? language.LC__UNIVERSAL__YES : language.LC__UNIVERSAL__NO}</span>
                                }
                            </td>
                        </tr>
                    }

                    if (sk.typeName == 'select') {
                        let valueData: any[] = JSON.parse(sk.value)

                        return <tr key={sk.key}>
                            <td>{sk.label}</td>
                            <td>
                                {(is_editable)
                                    ? <select onChange={this.onHandleSettingsSelect}
                                              value={(setting === null) ? '' : setting.value} name={sk.key}
                                              className="input ">
                                        {(valueData.map((val, i) => {
                                            return <option key={val.key + i} value={val.key}>{val.text}</option>
                                        }))}
                                    </select>
                                    :
                                    <span>{(setting === null) ? '-' : valueData.find(x => x.key == setting.value).text}</span>
                                }
                            </td>
                        </tr>
                    }

                    return <Fragment></Fragment>
                })}

                </tbody>
            </table>
        </div>
    }
}
