import * as React from 'react'
import {CsvMapping} from "../CsvMappingEditorCompose";
import {ChangeEvent} from "react";

interface CsvMappingContainerProps {
    language: {[key: string]: string},
    image_url: string,
    mapping: CsvMapping[],
    mapping_template: CsvMapping[],
    csv_data: any[],
    handleMappingAdd: Function,
    handleMappingDelete: Function,
    handleMappingChange: Function,
}

interface CsvMappingContainerState {

}

export class CsvMappingContainer extends React.Component<CsvMappingContainerProps, CsvMappingContainerState> {
    constructor(props: CsvMappingContainerProps) {
        super(props);

        this.onAdd = this.onAdd.bind(this)
        this.onDelete = this.onDelete.bind(this)
        this.onChange = this.onChange.bind(this)
    }

    onAdd(mapEl: CsvMapping) {
        this.props.handleMappingAdd(mapEl)
    }

    onDelete(mapEl: CsvMapping) {
        this.props.handleMappingDelete(mapEl)
    }

    onChange(e: ChangeEvent<HTMLInputElement>, mapEl: CsvMapping) {
        mapEl.value = e.target.value

        this.props.handleMappingChange(mapEl)
    }

    render() {
        return <table id="mapper_assignment_table" className="listing m10">
            <thead>
            <tr>
                <th>Target Field</th>
                <th>Source Header</th>
                <th>First Line</th>
            </tr>
            </thead>
            <tbody>
            {this.props.mapping_template.map((tmpl, i) => {
                let mj = this.props.mapping.findIndex(e => tmpl.key === e.key)
                let mapEl: CsvMapping = {
                    id: null,
                    key: tmpl.key,
                    value: tmpl.value,
                    from_template: true,
                }

                if(mj !== null && mj !== -1) {
                    mapEl = this.props.mapping[mj]
                }

                let csv = (this.props.csv_data !== null && this.props.csv_data !== undefined) ? ((this.props.csv_data.length > 0) ? this.props.csv_data[0] : null) : null;

                let field = null

                if(csv) {
                    if(csv.hasOwnProperty(mapEl.key)) {
                        let f = csv[mapEl.key]

                        if(!(typeof f === 'object')) {
                            field = <span>{f}</span>
                        }
                    }
                }

                return <tr key={i}>
                    <td>{mapEl.key}</td>
                    <td>
                        <input className="input" style={{marginRight: "10px"}} value={mapEl.value} disabled={mapEl.from_template} onChange={(e) => this.onChange(e, mapEl)}/>
                        <button className="btn" style={{marginRight: "10px"}} onClick={() => this.onAdd(mapEl)} type="button" disabled={!mapEl.from_template}>
                            <img src={this.props.image_url + "add.png"} className="mr5" alt="" />
                            {this.props.language.LC__UNIVERSAL__ADD}
                        </button>
                        <button className="btn" style={{marginRight: "10px"}} onClick={() => this.onDelete(mapEl)} type="button" disabled={mapEl.from_template}>
                            <img src={this.props.image_url + "delete.png"} className="mr5" alt="" />
                            {this.props.language.LC__UNIVERSAL__DELETE}
                        </button>
                    </td>
                    <td>
                        {field}
                    </td>
                </tr>
            })}
            </tbody>
        </table>
    }
}
