import * as React from "react"
import {Profile, ProfileSettingKey} from "../../Profile";
import {AssignmentOption} from "../../mapping_editor/components/AssignmentSelect";
import {Fragment} from "react";
import {ObjectTypeAssignmentSelector} from "../../mapping_editor/components/ObjectTypeAssignmentSelector";

interface DiscoveredConfigTableProps {
    language: {[key: string]: string},
    image_url: string,
    is_editable: boolean,
    is_loading: boolean,
    spinner: object,
    active_profile: Profile,
    object_type_listing: AssignmentOption[],
    handleObjectTypeSelection: Function,
    settings_keys: ProfileSettingKey[],
    handleSettingEdit: Function,
    handleObjectTypeRemove: Function,
}

export class DiscoveredConfigTable extends React.Component<DiscoveredConfigTableProps, {}> {
    constructor(props: DiscoveredConfigTableProps) {
        super(props);

        this.onHandleObjectTypeSelection = this.onHandleObjectTypeSelection.bind(this);
        this.onHandleSettingsEdit = this.onHandleSettingsEdit.bind(this);
        this.onHandleObjectTypeRemove = this.onHandleObjectTypeRemove.bind(this)
    }

    onHandleObjectTypeSelection (object_type_id: string, key: string) {
        this.props.handleObjectTypeSelection(object_type_id, key);
    }

    onHandleSettingsEdit (e: React.ChangeEvent<HTMLInputElement>) {
        let sk: ProfileSettingKey = this.props.settings_keys.find(k => k.key === e.target.name) ?? null

        if(sk === null) return;

        this.props.handleSettingEdit(sk, (sk.typeName === 'bool') ? e.target.checked : e.target.value)
    }

    onHandleObjectTypeRemove (key: string) {
        this.props.handleObjectTypeRemove(key);
    }

    render() {
        return <table id="mapper_option_table" className="contentTable m10">
            <thead></thead>
            <tbody>

            <tr key="headline-settings">
                <th>{this.props.language.LC__CMDB__SMVIEW_CONNECT__CONNECT_SETTINGS_SETTINGS}</th>
            </tr>

            {this.props.settings_keys.map((sk, i) => {
                if(!sk.visible) {
                    return <Fragment></Fragment>
                }

                let setting = this.props.active_profile.settings.find(x => x.key == sk.key) ?? null

                if(sk.typeName == 'text') {
                    return <tr key={sk.key}>
                        <td>{sk.label}</td>
                        <td>
                            {(this.props.is_editable)
                                ? <input className="input"
                                         type={sk.typeName}
                                         name={sk.key}
                                         onChange={this.onHandleSettingsEdit}
                                         value={(setting === null) ? '' : setting.value}/>
                                : <span>{(setting === null) ? '-' : setting.value}</span>
                            }
                        </td>
                    </tr>
                }

                if(sk.typeName == 'password') {
                    return <tr key={sk.key}>
                        <td>{sk.label}</td>
                        <td>
                            {(this.props.is_editable)
                                ? <input className="input"
                                         type={sk.typeName}
                                         name={sk.key}
                                         onChange={this.onHandleSettingsEdit}
                                         value={(setting === null) ? '' : setting.value}/>
                                : <span>{(setting === null) ? '-' : '*********'}</span>
                            }
                        </td>
                    </tr>
                }

                if(sk.typeName == 'bool') {
                    let checked = false;

                    if(setting !== null) {
                        if(setting.value) {
                            checked = true;
                        }
                    }

                    return <tr key={sk.key}>
                        <td>{sk.label}</td>
                        <td>
                            {(this.props.is_editable)
                                ? <input type="checkbox"
                                         name={sk.key}
                                         onChange={this.onHandleSettingsEdit}
                                         checked={(checked)} />
                                : <span>{(checked) ? 'ja' : 'nein'}</span>
                            }
                        </td>
                    </tr>
                }

                return <Fragment></Fragment>
            })}

            <tr key="headline-types">
                <th>{this.props.language.LC__CMDB__OBJTYPE}</th>
            </tr>

            {this.props.active_profile.discovered_types.map((dt, i) => {
                let typeId = 0
                if(dt.value != null) {
                    typeId = parseInt(dt.value)
                }

                return <tr key={dt.key}>
                    <td>{dt.key}</td>
                    <td>
                        <ObjectTypeAssignmentSelector object_type_options={this.props.object_type_listing}
                                                      object_type_key={dt.key}
                                                      selectedOption={typeId > 0 ? typeId.toString() : null}
                                                      onOptionSelect={this.onHandleObjectTypeSelection}
                                                      onOptionRemove={this.onHandleObjectTypeRemove}
                                                      language={this.props.language}
                                                      image_url={this.props.image_url}
                                                      is_loading={this.props.is_loading}
                                                      is_editable={this.props.is_editable} />
                        {this.props.spinner}
                    </td>
                </tr>
            })}

            </tbody>
        </table>
    }
}
