import * as React from 'react'
import {MatchingTable} from "./components/MatchingTable";
import {AssignmentTable} from "./components/AssignmentTable";
import {ProfileContainer} from "./components/ProfileContainer";
import {api, ApiProps, ApiMethodBaseConfig, hasApiMethodBase, getApiMethodBase} from "../api";
import {ConfigTable} from "./components/ConfigTable";
import {Profile, ObjectTypeMapping, ObjectTypeMappingKey, ProfileSettingKey} from "../Profile";
import {Fragment} from "react";
import {DataMappingTableContainer} from "./components/DataMappingTableContainer";
import {MainDialogSaveButton} from "./components/MainDialogSaveButton";
import * as ReactDOM from "react-dom";
import {DataMappingOverviewTableContainer} from "./components/DataMappingOverviewTableContainer";
import {OpaqueOverlay} from "../OpaqueOverlay";

export interface SourceField {
    key: string,
    title: string,
    first_line: string,
}

export interface CategoryProperty {
    id: number, // 2
    cat_title: string, // Global
    const: string, // C__CATG__GLOBAL
    title: string, // Description
    key: string, // title
    combined_key: string, // C__CATG__GLOBAL-title,
    type: string,
    dialog_plus_identifier?: string,
}

export interface MappingItem {
    id: number,
    source_field: string,
    cat_property?: string,
    meta1: string,
}

export interface DataMappingConfigItem {
    type: string,
    label: string,
    api_base: string,
    content_base: string,
    observe_custom?: string,
}

export interface DataMappingItem {
    type: string,
    source: string,
    target: string,
}

export interface DataMappingSourceItem {
    id: number,
    type: string,
    value: string,
}

export interface MappingEditorComposeProps {
    is_editable: boolean,
    language: {[key: string]: string},
    api_props: ApiProps,
    image_url: string,
    spinner_url: string,
    source_fields?: SourceField[],
    connect_module_name: string,
    object_types: {[key: number]: string},
    profile_id_binding: string,
    object_type_mapping_keys: ObjectTypeMappingKey[],
    settings_keys: ProfileSettingKey[],
    api_method_bases: ApiMethodBaseConfig[],
    data_mapping_config: DataMappingConfigItem[],
    matching_source_fields?: SourceField[],
    dev_mode?: boolean,
}

interface MappingEditorComposeState {
    is_loading: boolean,
    matching_mapping: MappingItem[],
    assignment_mapping: MappingItem[],
    cat_compare_assignment_mapping: MappingItem[],
    cat_compare_prop_assignment_mapping: MappingItem[],
    type_assignment_mapping: MappingItem[],
    value_pipeline_assignment_mapping: MappingItem[],
    editable_assignment?: string,
    cat_compare_editable_assignment?: string,
    cat_compare_prop_editable_assignment?: string,
    cat_properties: CategoryProperty[],
    profiles: Profile[],
    active_profile: Profile,
    selected_profile_id: number,
    external_api_status: boolean,
    data_mapping: DataMappingItem[],
    data_mapping_external_data_items: DataMappingItem[],
    data_mapping_internal_data_items: DataMappingSourceItem[],
    source_fields: SourceField[],
    available_pipeline_elements: string[],
    calculatedPipelineValue?: string,
}

export class MappingEditorCompose extends React.Component<MappingEditorComposeProps, MappingEditorComposeState> {
    constructor (props: MappingEditorComposeProps) {
        super(props)

        let selected_profile_id = 0
        if (this.props.profile_id_binding) {
            const input = document.getElementById(this.props.profile_id_binding) as HTMLInputElement
            selected_profile_id = parseInt(input.value)

            if(isNaN(selected_profile_id)) {
                selected_profile_id = 0
            }
        }

        this.state = {
            is_loading: false,
            matching_mapping: [],
            assignment_mapping: [],
            cat_compare_assignment_mapping: [],
            cat_compare_prop_assignment_mapping: [],
            type_assignment_mapping: [],
            value_pipeline_assignment_mapping: [],
            cat_properties: [],
            profiles: [],
            active_profile: {
                id: null,
                title: '',
                creation_date: null,
                min_matching_field_count: 1,
                add_new_objects: false,
                add_new_references: false,
                object_type_mappings: [],
                settings: [],
                active: true,
                discovered_types: [],
            },
            selected_profile_id: selected_profile_id,
            external_api_status: false,
            data_mapping: [],
            data_mapping_external_data_items: [],
            data_mapping_internal_data_items: [],
            source_fields: this.props.source_fields != null ? this.props.source_fields : [],
            available_pipeline_elements: [],
        }

        this.loadData = this.loadData.bind(this)
        this.loadProfiles = this.loadProfiles.bind(this)
        this.loadProfile = this.loadProfile.bind(this)
        this.addNewProfile = this.addNewProfile.bind(this)
        this.copyProfile = this.copyProfile.bind(this)
        this.handleProfileSelection = this.handleProfileSelection.bind(this)
        this.deleteProfile = this.deleteProfile.bind(this)
        this.loadProperties = this.loadProperties.bind(this)
        this.handleMatchingItemAdd = this.handleMatchingItemAdd.bind(this)
        this.handleMatchingItemDelete = this.handleMatchingItemDelete.bind(this)
        this.handleEditAssignment = this.handleEditAssignment.bind(this)
        this.handleCatCompareEditAssignment = this.handleCatCompareEditAssignment.bind(this)
        this.handleCatComparePropEditAssignment = this.handleCatComparePropEditAssignment.bind(this)
        this.handleShowAssignmentSelector = this.handleShowAssignmentSelector.bind(this)
        this.handleSaveProfile = this.handleSaveProfile.bind(this)
        this.handleOptionSelection = this.handleOptionSelection.bind(this)
        this.handleObjectTypeSelection = this.handleObjectTypeSelection.bind(this)
        this.handleProfileTitleChange = this.handleProfileTitleChange.bind(this)
        this.handleMinMatchingFieldCount = this.handleMinMatchingFieldCount.bind(this)
        this.handleMatchingPropertyChange = this.handleMatchingPropertyChange.bind(this)
        this.handleSettingEdit = this.handleSettingEdit.bind(this)
        this.loadExternalApiStatus = this.loadExternalApiStatus.bind(this)
        this.handleDataMappingEdit = this.handleDataMappingEdit.bind(this)
        this.handleDataMappingRemove = this.handleDataMappingRemove.bind(this)
        this.loadDataMapping = this.loadDataMapping.bind(this)
        this.loadMappingExternalData = this.loadMappingExternalData.bind(this)
        this.loadMappingInternalData = this.loadMappingInternalData.bind(this)
        this.mountMainSaveButton = this.mountMainSaveButton.bind(this)
        this.handleShowCatCompareAssignmentSelector = this.handleShowCatCompareAssignmentSelector.bind(this)
        this.handleShowCatComparePropAssignmentSelector = this.handleShowCatComparePropAssignmentSelector.bind(this)
        this.getCustomIdentifier = this.getCustomIdentifier.bind(this)
        this.onAssignmentObjectTypeSelection = this.onAssignmentObjectTypeSelection.bind(this)
        this.onAssignmentObjectTypeRemove = this.onAssignmentObjectTypeRemove.bind(this)
        this.handleObjectTypeRemove = this.handleObjectTypeRemove.bind(this)
        this.loadProfileAttributes = this.loadProfileAttributes.bind(this)
        this.handleAddPipelineElement = this.handleAddPipelineElement.bind(this)
        this.handlePipelineElementChange = this.handlePipelineElementChange.bind(this)
        this.handlePipelineElementMove = this.handlePipelineElementMove.bind(this)
        this.handlePipelineElementDelete = this.handlePipelineElementDelete.bind(this)
        this.loadAvailableValuePipelineFunctions = this.loadAvailableValuePipelineFunctions.bind(this)
        this.calculateValuePipelineValue = this.calculateValuePipelineValue.bind(this)
    }

    componentDidMount () {
        this.setState((current) => ({
            ...current,
            is_loading: true,
        }))

        this.loadData()
        this.mountMainSaveButton()
    }

    /**
     * BEWARE OF THE GREAT HACK - AND CONTINUE WITH CAUTION
     *
     * Ich will das Verhalten des Speicherbuttons von idoit erweitern -> nämlich dass erstmal der Mapping Dialog save() ausgeführt wird
     * und dann das Formular abgesendet wird.
     * ...
     * Daher wird nun der Speicher Button im react erzeugt und durch ***zauberei*** in die navBar eingefügt
     */
    mountMainSaveButton() {
        if(!this.props.is_editable) return

        //erstelle ein neues container element als ziel für den button
        let new_node = document.createElement('span')
        new_node.setAttribute('id', 'vnd-save-btn')

        //lokalisiere die navBar
        let navbar = document.getElementById('navBar')
        //füge da erstmal das span an die erste stelle ein
        navbar.insertBefore(new_node, navbar.firstChild)

        //erzeuge den button und setze ihn in das span
        let btn = <MainDialogSaveButton language={this.props.language} image_url={this.props.image_url} saveHandler={this.handleSaveProfile}/>
        ReactDOM.render(
            btn,
            document.getElementById('vnd-save-btn')
        )
    }

    loadData() {
        this.loadDataMapping()
        this.loadAvailableValuePipelineFunctions()

        this.loadProfiles()
            .then(() => {
                this.loadProperties()
                    .finally(() => {
                        this.setState((current) => ({
                            ...current,
                            is_loading: false
                        }), () => {
                            if (this.state.selected_profile_id > 0) {
                                this.loadProfile().then(() => {
                                    this.loadExternalApiStatus()
                                    this.loadMappingExternalData()
                                    //this.loadMappingInternalData()
                                })
                            }
                        })
                    })
            })
            .catch(() => {
                this.setState((current) => ({
                    ...current,
                    is_loading: false,
                }))
            })
    }

    loadAvailableValuePipelineFunctions() {
        return api('connect.custom_functions.read', {include_builtin: true}, this.props.api_props)
            .then(result => result.result)
            .then(data => {
                let fns = (Array.isArray(data)) ? data.map(cf => cf.key) : []

                this.setState((current) => {
                    return {
                        ...current,
                        available_pipeline_elements: fns,
                    }
                })
            })
    }

    calculateValuePipelineValue(source_field: SourceField, pipeline: string) {
        api('connect.custom_functions.calculate_value', {profile_id: this.state.selected_profile_id, source_key: source_field.key, pipeline: pipeline}, this.props.api_props)
            .then(result => result.result)
            .then(data => {
                this.setState((current) => {
                    return {
                        ...current,
                        calculatedPipelineValue: data.value,
                    }
                })
            })
    }

    loadMappingExternalData() {
        return Promise.all(this.props.data_mapping_config.map((config) => {
            api(config.api_base + '.read', {profile_id: this.state.active_profile.id}, this.props.api_props)
                .then(result => result.result)
                .then(data => {
                    let items: DataMappingItem[] = this.state.data_mapping_external_data_items

                    data.map((d) => {
                        let item: DataMappingItem = {
                            type: config.type,
                            source: d,
                            target: null,
                        }

                        items.push(item)
                    })

                    this.setState((current) => ({
                        ...current,
                        data_mapping_external_data_items: items,
                    }))
                })
                .catch((reason) => {
                    console.log('mapping external data not loaded, probably no data available', reason)
                })
        }))
    }

    getCustomIdentifier(config: DataMappingConfigItem): string {
        if(config.observe_custom != null && config.observe_custom != "" && config.observe_custom != undefined) {
            let mapping = this.state.assignment_mapping.filter(m => m.source_field == config.observe_custom);

            if(mapping.length == 1) {
                let cat = this.state.cat_properties.filter(p => p.combined_key == mapping[0].cat_property)

                if(cat.length == 1) {
                    if(cat[0].dialog_plus_identifier != null) {
                        return cat[0].dialog_plus_identifier
                    }
                }
            }
        }

        return null
    }

    loadMappingInternalData() {
        return Promise.all(this.props.data_mapping_config.map((config) => {
            let params = {}

            let custem_ident = this.getCustomIdentifier(config)
            if(custem_ident != null) {
                params = {
                    ...params,
                    custom: {
                        identifier: custem_ident,
                    }
                }
            }

            api(config.content_base + '.read', params, this.props.api_props)
                .then(result => result.result)
                .then(data => {
                    let items = this.state.data_mapping_internal_data_items

                    data.map((d) => {
                        let item: DataMappingSourceItem = {
                            id: d.id,
                            type: config.type,
                            value: d.value,
                        }

                        items.push(item)
                    })

                    this.setState((current) => {
                        return {
                            ...current,
                            data_mapping_internal_data_items: items,
                        }
                    })
                })
        }))
    }

    loadExternalApiStatus() {
        if(hasApiMethodBase(this.props.api_method_bases, 'status')) {
            return api(getApiMethodBase(this.props.api_method_bases, 'status').base + '.status', {profile_id: this.state.active_profile.id}, this.props.api_props)
                .then(result => result.result)
                .then(status => {
                    this.setState((current) => ({
                        ...current,
                        external_api_status: status.online
                    }))
                })
                .catch(() => {
                    this.setState((current) => ({
                        ...current,
                        external_api_status: false
                    }))
                })
        }

        return null
    }

    loadProfileAttributes() {
        if(hasApiMethodBase(this.props.api_method_bases, 'attributes')) {
            return api(getApiMethodBase(this.props.api_method_bases, 'attributes').base + '.attributes', {id: this.state.active_profile.id}, this.props.api_props)
                .then(result => result.result)
                .then(attributes => {
                    if(attributes !== undefined) {
                        let fields: SourceField[] = []

                        attributes.map((attr) => {
                            let field: SourceField = {
                                key: attr.key,
                                title: attr.title,
                                first_line: attr.first_line,
                            }
                            fields.push(field)
                        })

                        this.setState((current) => ({
                            ...current,
                            source_fields: fields,
                        }))
                    }
                })
        }
    }

    loadProfiles() {
        return api('connect.profile.read', {connect_module: this.props.connect_module_name}, this.props.api_props)
            .then(result => result.result)
            .then(profiles => {
                if (profiles.length == 0) {
                    this.addNewProfile()
                    return
                }

                this.setState((current) => {
                    return {
                        ...current,
                        profiles: profiles.map(p => this._mapProfile(p)),
                    }
                })
            })
    }

    loadProfile() {
        this.setState((current) => {
            return {
                ...current,
                is_loading: true,
            }
        })
        const id = this.state.selected_profile_id

        // write profile id back to DOM
        const input = document.getElementById(this.props.profile_id_binding) as HTMLInputElement
        input.value = id.toString();

        return api('connect.profile.read', {id: id}, this.props.api_props)
            .then(result => result.result)
            .then(profiles => {
                const profile = profiles[0]

                this.setState((current) => {
                    return {
                        ...current,
                        active_profile: this._mapProfile(profile),
                        selected_profile_id: profile.id
                    }
                })

                api('connect.profile_fields.read', {profile_id: id}, this.props.api_props)
                    .then(result => result.result)
                    .then(source_fields => {
                        let _fields: SourceField[] = source_fields.map(sf => {
                            let field: SourceField = {
                                key: sf.key,
                                title: sf.title,
                                first_line: sf.first_line,
                            }

                            return field
                        })

                        if(_fields.length > 0) {
                            this.setState((current) => {
                                return {
                                    ...current,
                                    source_fields: _fields,
                                }
                            })
                        }
                    }).then(() => {
                        api('connect.profile_assignment.read', {profile_id: id}, this.props.api_props)
                            .then(result => result.result)
                            .then(profile_assignments => {
                                let matching_mapping = []
                                let assignment_mapping = []
                                let cat_compare_assignment_mapping = []
                                let cat_compare_prop_assignment_mapping = []
                                let type_assignment_mapping = []
                                let value_pipeline_assignment_mapping = []

                                for (const pa of profile_assignments) {
                                    let profile_assignment: MappingItem = {
                                        id: pa.id,
                                        source_field: pa.source_field,
                                        cat_property: null,
                                        meta1: pa.meta1,
                                    }

                                    if(pa.category_const !== null && pa.property_id !== null) {
                                        profile_assignment.cat_property = (pa.category_const != null && pa.category_const !== "") ? pa.category_const + '-' + pa.property_key : pa.property_id
                                    }

                                    if (pa.type == 'matching') {
                                        matching_mapping.push(profile_assignment)
                                    } else if(pa.type == 'cat_compare') {
                                        cat_compare_assignment_mapping.push(profile_assignment)
                                    } else if(pa.type == 'cat_compare_prop') {
                                        cat_compare_prop_assignment_mapping.push(profile_assignment)
                                    } else if(pa.type == 'object_type') {
                                        type_assignment_mapping.push(profile_assignment)
                                    } else if(pa.type == 'value_pipeline') {
                                        value_pipeline_assignment_mapping.push(profile_assignment)
                                    } else {
                                        assignment_mapping.push(profile_assignment)
                                    }
                                }

                                this.setState((current) => {
                                    return {
                                        ...current,
                                        matching_mapping: matching_mapping,
                                        assignment_mapping: assignment_mapping,
                                        cat_compare_assignment_mapping: cat_compare_assignment_mapping,
                                        cat_compare_prop_assignment_mapping: cat_compare_prop_assignment_mapping,
                                        type_assignment_mapping: type_assignment_mapping,
                                        value_pipeline_assignment_mapping: value_pipeline_assignment_mapping,
                                    }
                                })
                            })
                            .catch(e => {
                                // todo: handle error
                            })
                            .finally(() => {
                                this.loadMappingInternalData()

                                this.setState((current) => {
                                    return {
                                        ...current,
                                        is_loading: false,
                                    }
                                })
                            })
                    })
                    .then(() => {
                        this.loadProfileAttributes()
                    })
            })
            .catch(() => {
                this.setState((current) => {
                    return {
                        ...current,
                        is_loading: false,
                    }
                })
            })
    }

    handleProfileSelection(selected_profile_id: number) {
        this.setState((current) => {
            return {
                ...current,
                selected_profile_id: selected_profile_id,
            }
        }, this.loadData)
    }

    deleteProfile() {
        return api('connect.profile.delete', {id: this.state.selected_profile_id}, this.props.api_props)
            .then(result => result.result)
            .then(() => {
                if (this.state.profiles.length == 0) {
                    this.addNewProfile()
                    return
                }
                const active_profile = this.state.profiles[0]

                this.setState((current) => {
                    return {
                        ...current,
                        active_profile: active_profile,
                    }
                })
            })
    }

    addNewProfile (title: string = '') {
        const active_profile: Profile = {
            id: -1,
            creation_date: null,
            title: title,
            min_matching_field_count: 1,
            add_new_objects: false,
            add_new_references: false,
            object_type_mappings: [],
            settings: [],
            active: true,
            discovered_types: [],
        }

        let profiles = this.state.profiles;
        profiles.push(active_profile)

        this.setState((current) => {
            return {
                ...current,
                active_profile: active_profile,
                selected_profile_id: active_profile.id,
                profiles: profiles,
            }
        })
    }

    copyProfile() {
        return api('connect.profile.copy', {id: this.state.selected_profile_id}, this.props.api_props)
            .then(result => result.result)
            .then(() => {
                this.loadData()
            })
    }

    _mapProfile (profile) {
        const date = new Date(profile.creation_date * 1000)
        profile.id = parseInt(profile.id)
        profile.add_new_objects = profile.add_new_objects == '1'
        profile.add_new_references = profile.add_new_references == '1'
        profile.creation_date = date.toString()
        profile.min_matching_field_count = profile.min_matching_field_count ? parseInt(profile.min_matching_field_count) : 0
        profile.active = profile.active == '1'

        return profile
    }

    loadProperties () {
        return api('shd.properties.read', { withoutDialog: 1 }, this.props.api_props)
            .then(result => result.result)
            .then(props => {
                let mod_props = props.map(p => {
                    p.combined_key = p.const + '-' + p.key
                    return p
                })

                this.setState((current) => {
                    return {
                        ...current,
                        cat_properties: mod_props,
                    }
                })
            })
    }

    handleMatchingItemAdd (meta1?: string) {
        let matching_mapping = this.state.matching_mapping
        const id = matching_mapping.length + 1
        // select first item of each list
        const first_source_field = this.state.source_fields[0]
        const first_cat_property = this.state.cat_properties[0]
        matching_mapping.push({
            id: id,
            source_field: first_source_field.key,
            cat_property: first_cat_property.combined_key,
            meta1: (meta1) ? meta1 : null,
        })

        this.setState((current) => {
            return {
                ...current,
                matching_mapping: matching_mapping,
            }
        })
    }

    handleMinMatchingFieldCount (count: number) {
        let active_profile = this.state.active_profile
        active_profile.min_matching_field_count = count

        this.setState((current) => {
            return {
                ...current,
                active_profile: active_profile,
            }
        })
    }

    handleMatchingPropertyChange (matching_id: number, property: string, value: string, meta1?: string) {
        let matches = []

        if(meta1 !== null) {
            matches = this.state.matching_mapping.filter(m => (m.id == matching_id && m.meta1 == meta1))
        } else {
            matches = this.state.matching_mapping.filter(m => (m.id == matching_id))
        }

        if (matches.length == 0) {
            return
        }

        const matching = matches[0]
        matching[property] = value

        if(meta1 !== null) {
            matching.meta1 = meta1
        }

        const matching_mapping = this.state.matching_mapping.filter(m => (m.id != matching_id))
        matching_mapping.push(matching)

        this.setState((current) => {
            return {
                ...current,
                matching_mapping: matching_mapping,
            }
        })
    }

    handleMatchingItemDelete (id: number) {
        const matching_mapping = this.state.matching_mapping.filter(m => (m.id != id))

        this.setState((current) => {
            return {
                ...current,
                matching_mapping: matching_mapping,
            }
        })
    }

    handleShowAssignmentSelector (source_field: string) {
        this.setState((current) => {
            return {
                ...current,
                editable_assignment: source_field,
            }
        })
    }

    handleShowCatCompareAssignmentSelector (source_field: string) {
        this.setState((current) => {
            return {
                ...current,
                cat_compare_editable_assignment: source_field,
            }
        })
    }

    handleShowCatComparePropAssignmentSelector (source_field: string) {
        this.setState((current) => {
            return {
                ...current,
                cat_compare_prop_editable_assignment: source_field,
            }
        })
    }

    handleEditAssignment (source_field: string, cat_property?: string) {
        const assignments = this.state.assignment_mapping.filter(m => (m.source_field == source_field))
        let assignment: MappingItem
        if (assignments.length == 0) {
            // create a new assignment
            assignment = {
                id: null,
                source_field: source_field,
                cat_property: cat_property,
                meta1: null,
            }
        } else {
            // update existing assignment
            assignment = assignments[0]
            assignment.cat_property = cat_property
        }

        const assignment_mapping = this.state.assignment_mapping.filter(m => (m.source_field != source_field))
        if (cat_property) {
            // add item only if a category-property is assigned
            assignment_mapping.push(assignment)
        }

        this.setState(() => ({assignment_mapping}), () => {
            // hide assignment selector
            this.handleShowAssignmentSelector(null)
        })
    }

    handleCatCompareEditAssignment (source_field: string, cat_property?: string) {
        const assignments = this.state.cat_compare_assignment_mapping.filter(m => (m.source_field == source_field))
        let assignment: MappingItem
        if (assignments.length == 0) {
            // create a new assignment
            assignment = {
                id: null,
                source_field: source_field,
                cat_property: cat_property,
                meta1: null,
            }
        } else {
            // update existing assignment
            assignment = assignments[0]
            assignment.cat_property = cat_property
        }

        const cat_compare_assignment_mapping = this.state.cat_compare_assignment_mapping.filter(m => (m.source_field != source_field))
        if (cat_property) {
            // add item only if a category-property is assigned
            cat_compare_assignment_mapping.push(assignment)
        }

        this.setState(() => ({cat_compare_assignment_mapping}), () => {
            // hide assignment selector
            this.handleShowCatCompareAssignmentSelector(null)
        })
    }

    handleCatComparePropEditAssignment (source_field: string, cat_property?: string) {
        const assignments = this.state.cat_compare_prop_assignment_mapping.filter(m => (m.source_field == source_field))
        let assignment: MappingItem
        if (assignments.length == 0) {
            // create a new assignment
            assignment = {
                id: null,
                source_field: source_field,
                cat_property: cat_property,
                meta1: null,
            }
        } else {
            // update existing assignment
            assignment = assignments[0]
            assignment.cat_property = cat_property
        }

        const cat_compare_prop_assignment_mapping = this.state.cat_compare_prop_assignment_mapping.filter(m => (m.source_field != source_field))
        if (cat_property) {
            // add item only if a category-property is assigned
            cat_compare_prop_assignment_mapping.push(assignment)
        }

        this.setState(() => ({cat_compare_prop_assignment_mapping}), () => {
            // hide assignment selector
            this.handleShowCatComparePropAssignmentSelector(null)
        })
    }

    handleOptionSelection (e: React.ChangeEvent<HTMLInputElement>) {
        let active_profile = this.state.active_profile
        active_profile[e.target.name] = e.target.checked
        this.setState({active_profile})
    }

    handleObjectTypeSelection (object_type_id: string, object_type_key: string) {
        let active_profile = this.state.active_profile

        let mappings = active_profile.object_type_mappings;
        if(mappings.some(e => e.property_key === object_type_key)) {
            let i = mappings.findIndex(e => e.property_key === object_type_key);
            let mapping = mappings[i];
            mapping.object_type_id = parseInt(object_type_id)
            mappings[i] = mapping
        } else {
            let mapping: ObjectTypeMapping = {
                property_key: object_type_key,
                object_type_id: parseInt(object_type_id),
            }

            mappings.push(mapping);
        }

        active_profile.object_type_mappings = mappings;

        this.setState({active_profile})
    }

    handleObjectTypeRemove(object_type_key: string) {
        let active_profile = this.state.active_profile
        let mappings = active_profile.object_type_mappings

        let i = mappings.findIndex(m => m.property_key == object_type_key)

        if(i === -1) {
            return
        }

        mappings.splice(i, 1)
        active_profile.object_type_mappings = mappings;

        this.setState({active_profile})
    }

    handleProfileTitleChange (e: React.ChangeEvent<HTMLInputElement>) {
        let active_profile = this.state.active_profile
        active_profile.title = e.target.value
        this.setState({active_profile})
    }

    handleSaveProfile () {
        let profile = this.state.active_profile
        const data = {
            connect_module: this.props.connect_module_name,
            title: profile.title,
            add_objects: profile.add_new_objects,
            add_references: profile.add_new_references,
            min_matching_field_count: profile.min_matching_field_count,
            object_type_mappings: profile.object_type_mappings,
            settings: profile.settings,
            active: profile.active,
            discovered_types: profile.discovered_types,
        }

        let action = 'create'
        if (profile.id && parseInt(profile.id.toString()) > 0) {
            action = 'update'
            data['id'] = profile.id
        }

        const profile_params = {
            data: data
        }

        this.setState((current) => {
            return {
                ...current,
                is_loading: true,
            }
        })

        // save profile
        return api('connect.profile.' + action, profile_params, this.props.api_props)
            .then(result => result.result)
            .then((result) => {
                let active_profile = this.state.active_profile
                active_profile.id = result.id

                this.setState((current) => {
                    return {
                        ...current,
                        active_profile: active_profile,
                    }
                })

                // load profiles again
                this.loadProfiles().then()

                // save matching mapping
                const matching_params = {
                    data: {
                        profile_id: active_profile.id,
                        type: 'matching',
                        assignments: this.state.matching_mapping
                    }
                }
                api('connect.profile_assignment.' + action, matching_params, this.props.api_props)
                    .then(result => result.result)
                    .then(() => {
                        // save assignment mapping
                        const assignment_params = {
                            data: {
                                profile_id: active_profile.id,
                                type: 'assignment',
                                assignments: this.state.assignment_mapping,
                            }
                        }
                        api('connect.profile_assignment.' + action, assignment_params, this.props.api_props)
                    })
                    .then(() => {
                        // save assignment mapping
                        const cat_compare_assignment_params = {
                            data: {
                                profile_id: active_profile.id,
                                type: 'cat_compare',
                                assignments: this.state.cat_compare_assignment_mapping,
                            }
                        }
                        api('connect.profile_assignment.' + action, cat_compare_assignment_params, this.props.api_props)
                    })
                    .then(() => {
                        // save assignment mapping
                        const cat_compare_prop_assignment_params = {
                            data: {
                                profile_id: active_profile.id,
                                type: 'cat_compare_prop',
                                assignments: this.state.cat_compare_prop_assignment_mapping,
                            }
                        }
                        api('connect.profile_assignment.' + action, cat_compare_prop_assignment_params, this.props.api_props)
                    })
                    .then(() => {
                        // save assignment mapping
                        const type_assignment_params = {
                            data: {
                                profile_id: active_profile.id,
                                type: 'object_type',
                                assignments: this.state.type_assignment_mapping,
                            }
                        }
                        api('connect.profile_assignment.' + action, type_assignment_params, this.props.api_props)
                    })
                    .then(() => {
                        // save assignment mapping
                        const value_pipeline_assignment_params = {
                            data: {
                                profile_id: active_profile.id,
                                type: 'value_pipeline',
                                assignments: this.state.value_pipeline_assignment_mapping,
                            }
                        }
                        api('connect.profile_assignment.' + action, value_pipeline_assignment_params, this.props.api_props)
                    })
                    .then(() => {
                        this.saveDataMapping()
                            .finally(() => {
                                this.setState((current) => {
                                    return {
                                        ...current,
                                        selected_profile_id: active_profile.id,
                                    }
                                })
                            })
                    })
                    .then(() => {
                        this.loadMappingInternalData()
                    })
                    .catch(() => {
                        this.setState((current) => {
                            return {
                                ...current,
                                is_loading: false,
                            }
                        })
                    })
            })
            .catch(() => {
                this.setState((current) => {
                    return {
                        ...current,
                        is_loading: false,
                    }
                })
            })
            .finally(() => {
                this.setState((current) => {
                    return {
                        ...current,
                        is_loading: false,
                    }
                })
            })
    }

    saveDataMapping() {
        let data = {
            data_mapping: this.state.data_mapping,
        }

        return api('connect.data_mapping.update', {data: data}, this.props.api_props)
            .then(result => result.result)
            .then((data) => {
                console.log(data)
            })
    }

    handleSettingEdit(key: ProfileSettingKey, value: string) {
        let profile = this.state.active_profile
        let index = profile.settings.findIndex(x => x.key == key.key);

        index === -1 ? profile.settings.push({key: key.key, value: value}) : profile.settings[index].value = value

        this.setState((current) => ({
            ...current,
            active_profile: profile,
        }))
    }

    handleDataMappingEdit(type: string, source: string, target: string) {
        let mappings = this.state.data_mapping

        if(mappings.some(e => e.type === type && e.source === source)) {
            let index = mappings.findIndex(e => e.type === type && e.source === source)
            let mapping = mappings[index]
            mapping.target = target
            mappings[index] = mapping
        } else {
            let mapping: DataMappingItem = {
                type: type,
                source: source,
                target: target,
            }
            mappings.push(mapping)
        }

        this.setState((current) => {
            return {
                ...current,
                data_mapping: mappings,
            }
        })
    }

    handleDataMappingRemove(type: string, source: string) {
        let mappings = this.state.data_mapping

        if(mappings.some(e => e.type === type && e.source === source)) {
            let index = mappings.findIndex(e => e.type === type && e.source === source)
            mappings.splice(index, 1)
        }

        this.setState((current) => {
            return {
                ...current,
                data_mapping: mappings,
            }
        })
    }

    loadDataMapping() {

        return api('connect.data_mapping.read', {}, this.props.api_props)
            .then(result => result.result)
            .then((data) => {
                let m: DataMappingItem[] = []//this.state.data_mapping

                data.map((mapping) => {
                    let item: DataMappingItem = {
                        type: mapping.type,
                        source: mapping.source,
                        target: mapping.target,
                    }

                    m.push(item)
                })

                this.setState((current) => ({
                    ...current,
                    data_mapping: m,
                    data_mapping_external_data_items: [],
                    data_mapping_internal_data_items: [],
                }))
            })
    }

    onAssignmentObjectTypeSelection(object_type_id: string, object_type_key: string) {
        const assignments = this.state.type_assignment_mapping.filter(m => m.source_field == object_type_key)
        let assignment: MappingItem

        if (assignments.length == 0) {
            // create a new assignment
            assignment = {
                id: null,
                source_field: object_type_key,
                cat_property: object_type_id,
                meta1: null,
            }
        } else {
            // update existing assignment
            assignment = assignments[0]
            assignment.cat_property = object_type_id
        }

        const type_assignment_mapping = this.state.type_assignment_mapping.filter(m => (m.source_field != object_type_key))
        if(object_type_key) {
            type_assignment_mapping.push(assignment)
        }

        this.setState(() => ({type_assignment_mapping}), () => {
        })
    }

    onAssignmentObjectTypeRemove(object_type_key: string) {
        let type_assignment_mapping = this.state.type_assignment_mapping
        let i = type_assignment_mapping.findIndex(m => m.source_field == object_type_key)

        if(i === -1) {
            return
        }

        type_assignment_mapping.splice(i, 1)

        this.setState(() => ({type_assignment_mapping}))
    }

    handleAddPipelineElement(sf: SourceField) {
        let mapping = this.state.value_pipeline_assignment_mapping
        let m = mapping.filter((el) => { return el.source_field == sf.key })
        let item = m.length > 0 ? m[0] : null

        if(item) { //there already is a mapping
            if(item.meta1 == null) {
                item.meta1 += '#tbd#'
            } else {
                item.meta1 += '|#tbd#'
            }

            let index = mapping.findIndex((_mapItem) => { return _mapItem.source_field == item.source_field })
            mapping[index] = item
        } else { //new mapping
            item = {
                source_field: sf.key,
                id: null,
                cat_property: null,
                meta1: '#tbd#',
            }
            mapping.push(item)
        }

        this.setState((current) => {
            return {
                ...current,
                value_pipeline_assignment_mapping: mapping,
            }
        })
    }

    handlePipelineElementChange(sf: SourceField, elementIndex: number, fn: string) {
        let mapping = this.state.value_pipeline_assignment_mapping
        let m = mapping.filter((el) => { return el.source_field == sf.key })
        let item = m.length > 0 ? m[0] : null
        let pipeline_elements = item.meta1.split('|')

        pipeline_elements[elementIndex] = fn

        item.meta1 = pipeline_elements.join('|')

        let index = mapping.findIndex((_mapItem) => { return _mapItem.source_field == item.source_field })
        mapping[index] = item

        this.setState((current) => {
            return {
                ...current,
                value_pipeline_assignment_mapping: mapping,
            }
        })
    }

    handlePipelineElementMove(sf: SourceField, elementIndex: number, direction: number) {
        let mapping = this.state.value_pipeline_assignment_mapping
        let m = mapping.filter((el) => { return el.source_field == sf.key })
        let item = m.length > 0 ? m[0] : null
        let pipeline_elements = item.meta1.split('|')

        if((elementIndex + direction) < 0) {
            console.log('move not possible, index < 0 with direction ' + direction)
            return
        }

        if((elementIndex + direction) > (pipeline_elements.length - 1)) {
            console.log('move not possible, index > array length with direction ' + direction)
            return
        }

        //move items
        let array_move = function (arr, old_index, new_index) { //https://stackoverflow.com/questions/5306680/move-an-array-element-from-one-array-position-to-another
            if (new_index >= arr.length) {
                let k = new_index - arr.length + 1;
                while (k--) {
                    arr.push(undefined);
                }
            }
            arr.splice(new_index, 0, arr.splice(old_index, 1)[0]);
        }
        array_move(pipeline_elements, elementIndex, elementIndex + direction)

        item.meta1 = pipeline_elements.join('|')

        let index = mapping.findIndex((_mapItem) => { return _mapItem.source_field == item.source_field })
        mapping[index] = item

        this.setState((current) => {
            return {
                ...current,
                value_pipeline_assignment_mapping: mapping,
            }
        })
    }

    handlePipelineElementDelete(sf: SourceField, elementIndex: number) {
        let mapping = this.state.value_pipeline_assignment_mapping
        let m = mapping.filter((el) => { return el.source_field == sf.key })
        let item = m.length > 0 ? m[0] : null
        let pipeline_elements = item.meta1.split('|')

        //delete item
        pipeline_elements.splice(elementIndex, 1)

        item.meta1 = pipeline_elements.join('|')

        let index = mapping.findIndex((_mapItem) => { return _mapItem.source_field == item.source_field })
        mapping[index] = item

        this.setState((current) => {
            return {
                ...current,
                value_pipeline_assignment_mapping: mapping,
            }
        })
    }

    render () {
        let object_type_listing = []
        for (const id in this.props.object_types) {
            object_type_listing.push({
                id: id,
                title: this.props.object_types[id]
            })
        }

        const spinner = this.state.is_loading ? <img src={this.props.spinner_url} alt="..." /> : null

        let statusAvailable = hasApiMethodBase(this.props.api_method_bases, 'status')
        let externalApiStatus = null
        let externalApiStatusColor = null;
        if (statusAvailable) {
            const status = this.state.external_api_status ? this.props.language.LC__CMDB__SMVIEW_CONNECT__API_ONLINE : this.props.language.LC__CMDB__SMVIEW_CONNECT__API_NOT_REACHABLE
            externalApiStatus = <span style={{float: "right"}}>{this.props.language.LC__CMDB__SMVIEW_CONNECT__API_EXTERNAL_STATUS}: {status}</span>

            if(this.state.external_api_status == false && this.state.is_loading) {
                externalApiStatusColor = '#00c4ff'
            }

            if(this.state.external_api_status == false && !this.state.is_loading) {
                externalApiStatusColor = '#ff8400'
            }
        }

        const overlay = this.state.is_loading ? <OpaqueOverlay /> : null

        return <div id="mapping_editor">

            {overlay}

            <h3 className="gradient p5 text-shadow border-top border-bottom">
                {this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_TITLE}
                <ProfileContainer language={this.props.language}
                                  image_url={this.props.image_url}
                                  is_editable={this.props.is_editable}
                                  is_loading={this.state.is_loading}
                                  profiles={this.state.profiles}
                                  selected_profile_id={this.state.selected_profile_id}
                                  onProfileSelectionChange={this.handleProfileSelection}
                                  onProfileChange={this.loadData}
                                  onProfileDelete={this.deleteProfile}
                                  onProfileSave={this.handleSaveProfile}
                                  onProfileCreate={this.addNewProfile}
                                  onProfileCopy={this.copyProfile}/>
            </h3>

            <fieldset className="overview">
                <legend style={(externalApiStatusColor != null) ? {backgroundColor: externalApiStatusColor} : {}}>
                    <span>
                        {this.props.language.LC__CMDB__SMVIEW_CONNECT__CONNECT_SETTINGS_SETTINGS}
                        {externalApiStatus}
                        {spinner}
                    </span>
                </legend>
                <ConfigTable language={this.props.language}
                             image_url={this.props.image_url}
                             is_editable={this.props.is_editable}
                             is_loading={this.state.is_loading}
                             active_profile={this.state.active_profile}
                             spinner={spinner}
                             object_type_listing={object_type_listing}
                             handleObjectTypeSelection={this.handleObjectTypeSelection}
                             handleOptionSelection={this.handleOptionSelection}
                             object_type_mapping_keys={this.props.object_type_mapping_keys}
                             settings_keys={this.props.settings_keys}
                             handleSettingEdit={this.handleSettingEdit}
                             handleObjectTypeRemove={this.handleObjectTypeRemove}/>
            </fieldset>

            {this.props.object_type_mapping_keys.map((mappingKey, i) => {
                let selected_object_type: ObjectTypeMapping = this.state.active_profile.object_type_mappings.find(e => e.property_key === mappingKey.property_key) ?? null;
                let count_keys = this.props.object_type_mapping_keys.length

                if(selected_object_type == null) {
                    return <Fragment></Fragment>
                } else {
                    return <fieldset className="overview" key={mappingKey.property_key}>
                        <legend>
                        <span>
                            {this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_MATCHING}
                            <span style={{marginLeft: '10px', marginRight: '10px'}}>&middot;</span>
                            {(mappingKey.label == null) ? mappingKey.property_key : mappingKey.label}
                        </span>
                        </legend>
                        <MatchingTable language={this.props.language}
                                       image_url={this.props.image_url}
                                       is_editable={this.props.is_editable}
                                       is_loading={this.state.is_loading}
                                       mapping={(count_keys == 1) ? this.state.matching_mapping : this.state.matching_mapping.filter(mm => mm.meta1 == mappingKey.property_key)}
                                       max_field_count={2}
                                       min_matching_field_count={this.state.active_profile.min_matching_field_count}
                                       source_fields={(this.props.matching_source_fields) ? this.props.matching_source_fields : this.state.source_fields}
                                       cat_properties={this.state.cat_properties}
                                       onMatchingItemAdd={this.handleMatchingItemAdd}
                                       onMatchingItemDelete={this.handleMatchingItemDelete}
                                       onSourceFieldChange={this.handleMatchingPropertyChange}
                                       onCatPropertyChange={this.handleMatchingPropertyChange}
                                       onMinMatchingFieldCountChange={this.handleMinMatchingFieldCount}
                                       objectTypePropertyKey={mappingKey.property_key} />
                    </fieldset>
                }
            })}

            <fieldset className="overview">
                <legend>
                    <span>{this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_ASSIGNMENT}</span>
                </legend>
                <AssignmentTable dev_mode={this.props.dev_mode}
                                 language={this.props.language}
                                 image_url={this.props.image_url}
                                 is_editable={this.props.is_editable}
                                 is_loading={this.state.is_loading}
                                 mapping={this.state.assignment_mapping}
                                 cat_compare_mapping={this.state.cat_compare_assignment_mapping}
                                 source_fields={this.state.source_fields}
                                 cat_properties={this.state.cat_properties}
                                 editable_assignment={this.state.editable_assignment}
                                 cat_compare_editable_assignment={this.state.cat_compare_editable_assignment}
                                 onShowAssignmentSelector={this.handleShowAssignmentSelector}
                                 onShowCatCompareAssignmentSelector={this.handleShowCatCompareAssignmentSelector}
                                 onEditAssignment={this.handleEditAssignment}
                                 onEditCatCompareAssignment={this.handleCatCompareEditAssignment}
                                 onShowCatComparePropAssignmentSelector={this.handleShowCatComparePropAssignmentSelector}
                                 onEditCatComparePropAssignment={this.handleCatComparePropEditAssignment}
                                 cat_compare_prop_mapping={this.state.cat_compare_prop_assignment_mapping}
                                 cat_compare_prop_editable_assignment={this.state.cat_compare_prop_editable_assignment}
                                 //object_type_listing={object_type_listing.filter(ot => this.state.active_profile.object_type_mappings.map(m => m.object_type_id).indexOf(ot.id) !== -1)}
                                 object_type_listing={object_type_listing}
                                 has_multi_object_types={this.props.object_type_mapping_keys.length > 1}
                                 type_assignment_mapping={this.state.type_assignment_mapping}
                                 handleObjectTypeSelection={this.onAssignmentObjectTypeSelection}
                                 handleObjectTypeRemove={this.onAssignmentObjectTypeRemove}
                                 handleAddPipelineElement={this.handleAddPipelineElement}
                                 value_pipeline_assignment_mapping={this.state.value_pipeline_assignment_mapping}
                                 handlePipelineElementChange={this.handlePipelineElementChange}
                                 handlePipelineElementMove={this.handlePipelineElementMove}
                                 handlePipelineElementDelete={this.handlePipelineElementDelete}
                                 available_pipeline_elements={this.state.available_pipeline_elements}
                                 handleCalculatePipelineValue={this.calculateValuePipelineValue}
                                 calculatedPipelineValue={this.state.calculatedPipelineValue} />
            </fieldset>

            <fieldset className="overview">
                <legend>
                    <span>{this.props.language.LC__CMDB__SMVIEW_CONNECT__DATA_MAPPING}</span>
                </legend>
                {spinner}

                <DataMappingTableContainer data_mapping_config={this.props.data_mapping_config}
                                  data_mapping={this.state.data_mapping}
                                  data_items={this.state.data_mapping_external_data_items}
                                  source_data_items={this.state.data_mapping_internal_data_items}
                                  is_loading={this.state.is_loading}
                                  active_profile={this.state.active_profile}
                                  is_editable={this.props.is_editable}
                                  language={this.props.language}
                                  image_url={this.props.image_url}
                                  onEditMapping={this.handleDataMappingEdit}
                                  getCustomIdentifier={this.getCustomIdentifier} />

                <DataMappingOverviewTableContainer items={this.state.data_mapping}
                                                   is_editable={this.props.is_editable}
                                                   language={this.props.language}
                                                   image_url={this.props.image_url}
                                                   is_loading={this.state.is_loading}
                                                   onAddMapping={this.handleDataMappingEdit}
                                                   onDeleteMapping={this.handleDataMappingRemove} />

            </fieldset>

            <div>
                <span>{this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_SAVE_PROFILE}</span><br />
                <input className="input input-small" type="text" value={this.state.active_profile.title} onChange={this.handleProfileTitleChange} />
                <button className="btn"
                        type="button"
                        onClick={this.handleSaveProfile}
                        disabled={this.state.is_loading || ! this.props.is_editable}>{this.props.language.LC__UNIVERSAL__BUTTON_SAVE}</button>
            </div>

        </div>
    }
}
