import * as React from 'react'
import {AssignmentSelect} from '../components/AssignmentSelect'
const ReactTestRenderer = require('react-test-renderer');

const options = [
    {
        id: 'C__CATG__GLOBAL-title',
        title: 'Title',
    },
    {
        id: 'C__CATG__GLOBAL-description',
        title: 'Description',
    },
    {
        id: 'C__CATG__MODEL-manufacturer',
        title: 'Manufacturer',
    },
    {
        id: 'C__CATG__GLOBAL-serial_number',
        title: 'Serial Number',
    }
]

it('renders read-only assignment selector correctly', () => {
    const component = ReactTestRenderer.create(
        <AssignmentSelect is_loading={true}
                          options={options}
                          selectedOption={'C__CATG__GLOBAL-description'}
                          onOptionSelect={null} />,
    );
    expect(component).toMatchSnapshot();
});

it('renders assignment selector correctly', () => {
    const optionSelectMockFunction = jest.fn();
    const component = ReactTestRenderer.create(
        <AssignmentSelect is_loading={false}
                          options={options}
                          selectedOption={'C__CATG__GLOBAL-description'}
                          onOptionSelect={optionSelectMockFunction} />,
    );
    expect(component).toMatchSnapshot();

    const instance = component.root;
    let spans = instance.findAllByType('span');
    let span = spans[0]
    span.props.onClick()

    let input = instance.findByType('input');
    input.props.onChange({
        target: { value: 'nu' }
    });

    let lis = instance.findAllByType('li');
    let li = lis[0]
    li.props.onClick()
    expect(optionSelectMockFunction).toHaveBeenCalledWith('C__CATG__MODEL-manufacturer');
});

it('renders assignment selector with reset button correctly', () => {
    const optionSelectMockFunction = jest.fn();
    const component = ReactTestRenderer.create(
        <AssignmentSelect is_loading={false}
                          options={options}
                          selectedOption={'C__CATG__GLOBAL-description'}
                          onOptionSelect={optionSelectMockFunction}
                          showResetButton={true} />,
    );
    expect(component).toMatchSnapshot();

    const instance = component.root;
    let button = instance.findByType('button');
    button.props.onClick()
    expect(optionSelectMockFunction).toHaveBeenCalledWith(null);
});
