import * as React from 'react'
import {ConfigTable} from "../components/ConfigTable";
import {Profile, ProfileSettingKey} from "../../Profile";
import {AssignmentOption} from "../components/AssignmentSelect";
const ReactTestRenderer = require('react-test-renderer');

let lang = {}

let profile: Profile = {
    id: 1,
    title: 'Profile 01',
    creation_date: null,
    min_matching_field_count: 1,
    add_new_objects: false,
    add_new_references: false,
    object_type_mappings: [],
    settings: [],
    active: true,
    discovered_types: [],
}

let object_types: AssignmentOption[] = [
    {id: '1', title: 'Type 1'},
    {id: '2', title: 'Type 2'},
]

let mapping_keys= [
    {property_key: 'foo', object_type_id: 1},
    {property_key: 'bar', object_type_id: 2}
]

let settings_keys: ProfileSettingKey[] = [
    {key: 'key1', label: 'label', typeName: 'text', visible: true} as ProfileSettingKey,
    {key: 'key2', label: 'label', typeName: 'password', visible: false} as ProfileSettingKey,
]

it('renders read-only config table correctly', () => {
    const component = ReactTestRenderer.create(
        <ConfigTable language={lang}
                     image_url={'img/icons/'}
                     is_editable={false}
                     is_loading={false}
                     spinner={null}
                     active_profile={profile}
                     object_type_listing={object_types}
                     handleObjectTypeSelection={null}
                     handleOptionSelection={null}
                     object_type_mapping_keys={mapping_keys}
                     settings_keys={settings_keys}
                     handleSettingEdit={null}
                     handleObjectTypeRemove={null}/>
    )
    expect(component).toMatchSnapshot()
})

it('renders config table correctly', () => {
    const handleObjectTypeSelectionMockFunction = jest.fn();
    const handleOptionSelectionMockFunction = jest.fn();
    const handleSettingEditMockFunction = jest.fn();
    const handleOptionRemoveMockFunction = jest.fn();

    const component = ReactTestRenderer.create(
        <ConfigTable language={lang}
                     image_url={'img/icons/'}
                     is_editable={true}
                     is_loading={false}
                     spinner={null}
                     active_profile={profile}
                     object_type_listing={object_types}
                     handleObjectTypeSelection={handleObjectTypeSelectionMockFunction}
                     handleOptionSelection={handleOptionSelectionMockFunction}
                     object_type_mapping_keys={mapping_keys}
                     settings_keys={settings_keys}
                     handleSettingEdit={handleSettingEditMockFunction}
                     handleObjectTypeRemove={handleOptionRemoveMockFunction} />
    )
    expect(component).toMatchSnapshot()
})
