import * as React from 'react'
import {ProfileContainer} from '../components/ProfileContainer'
const ReactTestRenderer = require('react-test-renderer');

const lang = {
    ['LC__CMDB__SMVIEW_CONNECT__PROFILES']: 'Profiles',
    ['LC__UNIVERSAL__LOAD']: 'Load',
    ['LC_UNIVERSAL__DELETE']: 'Delete',
};

const profiles = [
    {
        id: 1,
        title: 'Profile 01',
        creation_date: null,
        min_matching_field_count: 1,
        add_new_objects: false,
        add_new_references: false,
        object_type_mappings: [],
        settings: [],
        active: true,
        discovered_types: [],
    }
]

it('renders read-only profile container correctly', () => {
    const component = ReactTestRenderer.create(
        <ProfileContainer language={lang}
                          is_editable={false}
                          is_loading={false}
                          profiles={profiles}
                          image_url={'img/icons/'}
                          selected_profile_id={null}
                          onProfileSelectionChange={null}
                          onProfileChange={null}
                          onProfileDelete={null}
                          onProfileSave={null}
                          onProfileCreate={null}
                          onProfileCopy={null} />,
    );
    expect(component).toMatchSnapshot();
});

it('renders profile container correctly', () => {
    const profileSelectionChangeMockFunction = jest.fn();
    const profileChangeMockFunction = jest.fn();
    const profileDeleteMockFunction = jest.fn();
    const profileSaveMockFunction = jest.fn();
    const profileCreateMockFunction = jest.fn();
    const profileCopyMockFunction = jest.fn();

    const component = ReactTestRenderer.create(
        <ProfileContainer language={lang}
                          is_editable={true}
                          is_loading={false}
                          profiles={profiles}
                          image_url={'img/icons/'}
                          selected_profile_id={null}
                          onProfileSelectionChange={profileSelectionChangeMockFunction}
                          onProfileChange={profileChangeMockFunction}
                          onProfileDelete={profileDeleteMockFunction}
                          onProfileSave={profileSaveMockFunction}
                          onProfileCreate={profileCreateMockFunction}
                          onProfileCopy={profileCopyMockFunction}/>,
    );

    const instance = component.root;
    let select = instance.findByProps({name: "profile_box"});
    select.props.onChange({
        currentTarget: { value: '4' }
    });
    expect(profileSelectionChangeMockFunction).toHaveBeenCalledWith(4);

    // let buttons = instance.findAllByProps({className: "btn btn-small"});
    // let button = buttons[0]
    // button.props.onClick();
    // expect(profileCreateMockFunction).toHaveBeenCalled();
    //
    // button = buttons[1]
    // button.props.onClick();
    // expect(profileChangeMockFunction).toHaveBeenCalled();
    //
    // button = buttons[2]
    // button.props.onClick();
    // expect(profileDeleteMockFunction).toHaveBeenCalled();
    //
    // button = buttons[3]
    // button.props.onClick();
    // expect(profileSaveMockFunction).toHaveBeenCalled();

    expect(component).toMatchSnapshot();
});

it('renders disabled profile container without profiles correctly', () => {
    const component = ReactTestRenderer.create(
        <ProfileContainer language={lang}
                          is_editable={true}
                          is_loading={false}
                          profiles={[]}
                          image_url={'img/icons/'}
                          selected_profile_id={null}
                          onProfileSelectionChange={null}
                          onProfileChange={null}
                          onProfileDelete={null}
                          onProfileSave={null}
                          onProfileCreate={null}
                          onProfileCopy={null} />,
    );
    expect(component).toMatchSnapshot();
});
