import * as React from 'react'
import {CategoryProperty, MappingItem, SourceField} from "../MappingEditorCompose";
import {AssignmentOption, AssignmentSelect} from "./AssignmentSelect";
import {ObjectTypeAssignmentSelector} from "./ObjectTypeAssignmentSelector";
import {Fragment} from "react";

interface AssignmentTableProps {
    language: {[key: string]: string},
    image_url: string,
    is_editable: boolean,
    is_loading: boolean,
    mapping: MappingItem[],
    cat_compare_mapping: MappingItem[],
    cat_compare_prop_mapping: MappingItem[],
    source_fields: SourceField[],
    cat_properties: CategoryProperty[],
    editable_assignment?: string,
    cat_compare_editable_assignment?: string,
    cat_compare_prop_editable_assignment?: string,
    type_assignment?: string,
    onShowAssignmentSelector: Function,
    onShowCatCompareAssignmentSelector: Function,
    onShowCatComparePropAssignmentSelector: Function,
    onEditAssignment: Function,
    onEditCatCompareAssignment: Function,
    onEditCatComparePropAssignment: Function,
    type_assignment_mapping: MappingItem[],
    object_type_listing: AssignmentOption[],
    handleObjectTypeSelection: Function,
    handleObjectTypeRemove: Function,
    has_multi_object_types: boolean,
    handleAddPipelineElement: Function,
    value_pipeline_assignment_mapping: MappingItem[],
    handlePipelineElementChange: Function,
    handlePipelineElementMove: Function,
    handlePipelineElementDelete: Function,
    available_pipeline_elements: string[],
    handleCalculatePipelineValue: Function,
    calculatedPipelineValue: string,
    dev_mode: boolean,
}

export class AssignmentTable extends React.Component<AssignmentTableProps, {}> {
    constructor (props: AssignmentTableProps) {
        super(props)
    }

    render () {
        let trs: JSX.Element[] = []
        for (const source_field of this.props.source_fields) {
            const mapItems = this.props.mapping.filter(m => m.source_field == source_field.key)
            const mapItem = mapItems.length > 0 ? mapItems[0] : null

            const cat_compare_mapItems = this.props.cat_compare_mapping.filter(m => m.source_field == source_field.key)
            const cat_compare_mapItem = cat_compare_mapItems.length > 0 ? cat_compare_mapItems[0] : null

            const cat_compare_prop_mapItems = this.props.cat_compare_prop_mapping.filter(m => m.source_field == source_field.key)
            const cat_compare_prop_mapItem = cat_compare_prop_mapItems.length > 0 ? cat_compare_prop_mapItems[0] : null

            const type_mapItems = this.props.type_assignment_mapping.filter(m => m.source_field == source_field.key)
            const type_mapItem = type_mapItems.length > 0 ? type_mapItems[0] : null

            const value_pipeline_assignment__mapItems = this.props.value_pipeline_assignment_mapping.filter((m) => { return m.source_field == source_field.key })
            const value_pipeline_assignment__mapItem = value_pipeline_assignment__mapItems.length > 0 ? value_pipeline_assignment__mapItems[0] : null

            trs.push(<AssignmentTableRow key={'source_field_' + source_field.key}
                                         dev_mode={this.props.dev_mode}
                                         language={this.props.language}
                                         image_url={this.props.image_url}
                                         is_editable={this.props.is_editable}
                                         is_loading={this.props.is_loading}
                                         source_field={source_field}
                                         mapItem={mapItem}
                                         cat_compare_mapItem={cat_compare_mapItem}
                                         editable_assignment={this.props.editable_assignment}
                                         cat_compare_editable_assignment={this.props.cat_compare_editable_assignment}
                                         cat_properties={this.props.cat_properties}
                                         onShowAssignmentSelector={this.props.onShowAssignmentSelector}
                                         onShowCatCompareAssignmentSelector={this.props.onShowCatCompareAssignmentSelector}
                                         onEditAssignment={this.props.onEditAssignment}
                                         onEditCatCompareAssignment={this.props.onEditCatCompareAssignment}
                                         cat_compare_prop_mapItem={cat_compare_prop_mapItem}
                                         onEditCatComparePropAssignment={this.props.onEditCatComparePropAssignment}
                                         onShowCatComparePropAssignmentSelector={this.props.onShowCatComparePropAssignmentSelector}
                                         cat_compare_prop_editable_assignment={this.props.cat_compare_prop_editable_assignment}
                                         object_type_listing={this.props.object_type_listing}
                                         type_mapItem={type_mapItem}
                                         has_multi_object_types={this.props.has_multi_object_types}
                                         handleObjectTypeSelection={this.props.handleObjectTypeSelection}
                                         handleObjectTypeRemove={this.props.handleObjectTypeRemove}
                                         available_pipeline_elements={this.props.available_pipeline_elements}
                                         value_pipeline_assignment_mapItem={value_pipeline_assignment__mapItem}
                                         row_color={""}
                                         handleAddPipelineElement={this.props.handleAddPipelineElement}
                                         handlePipelineElementChange={this.props.handlePipelineElementChange}
                                         handlePipelineElementDelete={this.props.handlePipelineElementDelete}
                                         handlePipelineElementMove={this.props.handlePipelineElementMove}
                                         calculatedPipelineValue={this.props.calculatedPipelineValue}
                                         handleCalculatePipelineValue={this.props.handleCalculatePipelineValue} />)
        }

        return <table id="mapper_assignment_table" className="listing m10">
            <thead>
                <tr>
                    <th>{this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_COLUMN_NAME}</th>
                    <th>{this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_FIRST_LINE}</th>
                    <th>{this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_ASSIGNMENT}</th>
                    <th>{this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_CAT_COMPARE_ASSIGNMENT}</th>
                </tr>
            </thead>
            <tbody>
                {trs}
            </tbody>
        </table>
    }
}

interface AssignmentTableRowProps {
    language: {[key: string]: string},
    image_url: string,
    is_editable: boolean,
    is_loading: boolean,
    source_field: SourceField,
    mapItem: MappingItem,
    cat_properties: CategoryProperty[],
    editable_assignment?: string,
    cat_compare_editable_assignment?: string,
    cat_compare_prop_editable_assignment?: string,
    onShowAssignmentSelector: Function,
    onShowCatCompareAssignmentSelector: Function,
    onShowCatComparePropAssignmentSelector: Function,
    onEditAssignment: Function,
    onEditCatCompareAssignment: Function,
    cat_compare_mapItem: MappingItem,
    onEditCatComparePropAssignment: Function,
    cat_compare_prop_mapItem: MappingItem,
    object_type_listing: AssignmentOption[],
    type_mapItem: MappingItem,
    handleObjectTypeSelection: Function,
    handleObjectTypeRemove: Function,
    has_multi_object_types: boolean,
    available_pipeline_elements: string[],
    value_pipeline_assignment_mapItem: MappingItem,
    row_color: string,
    handleAddPipelineElement: Function,
    handlePipelineElementChange: Function,
    handlePipelineElementMove: Function,
    handlePipelineElementDelete: Function,
    handleCalculatePipelineValue: Function,
    calculatedPipelineValue: string,
    dev_mode: boolean,
}

interface AssignmentTableRowState {
    piplines_visible: boolean,
}

export class AssignmentTableRow extends React.Component<AssignmentTableRowProps, AssignmentTableRowState> {
    constructor(props: AssignmentTableRowProps) {
        super(props)

        this.state = {
            piplines_visible: false,
        }

        this.handleEditAssignment = this.handleEditAssignment.bind(this)
        this.handleEditCatCompareAssignment = this.handleEditCatCompareAssignment.bind(this)
        this.handleEditCatComparePropAssignment = this.handleEditCatComparePropAssignment.bind(this)
        this.handleObjectTypeSelection = this.handleObjectTypeSelection.bind(this)
        this.handleObjectTypeRemove = this.handleObjectTypeRemove.bind(this)
        this.onAddPipelineElement = this.onAddPipelineElement.bind(this)
        this.onChangePipelineElement = this.onChangePipelineElement.bind(this)
        this.onMovePipelineElement = this.onMovePipelineElement.bind(this)
        this.onDeletePipelineElement = this.onDeletePipelineElement.bind(this)
    }

    handleEditAssignment (cat_property: string) {
        this.props.onEditAssignment(this.props.source_field.key, cat_property)
    }

    handleEditCatCompareAssignment (cat_property: string) {
        this.props.onEditCatCompareAssignment(this.props.source_field.key, cat_property)
    }

    handleEditCatComparePropAssignment (cat_property: string) {
        this.props.onEditCatComparePropAssignment(this.props.source_field.key, cat_property)
    }

    handleObjectTypeSelection (object_type_id: string, key: string) {
        this.props.handleObjectTypeSelection(object_type_id, key)
    }

    handleObjectTypeRemove (key: string) {
        this.props.handleObjectTypeRemove(key)
    }

    onAddPipelineElement (sf: SourceField) {
        this.props.handleAddPipelineElement(sf)
    }

    onChangePipelineElement (sf: SourceField, elementIndex: number, fn: string) {
        this.props.handlePipelineElementChange(sf, elementIndex, fn)
    }

    onMovePipelineElement (sf: SourceField, elementIndex: number, direction: number) {
        this.props.handlePipelineElementMove(sf, elementIndex, direction)
    }

    onDeletePipelineElement (sf: SourceField, elementIndex: number) {
        this.props.handlePipelineElementDelete(sf, elementIndex)
    }

    render () {
        /**
         * main assignment selector
         * the 2 other assignments are only necessary if the selected target category is multi value
         */
        const source_field = this.props.source_field
        let value = this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_NO_ASSIGNMENT
        let valueCssClass = "input-size-small mr5 "
        if (this.props.mapItem) {
            const cat_property = this.props.cat_properties.filter(c => c.combined_key == this.props.mapItem.cat_property)[0]
            value = cat_property ? cat_property.cat_title + ' -> ' + cat_property.title : value
        } else {
            valueCssClass += "text-grey"
        }

        const cat_property_options = this.props.cat_properties.map(p => {return {
            id: p.combined_key,
            title: p.cat_title + ' > ' + p.title + (p.const.startsWith('C__CATG__CUSTOM_FIELDS') ? ' [custom]' : '')
        }})

        let assignment_selector
        const displaySelector = source_field.key == this.props.editable_assignment
        if (displaySelector) {
            assignment_selector = <AssignmentSelector language={this.props.language}
                                                      image_url={this.props.image_url}
                                                      is_loading={this.props.is_loading}
                                                      is_editable={this.props.is_editable}
                                                      cat_property_options={cat_property_options}
                                                      onEditAssignment={this.handleEditAssignment}
                                                      onHideAssignmentSelector={() => {this.props.onShowAssignmentSelector(null)}} />
        }

        /**
         * cat compare
         * assignment selector code for the category comparison value whis is used for the comparison of mutli value cat data on the right side
         */
        let cat_compare_value = this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_NO_ASSIGNMENT
        let cat_compare_valueCssClass = "input-size-small mr5 "
        if (this.props.cat_compare_mapItem) {
            const cat_compare_property = this.props.cat_properties.filter(c => c.combined_key == this.props.cat_compare_mapItem.cat_property)[0]
            cat_compare_value = cat_compare_property ? cat_compare_property.cat_title + ' -> ' + cat_compare_property.title : value
        } else {
            cat_compare_valueCssClass += "text-grey"
        }

        let cat_compare_assignment_selector
        const cat_compare_displaySelector = source_field.key == this.props.cat_compare_editable_assignment
        if (cat_compare_displaySelector) {
            cat_compare_assignment_selector = <AssignmentSelector language={this.props.language}
                                                                  image_url={this.props.image_url}
                                                                  is_loading={this.props.is_loading}
                                                                  is_editable={this.props.is_editable}
                                                                  cat_property_options={cat_property_options}
                                                                  onEditAssignment={this.handleEditCatCompareAssignment}
                                                                  onHideAssignmentSelector={() => {this.props.onShowCatCompareAssignmentSelector(null)}} />
        }

        /**
         * cat compare prop
         * assignment selector code for the category prop whis is used for the comparison of mutli value cat data on the left side
         */
        let cat_compare_prop_value = this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_NO_ASSIGNMENT
        let cat_compare_prop_valueCssClass = "input-size-small mr5 "
        if (this.props.cat_compare_prop_mapItem) {
            const cat_compare_prop_property = this.props.cat_properties.filter(c => c.combined_key == this.props.cat_compare_prop_mapItem.cat_property)[0]
            cat_compare_prop_value = cat_compare_prop_property ? cat_compare_prop_property.cat_title + ' -> ' + cat_compare_prop_property.title : value
        } else {
            cat_compare_prop_valueCssClass += "text-grey"
        }

        const cat_compare_prop_cat_property_options = this.props.cat_properties
            .filter(p => p.const == ((this.props.mapItem != null) ? this.props.mapItem.cat_property.split('-')[0] : null))
            .map(p => {return {
                id: p.combined_key,
                title: p.cat_title + ' > ' + p.title + (p.const.startsWith('C__CATG__CUSTOM_FIELDS') ? ' [custom]' : '')
            }})

        let cat_compare_prop_assignment_selector
        const cat_compare_prop_displaySelector = source_field.key == this.props.cat_compare_prop_editable_assignment
        if (cat_compare_prop_displaySelector) {
            cat_compare_prop_assignment_selector = <AssignmentSelector language={this.props.language}
                                                                  image_url={this.props.image_url}
                                                                  is_loading={this.props.is_loading}
                                                                  is_editable={this.props.is_editable}
                                                                  cat_property_options={cat_compare_prop_cat_property_options}
                                                                  onEditAssignment={this.handleEditCatComparePropAssignment}
                                                                  onHideAssignmentSelector={() => {this.props.onShowCatComparePropAssignmentSelector(null)}} />
        }

        const disabled = displaySelector || this.props.is_loading || ! this.props.is_editable

        const has_pipeline_elements = (this.props.value_pipeline_assignment_mapItem && this.props.value_pipeline_assignment_mapItem.meta1 != null && this.props.value_pipeline_assignment_mapItem.meta1 != '')
        let pipeline_enabled = true

        //todo: wenn feature complete dann rausnehmen
        if(this.props.dev_mode != true) {
            pipeline_enabled = false
        }

        return <Fragment>
            <tr key={source_field.key} id={source_field.key}>
                <td>
                    {source_field.title}

                    {(this.props.has_multi_object_types) ?
                        <Fragment>
                            <br />
                            <ObjectTypeAssignmentSelector object_type_options={this.props.object_type_listing}
                                                          object_type_key={source_field.title}
                                                          selectedOption={this.props.type_mapItem ? this.props.type_mapItem.cat_property : null}
                                                          onOptionSelect={this.handleObjectTypeSelection}
                                                          onOptionRemove={this.handleObjectTypeRemove}
                                                          language={this.props.language}
                                                          image_url={this.props.image_url}
                                                          is_loading={this.props.is_loading}
                                                          is_editable={this.props.is_editable} />
                        </Fragment> : <Fragment></Fragment>
                    }
                </td>
                <td>
                    {source_field.first_line}

                    {(this.props.is_editable && pipeline_enabled) ? <button type="button" className="btn btn-small ml5 float-right" onClick={() => {this.setState((current) => {return {...current, piplines_visible: !this.state.piplines_visible}})}}>
                        <img src={this.props.image_url + 'cog_edit.png'}
                             alt={this.props.language.LC__UNIVERSAL__EDIT}
                             title={this.props.language.LC__CMDB__SMVIEW_CONNECT__TOGGLE_PIPELINE_EDIT}/>
                    </button> : null}
                </td>
                <td>
                    <span className={valueCssClass}>{value}</span>
                    <button type="button" className="btn mr5" onClick={() => {this.props.onShowAssignmentSelector(this.props.source_field.key)}} disabled={disabled}>
                        <img src={this.props.image_url + "pencil.png"}
                             alt={this.props.language.LC__UNIVERSAL__EDIT}
                             title={this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_EDIT_ASSIGNMENT} />
                    </button>
                    <button type="button" className="btn" onClick={() => {this.props.onEditAssignment(this.props.source_field.key, null)}} disabled={disabled}>
                        <img src={this.props.image_url + "detach.png"}
                             alt={this.props.language.LC__UNIVERSAL__DETACH}
                             title={this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_DETACH_ASSIGNMENT} />
                    </button>
                    {assignment_selector}
                </td>
                <td>
                    <span className={cat_compare_valueCssClass}>{cat_compare_value}</span>
                    <button type="button" className="btn mr5" onClick={() => {this.props.onShowCatCompareAssignmentSelector(this.props.source_field.key)}} disabled={disabled}>
                        <img src={this.props.image_url + "pencil.png"}
                             alt={this.props.language.LC__UNIVERSAL__EDIT}
                             title={this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_EDIT_ASSIGNMENT} />
                    </button>
                    <button type="button" className="btn" onClick={() => {this.props.onEditCatCompareAssignment(this.props.source_field.key, null)}} disabled={disabled}>
                        <img src={this.props.image_url + "detach.png"}
                             alt={this.props.language.LC__UNIVERSAL__DETACH}
                             title={this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_DETACH_ASSIGNMENT} />
                    </button>
                    {cat_compare_assignment_selector}

                    <br/>

                    <span className={cat_compare_prop_valueCssClass}>{cat_compare_prop_value}</span>
                    <button type="button" className="btn mr5" onClick={() => {this.props.onShowCatComparePropAssignmentSelector(this.props.source_field.key)}} disabled={disabled}>
                        <img src={this.props.image_url + "pencil.png"}
                             alt={this.props.language.LC__UNIVERSAL__EDIT}
                             title={this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_EDIT_ASSIGNMENT} />
                    </button>
                    <button type="button" className="btn" onClick={() => {this.props.onEditCatComparePropAssignment(this.props.source_field.key, null)}} disabled={disabled}>
                        <img src={this.props.image_url + "detach.png"}
                             alt={this.props.language.LC__UNIVERSAL__DETACH}
                             title={this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_DETACH_ASSIGNMENT} />
                    </button>
                    {cat_compare_prop_assignment_selector}
                </td>
            </tr>
            {((this.state.piplines_visible || has_pipeline_elements) && pipeline_enabled) ? <tr key={source_field.key + '_vp'}>
                <td>
                    <strong>{this.props.language.LC__CMDB__SMVIEW_CONNECT__VALUE_PIPELINE}</strong>
                </td>
                <td>
                    {(this.props.calculatedPipelineValue) ? this.props.calculatedPipelineValue : this.props.source_field.first_line}

                    {(this.props.is_editable && has_pipeline_elements) ? <button type="button" className="btn btn-small ml5 float-right" onClick={() => {
                        this.props.handleCalculatePipelineValue(source_field, this.props.value_pipeline_assignment_mapItem.meta1)
                    }}>
                        <img src={this.props.image_url + 'bullet_sparkle.png'}
                             alt={this.props.language.LC__UNIVERSAL__EDIT}
                             title={this.props.language.LC__CMDB__SMVIEW_CONNECT__TOGGLE_PIPELINE_REFRESH}/>
                    </button> : null}
                </td>
                {(this.props.is_editable) ? <td>
                    {(has_pipeline_elements)
                        ? this.props.value_pipeline_assignment_mapItem.meta1.split('|').map((pipeline_element, elementIndex) => {
                        const list: AssignmentOption[] = this.props.available_pipeline_elements.map((el) => {
                            return {
                                id: el,
                                title: el,
                            }
                        })

                        return <Fragment key={pipeline_element + elementIndex}>
                            <div style={{padding: "5px", marginBottom: "5px"}}>
                                <strong>{pipeline_element}</strong>

                                <button type="button" className="btn btn-small float-right" onClick={() => {this.onMovePipelineElement(source_field, elementIndex, -1)}}>
                                    <img src={this.props.image_url + "arrow_up.png"}
                                         alt={this.props.language.LC__CMDB__SMVIEW_CONNECT__MOVE_PIPELINE_ELEMENT}
                                         title={this.props.language.LC__CMDB__SMVIEW_CONNECT__MOVE_PIPELINE_ELEMENT}/>
                                </button>

                                <button type="button" className="btn btn-small float-right" onClick={() => {this.onMovePipelineElement(source_field, elementIndex, 1)}}>
                                    <img src={this.props.image_url + "arrow_down.png"}
                                         alt={this.props.language.LC__CMDB__SMVIEW_CONNECT__MOVE_PIPELINE_ELEMENT}
                                         title={this.props.language.LC__CMDB__SMVIEW_CONNECT__MOVE_PIPELINE_ELEMENT}/>
                                </button>

                                <button type="button" className="btn btn-small float-right" onClick={() => {this.onDeletePipelineElement(source_field, elementIndex)}}>
                                    <img src={this.props.image_url + "delete.png"}
                                         alt={this.props.language.LC__UNIVERSAL__DELETE}
                                         title={this.props.language.LC__CMDB__SMVIEW_CONNECT__DELETE_PIPELINE_ELEMENT}/>
                                </button>
                            </div>

                            <AssignmentSelector language={this.props.language}
                                                image_url={this.props.image_url}
                                                is_loading={this.props.is_loading}
                                                is_editable={this.props.is_editable}
                                                cat_property_options={list}
                                                onEditAssignment={(fn) => {this.onChangePipelineElement(source_field, elementIndex, fn)}}
                                                onHideAssignmentSelector={() => {}}
                                                hide_abort={true}/>

                            <hr/>
                        </Fragment>
                    }) : null}

                    <button type="button" className="btn btn-small mt5" onClick={() => {
                        this.onAddPipelineElement(source_field)
                    }}>
                        <img src={this.props.image_url + "add.png"}
                             className="mr5"
                             alt={this.props.language.LC__UNIVERSAL__ADD}
                             title={this.props.language.LC__CMDB__SMVIEW_CONNECT__ADD_PIPELINE_ELEMENT}/>
                        {this.props.language.LC__CMDB__SMVIEW_CONNECT__ADD_PIPELINE_ELEMENT}
                    </button>
                </td> : <td>
                    {(has_pipeline_elements)
                        ? <strong>{this.props.value_pipeline_assignment_mapItem.meta1.split('|').map((el) => {return el + '()'}).join(' -> ')}</strong>
                        : null
                    }
                </td>}
                <td></td>
            </tr> : null}
        </Fragment>
    }
}

interface AssignmentSelectorProps {
    language: { [key: string]: string },
    image_url: string,
    is_loading: boolean,
    is_editable: boolean,
    cat_property_options: AssignmentOption[],
    onEditAssignment: Function,
    onHideAssignmentSelector: Function,
    hide_abort?: boolean,
}

interface AssignmentSelectorState {
    selected_cat_property: string,
}

class AssignmentSelector extends React.Component<AssignmentSelectorProps, AssignmentSelectorState> {
    constructor(props: AssignmentSelectorProps) {
        super(props)

        this.handleCatPropertyChange = this.handleCatPropertyChange.bind(this)
        this.handleEditAssignment = this.handleEditAssignment.bind(this)

        this.state = {
            selected_cat_property: null
        }
    }

    handleCatPropertyChange (selected_cat_property: string) {
        this.setState({selected_cat_property})
    }

    handleEditAssignment () {
        this.props.onEditAssignment(this.state.selected_cat_property)
    }

    render () {
        return <div id="mapper_assignment_selector">
            <AssignmentSelect options={this.props.cat_property_options}
                              selectedOption={this.state.selected_cat_property ?? ''}
                              onOptionSelect={this.handleCatPropertyChange}
                              showResetButton={false}
                              is_loading={this.props.is_loading} /><br />

            <button type="button" className="btn mt5 mr5" onClick={this.handleEditAssignment}>
                <img src={this.props.image_url + "tick.png"} className="mr5" alt="" /> {this.props.language.LC_UNIVERSAL__ACCEPT}
            </button>

            {(this.props.hide_abort) ? <Fragment></Fragment> :
                <button type="button" className="btn mt5" onClick={() => {
                    this.props.onHideAssignmentSelector(null)
                }}>
                    <img src={this.props.image_url + "cross.png"} className="mr5"
                         alt=""/> {this.props.language.LC_UNIVERSAL__ABORT}
                </button>
            }
        </div>
    }
}
