import * as React from 'react'
import {DataMappingItem} from "../MappingEditorCompose";
import {Fragment} from "react";

interface DataMappingOverviewTableContainerProps {
    items: DataMappingItem[],
    is_editable: boolean,
    is_loading: boolean,
    language: {[key: string]: string},
    image_url: string,
    onAddMapping: Function,
    onDeleteMapping: Function,
}

interface DataMappingOverviewTableContainerState {
    add_mode: boolean,
    new_mapping: DataMappingItem,
}

export class DataMappingOverviewTableContainer extends React.Component<DataMappingOverviewTableContainerProps, DataMappingOverviewTableContainerState> {
    constructor(props: DataMappingOverviewTableContainerProps) {
        super(props);

        this.state = {
            add_mode: false,
            new_mapping: {
                type: "",
                source: "",
                target: "",
            }
        }

        this.toggleAddMode = this.toggleAddMode.bind(this)
        this.onSave = this.onSave.bind(this)
        this.handleMappingEntryChange = this.handleMappingEntryChange.bind(this)
    }

    toggleAddMode() {
        this.setState((current) => ({
            ...current,
            add_mode: !this.state.add_mode,
        }))
    }

    onSave() {
        let mapping = this.state.new_mapping

        if(mapping.type != "" && mapping.target != "" && mapping.source != "") {
            this.props.onAddMapping(mapping.type, mapping.source, mapping.target)
        }

        this.toggleAddMode()
    }

    handleMappingEntryChange(e: React.ChangeEvent<HTMLInputElement>) {
        const target = e.target;
        const value = target.type === 'checkbox' ? target.checked : target.value;
        const name = target.name;

        let mapping = this.state.new_mapping
        mapping[name] = value

        this.setState((current) => ({
            ...current,
            new_mapping: mapping,
        }))
    }

    render() {
        const language = this.props.language
        return <Fragment>
            <table className="listing m10">
                <thead>
                <tr>
                    <th>{language.LC__CMDB__SMVIEW_CONNECT__MATCHING_ALL_MAPPINGS}</th>
                    <th>{language.LC__CMDB__SMVIEW_CONNECT__MATCHING_SOURCE}</th>
                    <th colSpan={2}>{language.LC__CMDB__SMVIEW_CONNECT__MATCHING_TARGET}</th>
                </tr>
                </thead>
                <tbody>
                {(this.props.items.map((item, i) => {
                    return <tr key={'all-mappings-' + item.type + i}>
                        <td>{item.type}</td>
                        <td>{item.source}</td>
                        <td>{item.target}</td>
                        <td>
                            <button className="btn btn-small" onClick={() => this.props.onDeleteMapping(item.type, item.source)} disabled={this.props.is_loading || !this.props.is_editable}>
                                <img src={this.props.image_url + "cross.png"}
                                     alt={language.LC__UNIVERSAL__DELETE}
                                     title={language.LC__CMDB__SMVIEW_CONNECT__MATCHING_DETACH_ASSIGNMENT} />
                            </button>
                        </td>
                    </tr>
                }))}
                {(this.state.add_mode) ?
                    <tr key="add-row">
                        <td><input value={this.state.new_mapping.type} onChange={this.handleMappingEntryChange} name="type" /></td>
                        <td><input value={this.state.new_mapping.source} onChange={this.handleMappingEntryChange} name="source" /></td>
                        <td><input value={this.state.new_mapping.target} onChange={this.handleMappingEntryChange} name="target" /></td>
                        <td></td>
                    </tr> : <Fragment></Fragment>
                }
                </tbody>
            </table>

            <div className="m10">
                <button type="button"
                        className="btn btn-small"
                        onClick={this.toggleAddMode}
                        disabled={this.props.is_loading || !this.props.is_editable}>
                    <span>
                        {(!this.state.add_mode) ?
                            <Fragment>{language.LC__UNIVERSAL__ADD} {language.LC__CMDB__SMVIEW_CONNECT__DATA_MAPPING}</Fragment>
                            : language.LC_UNIVERSAL__ABORT
                        }
                    </span>
                </button>

                {(this.state.add_mode) ?
                    <button type="button"
                            className="btn btn-small ml10"
                            onClick={this.onSave}>
                        <span>{language.LC__UNIVERSAL__BUTTON_SAVE}</span>
                    </button> : <Fragment></Fragment>
                }
            </div>
        </Fragment>
    }
}
