import * as React from 'react'
import {DataMappingConfigItem, DataMappingItem, DataMappingSourceItem} from "../MappingEditorCompose";
import {Fragment} from "react";
import {Profile} from "../../Profile";
import {AssignmentOption, AssignmentSelect} from "./AssignmentSelect";

interface DataMappingTableContainerProps {
    data_mapping_config: DataMappingConfigItem[],
    data_mapping: DataMappingItem[],
    data_items: DataMappingItem[],
    source_data_items: DataMappingSourceItem[],
    active_profile: Profile,
    is_editable: boolean,
    image_url: string,
    language: {[key: string]: string},
    onEditMapping: Function,
    is_loading: boolean,
    getCustomIdentifier: Function,
}

interface DataMappingTableProps {
    config: DataMappingConfigItem,
    is_loading: boolean,
    data_items: DataMappingItem[],
    source_data_items: DataMappingSourceItem[],
    data_mapping: DataMappingItem[],
    handle_mapping_edit: Function,
    is_editable: boolean,
    language: {[key: string]: string},
    image_url: string,
    source_data_items_custom: string,
}

interface DataMappingTableState {
    body_visible: boolean,
}

interface DataMappingRowProps {
    type: string,
    source: string,
    target: string,
    is_loading: boolean,
    is_editable: boolean,
    image_url: string,
    language: {[key: string]: string},
    options: DataMappingSourceItem[],
    handleMappingEdit: Function,
    source_data_items_custom: string,
}

interface DataMappingRowState {
    show_mapping_selector: boolean,
}

interface MappingSelectProps {
    image_url: string,
    language: {[key: string]: string},
    onToggle: Function,
    onEdit: Function,
    is_loading: boolean,
    options: DataMappingSourceItem[],
    type: string,
    source: string,
    target: string,
}

interface MappingSelectState {
    selected_data: string,
}

export class DataMappingTableContainer extends React.Component<DataMappingTableContainerProps, {}> {
    constructor(props: DataMappingTableContainerProps) {
        super(props);
    }

    render() {
        return <Fragment>
            {this.props.data_mapping_config.map((config, i) => {
                return <DataMappingTable key={config.type}
                                         is_loading={this.props.is_loading}
                                         config={config}
                                         data_items={this.props.data_items.filter((item) => {return item.type === config.type})}
                                         source_data_items={this.props.source_data_items}
                                         data_mapping={this.props.data_mapping}
                                         handle_mapping_edit={this.props.onEditMapping}
                                         is_editable={this.props.is_editable}
                                         language={this.props.language}
                                         image_url={this.props.image_url}
                                         source_data_items_custom={this.props.getCustomIdentifier(config)} />
            })}
        </Fragment>
    }
}

export class DataMappingTable extends React.Component<DataMappingTableProps, DataMappingTableState> {
    constructor(props) {
        super(props);

        this.state = {
            body_visible: false,
        }

        this.toggleBody = this.toggleBody.bind(this)
    }

    toggleBody() {
        let vis = this.state.body_visible

        this.setState((current) => ({
            ...current,
            body_visible: !vis,
        }))
    }

    render() {
        const spinner = this.props.is_loading ? <Fragment>{this.props.language.LC__CMDB__SMVIEW_CONNECT__IS_LOADING}</Fragment> : null
        let count = this.props.data_items.length

        return  <table className="listing m10">
            <thead onClick={this.toggleBody} className='mapping-editor-link'>
            <tr>
                <th>
                    {this.props.config.label}
                    <span className="mapping-editor-badge">{count}</span>
                </th>
                <th>
                    {spinner}
                    <span style={{float: 'right'}}>
                        {(this.state.body_visible) ? this.props.language.LC__CMDB__CLICK_TO_HIDE : this.props.language.LC__CMDB__CLICK_TO_REVEAL}
                    </span>
                </th>
            </tr>
            </thead>
            <tbody className={(this.state.body_visible) ? '' : 'mapping-editor-hidden'}>
            {this.props.data_items.map((item, i) => {
                return <DataMappingRow key={item.type + i}
                                       type={item.type}
                                       source={item.source}
                                       options={this.props.source_data_items.filter((d) => {return d.type === item.type})}
                                       target={(this.props.data_mapping.filter((m) => {return m.type === this.props.config.type && m.source === item.source})[0]?.target ?? null) as string}
                                       handleMappingEdit={this.props.handle_mapping_edit}
                                       is_editable={this.props.is_editable}
                                       is_loading={this.props.is_loading}
                                       language={this.props.language}
                                       image_url={this.props.image_url}
                                       source_data_items_custom={this.props.source_data_items_custom} />
            })}
            </tbody>
        </table>
    }
}

export class DataMappingRow extends React.Component<DataMappingRowProps, DataMappingRowState> {
    constructor(props) {
        super(props);

        this.state = {
            show_mapping_selector: false,
        }

        this.toggleMappingSelector = this.toggleMappingSelector.bind(this)
        this.onEdit = this.onEdit.bind(this)
    }

    toggleMappingSelector() {
        this.setState((current) => {
            return {
                ...current,
                show_mapping_selector: !this.state.show_mapping_selector,
            }
        })
    }

    onEdit(type: string, source: string, target: string) {
        this.toggleMappingSelector()
        this.props.handleMappingEdit(type, source, target)
    }

    render() {
        const disabled = this.props.is_loading || ! this.props.is_editable

        let value = this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_NO_ASSIGNMENT
        let valueCssClass = "input-size-small mr5 "

        if(this.props.target != null) {
            value = this.props.target
        } else {
            valueCssClass += "text-grey"
        }

        let mappingSelector = null

        if(this.state.show_mapping_selector) {
            mappingSelector = <MappingSelect language={this.props.language}
                                             image_url={this.props.image_url}
                                             options={this.props.options}
                                             type={this.props.type}
                                             source={this.props.source}
                                             target={this.props.target}
                                             onEdit={this.onEdit}
                                             onToggle={this.toggleMappingSelector}
                                             is_loading={this.props.is_loading} />
        }

        return <tr>
            <td>{this.props.source}</td>
            <td>
                <span className={valueCssClass}>{value}</span>

                <button type="button" className="btn mr5" onClick={() => {this.toggleMappingSelector()}} disabled={disabled}>
                    <img src={this.props.image_url + "pencil.png"}
                         alt={this.props.language.LC__UNIVERSAL__EDIT}
                         title={this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_EDIT_ASSIGNMENT} />
                </button>
                <button type="button" className="btn" onClick={() => {}} disabled={disabled}>
                    <img src={this.props.image_url + "detach.png"}
                         alt={this.props.language.LC__UNIVERSAL__DETACH}
                         title={this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_DETACH_ASSIGNMENT} />
                </button>

                {mappingSelector}

                <br />

                {(this.props.source_data_items_custom != null) ? 'Custom Dialog+ Identifier: ' + this.props.source_data_items_custom : <Fragment />}
            </td>
        </tr>
    }
}

export class MappingSelect extends React.Component<MappingSelectProps, MappingSelectState> {
    constructor(props) {
        super(props);

        this.state = {
            selected_data: null,
        }

        this.handleMappingSelect = this.handleMappingSelect.bind(this)
    }

    handleMappingSelect(selected_data: string) {
        this.setState((current) => {
            return {
                ...current,
                selected_data: selected_data,
            }
        })
    }

    render() {
        return <div>
            <AssignmentSelect options={this.props.options.map((op) => {return {id: op.value, title: op.value} as AssignmentOption})}
                              selectedOption={this.state.selected_data ?? ''}
                              onOptionSelect={this.handleMappingSelect}
                              is_loading={this.props.is_loading} />

            <button type="button" className="btn mt5 mr5" onClick={() => {this.props.onEdit(this.props.type, this.props.source, this.state.selected_data)}}>
                <img src={this.props.image_url + "tick.png"} className="mr5" alt="" /> {this.props.language.LC_UNIVERSAL__ACCEPT}
            </button>
            <button type="button" className="btn mt5" onClick={() => {this.props.onToggle()}}>
                <img src={this.props.image_url + "cross.png"} className="mr5" alt="" /> {this.props.language.LC_UNIVERSAL__ABORT}
            </button>
        </div>
    }
}
