import {AssignmentOption, AssignmentSelect} from "./AssignmentSelect";
import * as React from "react";
import {Fragment} from "react";

export interface ObjectTypeAssignmentSelectorProps {
    language: {[key: string]: string},
    image_url: string,
    is_loading: boolean,
    is_editable: boolean,
    object_type_options: AssignmentOption[],
    object_type_key: string,
    selectedOption: string,
    onOptionSelect: Function,
    onOptionRemove: Function,
}

export class ObjectTypeAssignmentSelector extends React.Component<ObjectTypeAssignmentSelectorProps, {}> {
    constructor(props: ObjectTypeAssignmentSelectorProps) {
        super(props);

        this.onHandleChange = this.onHandleChange.bind(this)
        this.onRemove = this.onRemove.bind(this)
    }

    onHandleChange(object_type_id: string) {
        this.props.onOptionSelect(object_type_id, this.props.object_type_key)
    }

    onRemove() {
        this.props.onOptionRemove(this.props.object_type_key)
    }

    render() {
        return <Fragment>
            <AssignmentSelect options={this.props.object_type_options}
                              selectedOption={this.props.selectedOption ?? ''}
                              onOptionSelect={this.onHandleChange}
                              is_loading={this.props.is_loading || ! this.props.is_editable} />
            <button type="button"
                    className="btn"
                    disabled={this.props.is_loading || ! this.props.is_editable}
                    onClick={this.onRemove}>
                <img src={this.props.image_url + "cross.png"}
                     alt={this.props.language.LC__UNIVERSAL__DELETE}
                     title={this.props.language.LC__CMDB__SMVIEW_CONNECT__MATCHING_DETACH_ASSIGNMENT} />
            </button>
        </Fragment>
    }
}
