import * as React from "react";
import {api, ApiMethodBaseConfig, ApiProps} from "../api";
import {OpaqueOverlay} from "../OpaqueOverlay";
import {FunctionList} from "./components/FunctionList";
import {ActionBar} from "./components/ActionBar";
import {MainDialogSaveButton} from "../mapping_editor/components/MainDialogSaveButton";
import * as ReactDOM from "react-dom";

export interface FunctionDefinition {
    id: number,
    key: string,
    value: string,
}

export interface ValuePipelineEditorComposeProps {
    is_editable: boolean,
    language: {[key: string]: string},
    api_props: ApiProps,
    image_url: string,
    spinner_url: string,
    api_method_bases: ApiMethodBaseConfig[],
}

export interface ValuePipelineEditorComposeState {
    is_loading: boolean,
    function_definitions: FunctionDefinition[],
}

export class ValuePipelineEditorCompose extends React.Component<ValuePipelineEditorComposeProps, ValuePipelineEditorComposeState> {
    constructor (props: ValuePipelineEditorComposeProps) {
        super(props)

        this.state = {
            is_loading: false,
            function_definitions: [],
        }

        this.loadData = this.loadData.bind(this)
        this.loadDefinitions = this.loadDefinitions.bind(this)
        this.saveDefinitions = this.saveDefinitions.bind(this)
        this.handleChangeFunctionKey = this.handleChangeFunctionKey.bind(this)
        this.handleChangeFunctionValue = this.handleChangeFunctionValue.bind(this)
        this.handleFunctionAdd = this.handleFunctionAdd.bind(this)
        this.handleFunctionRemove = this.handleFunctionRemove.bind(this)
        this.handleSave = this.handleSave.bind(this)
        this.mountMainSaveButton = this.mountMainSaveButton.bind(this)
    }

    componentDidMount() {
        this.loadData()
        this.mountMainSaveButton()
    }

    loadData() {
        this.setState((current) => ({
            ...current,
            is_loading: true,
        }))

        this.loadDefinitions()
            .then(() => {

            })
            .catch((error) => {
                console.log(error)
            })
            .finally(() => {
                this.setState((current) => ({
                    ...current,
                    is_loading: false,
                }))
            })
    }

    loadDefinitions() {
        return api('connect.custom_functions.read', {}, this.props.api_props)
            .then(result => result.result)
            .then((data) => {
                if(!Array.isArray(data)) {
                    throw new Error('custom function data is not an array')
                }

                let defs: FunctionDefinition[] = []

                data.map((_def) => {
                    let def: FunctionDefinition = {
                        id: parseInt(_def.id),
                        value: _def.value,
                        key: _def.key,
                    }

                    defs.push(def)
                })

                this.setState((current) => ({
                    ...current,
                    function_definitions: defs,
                }))
            })
    }

    saveDefinitions() {
        return api('connect.custom_functions.save', {
            definitions: this.state.function_definitions,
        }, this.props.api_props)
            .then(result => result.result)
            .then((data) => {
                console.log(data)
            })
    }

    /**
     * BEWARE OF THE GREAT HACK - AND CONTINUE WITH CAUTION
     *
     * Ich will das Verhalten des Speicherbuttons von idoit erweitern -> nämlich dass erstmal der JS save() ausgeführt wird
     * und dann das Formular abgesendet wird.
     * ...
     * Daher wird nun der Speicher Button im react erzeugt und durch ***zauberei*** in die navBar eingefügt
     */
    mountMainSaveButton() {
        if(!this.props.is_editable) return

        //erstelle ein neues container element als ziel für den button
        let new_node = document.createElement('span')
        new_node.setAttribute('id', 'vnd-save-btn')

        //lokalisiere die navBar
        let navbar = document.getElementById('navBar')
        //füge da erstmal das span an die erste stelle ein
        navbar.insertBefore(new_node, navbar.firstChild)

        //erzeuge den button und setze ihn in das span
        let btn = <MainDialogSaveButton language={this.props.language} image_url={this.props.image_url} saveHandler={this.handleSave}/>
        ReactDOM.render(
            btn,
            document.getElementById('vnd-save-btn')
        )
    }

    handleChangeFunctionKey(key: string, definition: FunctionDefinition) {
        let defs = this.state.function_definitions

        let i = defs.findIndex(e => e.id === definition.id);
        let def = defs[i];
        def.key = key
        defs[i] = def

        this.setState((current) => ({
            ...current,
            function_definitions: defs,
        }))
    }

    handleChangeFunctionValue(value: string, definition: FunctionDefinition) {
        let defs = this.state.function_definitions

        let i = defs.findIndex(e => e.id === definition.id);
        let def = defs[i];
        def.value = value
        defs[i] = def

        this.setState((current) => ({
            ...current,
            function_definitions: defs,
        }))
    }

    handleFunctionAdd() {
        let defs = this.state.function_definitions

        let new_key = 'new_function'
        let suffix = ''
        let count = 1

        while(defs.some((d) => d.key === new_key + suffix)) {
            suffix = count.toString()
            count++
        }

        let def: FunctionDefinition = {
            id: (new Date().valueOf()) * -1, //neue mit neg. id anlegen um dann im backend rauszufinden dass die neu sind
            value: null,
            key: new_key + suffix,
        }

        defs.push(def)

        this.setState((current) => ({
            ...current,
            function_definitions: defs,
        }))
    }

    handleFunctionRemove(_def: FunctionDefinition) {
        let defs = this.state.function_definitions
        let i = defs.findIndex(e => e.id === _def.id);

        defs.splice(i, 1)

        this.setState((current) => ({
            ...current,
            function_definitions: defs,
        }))
    }

    handleSave() {
        this.setState((current) => ({
            ...current,
            is_loading: true,
        }))

        return this.saveDefinitions()
            .then(() => {
                this.loadData()
            })
            .catch((error) => {
                console.log(error)
            })
            .finally(() => {
                this.setState((current) => ({
                    ...current,
                    is_loading: false,
                }))
            })
    }

    render() {
        const overlay = this.state.is_loading ? <OpaqueOverlay /> : null

        return <div id="value_pipeline_editor">
            {overlay}

            <fieldset className="overview">
                <legend>
                    {this.props.language.LC__CMDB__SMVIEW_CONNECT__VALUE_PIPELINE_FUNCTIONS}
                </legend>

                <ActionBar disabled={(this.state.is_loading || !this.props.is_editable)}
                           handleAdd={this.handleFunctionAdd}
                           handleSave={this.handleSave}
                           language={this.props.language}
                           image_url={this.props.image_url}/>

                <FunctionList definitions={this.state.function_definitions}
                              is_editable={this.props.is_editable}
                              is_loading={this.state.is_loading}
                              handleChangeKey={this.handleChangeFunctionKey}
                              handleChangeValue={this.handleChangeFunctionValue}
                              handleFunctionRemove={this.handleFunctionRemove}
                              language={this.props.language}
                              image_url={this.props.image_url}/>
            </fieldset>
        </div>
    }
}
