import * as React from "react";
import {FunctionDefinition} from "../ValuePipelineEditorCompose";
import AceEditor from "react-ace";
import "ace-builds/src-noconflict/mode-php";
import "ace-builds/src-noconflict/theme-monokai";
import "ace-builds/src-noconflict/ext-language_tools";
import {Fragment} from "react";

export interface FunctionListProps {
    definitions: FunctionDefinition[],
    is_editable: boolean,
    is_loading: boolean,
    handleChangeKey: Function,
    handleChangeValue: Function,
    handleFunctionRemove: Function,
    language: {[key: string]: string},
    image_url: string,
}

export interface FunctionListState {
}

export class FunctionList extends React.Component<FunctionListProps, FunctionListState> {
    constructor (props: FunctionListProps) {
        super(props)

        this.state = {}

        this.onChangeKey = this.onChangeKey.bind(this)
        this.onChangeValue = this.onChangeValue.bind(this)
        this.onFunctionRemove = this.onFunctionRemove.bind(this)
    }

    onChangeKey(event: React.ChangeEvent<HTMLInputElement>, def: FunctionDefinition) {
        this.props.handleChangeKey(event.target.value, def)
    }

    onChangeValue(value: string, def: FunctionDefinition) {
        this.props.handleChangeValue(value, def)
    }

    onFunctionRemove(def: FunctionDefinition) {
        this.props.handleFunctionRemove(def)
    }

    render() {
        return <table id="value_pipeline_option_table" className="contentTable m10">
            <thead></thead>
            <tbody>

            {this.props.definitions.map((def, i) => {
                return <tr key={"fd-" + i}>
                    <td>
                        {(this.props.is_editable && this.props.is_loading == false)
                            ? <input value={def.key} className="input m10" onChange={(event) => this.onChangeKey(event, def)}/>
                            : <span>{def.key}</span>
                        }

                        {(this.props.is_editable && this.props.is_loading == false)
                            ? <Fragment><br />
                                <FunctionActionButton def={def} handleClick={this.onFunctionRemove}>
                                    <img src={this.props.image_url + "delete.png"} className="mr5" alt=""/>
                                    {this.props.language.LC_UNIVERSAL__DELETE}
                                </FunctionActionButton></Fragment>
                            : <Fragment/>
                        }
                    </td>
                    <td>
                        <AceEditor height="200px"
                                   width="80%"
                                   value={def.value}
                                   mode="php"
                                   theme="monokai" fontSize="16px"
                                   highlightActiveLine={true}
                                   onChange={(value) => this.onChangeValue(value, def)}
                                   readOnly={!(this.props.is_editable && this.props.is_loading == false)}
                                   setOptions={{
                                       enableLiveAutocompletion: true,
                                       showLineNumbers: true,
                                       tabSize: 2
                                   }} />
                    </td>
                </tr>
            })}

            </tbody>
        </table>
    }
}

interface FunctionActionButtonProps {
    def: FunctionDefinition,
    handleClick: Function,
}

class FunctionActionButton extends React.Component<FunctionActionButtonProps, {}> {
    constructor (props: FunctionActionButtonProps) {
        super(props)

        this.state = {}

        this.onClick = this.onClick.bind(this)
    }

    onClick() {
        this.props.handleClick(this.props.def)
    }

    render() {
        return <button className="btn m10" type="button" onClick={this.onClick}>{this.props.children}</button>
    }
}
