const { CleanWebpackPlugin } = require('clean-webpack-plugin');
const { resolve } = require('path');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');

module.exports = {
    entry: {
        mapping_editor:[
            './src/mapping_editor/index.tsx',
            './../sass/mapping_editor.scss'
        ],
        csv_mapping_editor:[
            './src/csv_mapping_editor/index.tsx',
            './../sass/csv_mapping_editor.scss'
        ],
        csv_import_editor:[
            './src/csv_import_editor/index.tsx',
            './../sass/csv_import_editor.scss'
        ],
        baramundi_tree:[
            './src/baramundi_tree/index.tsx',
            './../sass/baramundi_tree.scss'
        ],
        discovery_editor:[
            './src/discovery_editor/index.tsx',
            './../sass/discovery_editor.scss'
        ],
        aeneis_listing:[
            './src/aeneis_listing/index.tsx',
            './../sass/aeneis_listing.scss'
        ],
        value_pipeline_editor: [
            './src/value_pipeline_editor/index.tsx',
            './../sass/value_pipeline_editor.scss'
        ]
    },
    output: {
        filename: '[name]/[name].js',
        path: resolve(__dirname, './../dist/'),
        publicPath: 'src/classes/modules/smview_importer/assets/dist/',
    },
    resolve: {
        cacheWithContext: false,
        extensions: ['.ts', '.tsx', '.js', '.json'],
        symlinks: false,
    },
    module: {
        rules: [
            {
                test: /\.tsx?$/,
                include: resolve(__dirname, 'src'),
                use: 'ts-loader',
                exclude: /node_modules/,
            },
            // sass to css file
            {
                test: /\.(scss)$/,
                include: resolve(__dirname, './../sass'),
                use: [MiniCssExtractPlugin.loader, 'css-loader', 'sass-loader'],
                sideEffects: true
            }
        ]
    },
    optimization: {
        minimize: false,
        // separate 3rd-party libs because they won't change so often
        moduleIds: 'deterministic',
        chunkIds: 'named',
        splitChunks: {
            chunks: 'async',
            maxAsyncRequests: 6,
            maxInitialRequests: 4,
            cacheGroups: {
                vendors: {
                    test: /[\\/]node_modules[\\/]/,
                    name: 'vendors',
                    chunks: 'all',
                    filename: 'vendors.js'
                }
            }
        }
    },
    plugins: [
        new CleanWebpackPlugin(),
        new MiniCssExtractPlugin({
            filename: '[name]/style.css'
        })
    ]
};
