<?php

/**
 * i-doit
 *
 * DAO: global category for assigned processes.
 *
 * @package    idoit
 * @subpackage smview_connect
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_smview_connect_cmdb_dao_g_assigned_processes extends isys_cmdb_dao_category_global
{

    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var string
     */
    protected $m_category = 'smview_connect_assigned_processes';

    /**
     * Is category multi-valued or single-valued?
     *
     * @var boolean
     */
    protected $m_multivalued = true;

    /**
     * New variable to determine if the current category is a reverse category of another one.
     *
     * @var  string
     */
    protected $m_reverse_category_of = 'isys_cmdb_dao_category_g_it_service_components';

    /**
     * category table
     *
     * @var string
     */
    protected $m_table = 'isys_catg_its_components_list';

    /**
     * Method for getting the object-browsers preselection.
     *
     * @param  int $p_obj_id
     * @return isys_component_dao_result
     */
    public function get_selected_objects($p_obj_id)
    {
        $l_dao = new isys_cmdb_dao_category_g_it_service_components($this->m_db);
        return $l_dao->get_data($p_obj_id);
    }

    /**
     * Method for returning the properties.
     *
     * @return array
     */
    protected function properties()
    {
        // basically use properties of BVG renting DAO
        $l_dao = new isys_cmdb_dao_category_g_it_service_components($this->m_db);
        $properties = $l_dao->get_properties();
        $properties['process_id'] = array_replace_recursive(
            isys_cmdb_dao_category_pattern::object_browser(),
            [
                C__PROPERTY__INFO => [
                    C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__BVG__CUSTOMER_TITLE',
                    C__PROPERTY__INFO__DESCRIPTION => 'Customer ID'
                ],
                C__PROPERTY__DATA => [
                    C__PROPERTY__DATA__FIELD => 'isys_catg_its_components_list__isys_obj__id',
                    C__PROPERTY__DATA__REFERENCES => [
                        'isys_obj',
                        'isys_obj__id'
                    ],
                    C__PROPERTY__DATA__SELECT => idoit\Module\Report\SqlQuery\Structure\SelectSubSelect::factory(
                        'SELECT o.isys_obj__title FROM isys_obj o
                                    INNER JOIN isys_catg_its_components_list l
                                    ON o.isys_obj__id = l.isys_catg_its_components_list__isys_obj__id',
                        'isys_catg_its_components_list',
                        'isys_catg_its_components_list__id',
                        'isys_catg_its_components_list__isys_obj__id'
                    ),
                    C__PROPERTY__DATA__JOIN             => [
                        idoit\Module\Report\SqlQuery\Structure\SelectJoin::factory(
                            'isys_catg_its_components_list',
                            'LEFT',
                            'isys_catg_its_components_list__isys_obj__id',
                            'isys_obj__id'
                        ),
                    ]
                ],
                C__PROPERTY__UI => [
                    C__PROPERTY__UI__ID => 'C__CATG__BVG_RENTING__CUSTOMER_ID',
                    C__PROPERTY__UI__PARAMS => [
                        'multiselection' => false,
                        'catFilter'      => 'C__CATG__BVG_RENTING',
                    ]
                ]
            ]
        );
        return $properties;
    }

    /**
     * Get data method, uses logical unit DAO.
     *
     * @param  int    $p_catg_list_id
     * @param  int    $p_obj_id
     * @param  string $p_condition
     * @param  array  $p_filter
     * @param  int    $p_status
     *
     * @return isys_component_dao_result
     * @throws isys_exception_database
     */
    public function get_data($p_catg_list_id = null, $p_obj_id = null, $p_condition = '', $p_filter = null, $p_status = null)
    {
        $l_obj_id = !empty($p_obj_id) ? $p_obj_id : $this->m_object_id;
        $l_sql = $this->_getBaseQuery($l_obj_id, $p_status);
        if (!empty($p_catg_list_id)) {
            $l_sql .= " AND isys_catg_its_components_list__id = " . $this->convert_sql_id($p_catg_list_id);
        }
        $l_sql .= $p_condition;
        return $this->retrieve($l_sql);
    }

    /**
     * Retrieves the number of saved category-entries to the given object.
     *
     * @param  int $p_obj_id
     * @return int
     * @throws isys_exception_database
     */
    public function get_count($p_obj_id = null)
    {
        $l_obj_id = ! empty($p_obj_id) ? $p_obj_id : $this->m_object_id;
        $l_sql = $this->_getBaseQuery($l_obj_id);
        return $this->retrieve($l_sql)->count();
    }

    /**
     * Get base SQL query.
     *
     * @param  int|null $connectedObjId
     * @param  int|null $status
     *
     * @return string
     */
    private function _getBaseQuery($connectedObjId = null, $status = null)
    {
        $l_sql = "SELECT *, isys_obj__id, isys_obj__title FROM isys_catg_its_components_list
                    JOIN isys_connection c ON isys_catg_its_components_list__isys_connection__id = c.isys_connection__id
                    JOIN isys_obj o ON isys_catg_its_components_list.isys_catg_its_components_list__isys_obj__id = o.isys_obj__id WHERE TRUE ";
        if (! empty($connectedObjId)) {
            $l_sql .= " AND c.isys_connection__isys_obj__id = " . $this->convert_sql_id($connectedObjId);
        }
        $status = $status ?? C__RECORD_STATUS__NORMAL;
        $l_sql .= " AND isys_catg_its_components_list__status = " . $this->convert_sql_int($status);
        return $l_sql;
    }

    /**
     * Creates new entity.
     *
     * @param array $p_data Properties in a associative array with tags as keys and their corresponding values as values.
     *
     * @return int|boolean  Returns created entity's identifier (int) or false (bool).
     */
    public function create_data($p_data)
    {
        // no op
        return false;
    }

    /**
     * Updates existing entity.
     *
     * @param int   $p_list_id Entity's identifier
     * @param array $p_data    Properties in a associative array with tags as keys and their corresponding values as values.
     *
     * @return boolean
     */
    public function save_data($p_list_id, $p_data)
    {
        // no op
        return false;
    }
}
