<?php

use idoit\Component\Property\Type\TextProperty;
use SHD\Smview\Connect\Domain\Azure\idoit\AzureMetadataCategoryData;
use SHD\Smview\Connect\Domain\idoit\Property\Type\JsonProperty;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class isys_smview_connect_cmdb_dao_g_azure_metadata extends isys_cmdb_dao_category_global
{
    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var  string
     */
    protected $m_category = 'smview_connect_azure_metadata';

    /**
     * This variable holds the language constant of the current category.
     *
     * @var string
     */
    protected $categoryTitle = 'LC__CMDB__CATG__SMVIEW_CONNECT__AZURE_METADATA_TITLE';

    /**
     * @var string
     */
    protected $m_category_const = 'C__CATG__SMVIEW_CONNECT_AZURE_METATDATA';

    /**
     * Category entry is purgable
     *
     * @var bool
     */
    protected $m_is_purgable = false;

    /**
     * Is category multi-valued or single-valued?
     *
     * @var  boolean
     */
    protected $m_multivalued = false;

    public $table = 'isys_catg_smview_connect_azure_metadata_list';

    /**
     * @return array
     */
    protected function properties(): array
    {
        $fieldAzureId = new TextProperty(
            'C__CATG__SMVIEW_CONNECT_AZURE_METATDATA__ID',
            'LC__CMDB__CATG__SMVIEW_CONNECT__AZURE_METADATA_ID',
            'isys_catg_smview_connect_azure_metadata_list__azure_id',
            'isys_catg_smview_connect_azure_metadata_list'
        );
        $fieldAzureId->getData()->setReadOnly(true);

        $fieldAzureType = new TextProperty(
            'C__CATG__SMVIEW_CONNECT_AZURE_METATDATA__TYPE',
            'LC__CMDB__CATG__SMVIEW_CONNECT__AZURE_METADATA_TYPE',
            'isys_catg_smview_connect_azure_metadata_list__type',
            'isys_catg_smview_connect_azure_metadata_list'
        );
        $fieldAzureType->getData()->setReadOnly(true);

        $fieldAzureDetails = new JsonProperty(
            'C__CATG__SMVIEW_CONNECT_AZURE_METATDATA__DETAILS',
            'LC__CMDB__CATG__SMVIEW_CONNECT__AZURE_METADATA_DETAILS',
            'isys_catg_smview_connect_azure_metadata_list__details',
            'isys_catg_smview_connect_azure_metadata_list'
        );
        $fieldAzureDetails->setVisible(false);

        return [
            AzureMetadataCategoryData::AZURE_ID => $fieldAzureId,
            AzureMetadataCategoryData::AZURE_TYPE  => $fieldAzureType,
            AzureMetadataCategoryData::AZURE_DETAILS => $fieldAzureDetails,
        ];
    }

    /**
     * @param int $cid
     * @param string $json
     * @return bool
     * @throws isys_exception_dao
     */
    public function setDetails(int $cid, string $json): bool
    {
        if($cid == 0) {
            return false;
        }

        return $this->update(sprintf(
            'update %s set %s__details = %s where %s__id = %s',
            $this->table,
            $this->table,
            $this->convert_sql_text($json),
            $this->table,
            $this->convert_sql_id($cid),
        )) && $this->apply_update();
    }
}
