<?php

use SHD\Smview\Connect\Service\DB\DaoUtil;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class isys_smview_connect_cmdb_dao_g_azure_relation extends isys_cmdb_dao_category_global
{
    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var  string
     */
    protected $m_category = 'smview_connect_azure_relation';

    /**
     * This variable holds the language constant of the current category.
     *
     * @var string
     */
    protected $categoryTitle = 'LC__CMDB__CATG__SMVIEW_CONNECT__AZURE_RELATION_TITLE';

    /**
     * @var string
     */
    protected $m_category_const = 'C__CATG__SMVIEW_CONNECT_AZURE_RELATION';

    /**
     * Category entry is purgable
     *
     * @var bool
     */
    protected $m_is_purgable = false;

    /**
     * Is category multi-valued or single-valued?
     *
     * @var  boolean
     */
    protected $m_multivalued = true;

    public $table = 'isys_catg_smview_connect_azure_relation_list';

    protected $m_connected_object_id_field = 'isys_catg_smview_connect_azure_relation_list__isys_obj__rel_id';

    /**
     * @param int $id1 Object 1
     * @param int $id2 Object 2
     * @param bool $oneway Create the relation only in one direction. Default: both ways
     * @return int              The list id
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function relateObjects(int $id1, int $id2, bool $oneway = false): int
    {
        $id = DaoUtil::allRows($this->retrieve(sprintf('
            select %1$s__id id
            from %1$s
            where %1$s__isys_obj__id = %2$s 
                and %1$s__isys_obj__rel_id = %3$s
        ', $this->table, $this->convert_sql_id($id1), $this->convert_sql_id($id2))))[0]['id'] ?? null;

        //es ist schon ein eintrag vorhanden in der objektkonstellation
        if($id > 0) {
            return $id;
        }

        //neuen eintrag erzeugen
        if ($this->update(sprintf('
            insert into %1$s
            set %1$s__isys_obj__id = %2$s, %1$s__isys_obj__rel_id = %3$s, %1$s__status = 2
        ', $this->table, $this->convert_sql_id($id1), $this->convert_sql_id($id2))) && $this->apply_update()) {
            $id = $this->get_last_insert_id();
        }

        if($oneway) {
            return $id;
        }

        $reverseId = DaoUtil::allRows($this->retrieve(sprintf('
            select %1$s__id id
            from %1$s
            where %1$s__isys_obj__id = %2$s 
                and %1$s__isys_obj__rel_id = %3$s
        ', $this->table, $this->convert_sql_id($id2), $this->convert_sql_id($id1))))[0]['id'] ?? null;

        //umkehrseite ist schon da
        if($reverseId > 0) {
            return $id;
        }

        //neuen reverse eintrag erzeugen
        $this->update(sprintf('
            insert into %1$s
            set %1$s__isys_obj__id = %2$s, %1$s__isys_obj__rel_id = %3$s, %1$s__status = 2
        ', $this->table, $this->convert_sql_id($id2), $this->convert_sql_id($id1))) && $this->apply_update();

        return $id;
    }
}
