<?php

use idoit\Module\Smview_connect\Console\Command\PrewarmApiCachesCommand;
use SHD\Smview\Connect\Service\Baramundi\EndpointImporter;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Telekom\ContractImporter;

/**
 * SM-VIEW Connect Import AJAX handler
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2021 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_ajax_handler_connect_connect extends isys_ajax_handler
{
    /**
     * Init method, which gets called from the framework.
     */
    public function init()
    {
        // We set the header information because we don't accept anything than JSON.
        header('Content-Type: application/json');

        $l_return = [
            'success' => true,
            'message' => null,
            'data'    => null
        ];

        try {
            switch ($this->m_get['func']) {
                case 'import_telekom':
                    /** @var ContractImporter $contractImporter */
                    $contractImporter = isys_application::instance()->container->get('smview_connect.telekom.contract_importer');
                    $contractImporter->import();
                    $l_return['message'] = 'Import finished.';
                    break;
                case 'import_baramundi':
                    /** @var EndpointImporter $endpointImporter */
                    $endpointImporter = isys_application::instance()->container->get('smview_connect.baramundi.endpoint_importer');
                    $endpointImporter->import();
                    $l_return['message'] = 'Import finished.';
                    break;
                case 'defaults_baramundi':
                    break;
                case 'prewarm_bramundi_api_cache':
                    $username = $this->m_get['username'];
                    $password = $this->m_get['password'];
                    $cmd = 'php '.isys_module_shd_smview_connect::getIdoitDir().'/console.php connect:cache:prewarm-api --remove --user '.$username.' --password '.$password;

                    $l_return['data'] = shell_exec($cmd);
                    break;
                case 'defaults_telekom':
                    break;
                case 'import_azure':
                    $azureProfiles = DaoUtil::allRows(\SHD\Smview\Connect\DAO\ProfileDAO::getInstance()->get_data(null, isys_module_shd_smview_connect::CONNECT_MODULE_AZURE));
                    foreach ($azureProfiles as $profile) {
                        if($profile['active'] != 1) {
                            continue;
                        }

                        /** @var \SHD\Smview\Connect\Service\Azure\Import\ResourceImporter $importer */
                        $importer = isys_application::instance()->container->get('smview_connect.azure.profile_'.$profile['id'].'.resource_importer');
                        $importer->import();
                    }

                    $l_return['message'] = 'Import finished';
                    break;
                case 'import_ldap':
                    $profiles = \SHD\Smview\Connect\Service\Ldap\LdapConfiguration::getProfiles();
                    foreach ($profiles as $profile) {
                        if($profile['active'] != 1) {
                            continue;
                        }

                        $importer = isys_application::instance()->container->get('smview_connect.ldap.profile_'.$profile['id'].'.importer');
                        $importer->import();
                    }

                    $l_return['message'] = 'Import finished';
                    break;
                case 'import_vsphere':
                    $vsphereProfiles = \SHD\Smview\Connect\Service\Vsphere\VsphereConfiguration::getProfiles();
                    foreach ($vsphereProfiles as $profile) {
                        if ($profile['active'] != 1) {
                            continue;
                        }

                        /** @var SHD\Smview\Connect\Service\Vsphere\Import\VmImporter $vmImporter */
                        $vmImporter = isys_application::instance()->container->get('smview_connect.vsphere.profile_' . $profile['id'] . '.vm_importer');
                        $vmImporter->import();
                    }
                    $l_return['message'] = 'Import finished.';
                    break;
            }
        } catch (Exception $e) {
            $l_return['success'] = false;
            $l_return['message'] = $e->getMessage();
        }

        echo isys_format_json::encode($l_return);

        $this->_die();
    }

    /**
     * This method defines, if the hypergate needs to be included for this request.
     *
     * @static
     * @return boolean
     */
    public static function needs_hypergate(): bool
    {
        return true;
    }
}
