<?php

use idoit\AddOn\AuthableInterface;
use SHD\Smview\Connect\Service\ConnectLogger;
use SHD\Smview\Connect\Service\AbstractConfiguration;
use SHD\Smview\Connect\Service\Aeneis\AeneisConfiguration;
use SHD\Smview\Connect\Service\Auth\IdoitAuth;
use SHD\Smview\Connect\Service\Azure\AzureConfiguration;
use SHD\Smview\Connect\Service\Baramundi\BaramundiConfiguration;
use SHD\Smview\Connect\Service\CacheManager;
use SHD\Smview\Connect\Service\Gcache\GcacheConfiguration;
use SHD\Smview\Connect\Service\Ldap\LdapConfiguration;
use SHD\Smview\Connect\Service\Telekom\ContractImporter;
use SHD\Smview\Connect\Service\Telekom\TelekomConfiguration;
use SHD\Smview\Connect\Service\Vsphere\VsphereConfiguration;
use Symfony\Component\HttpClient\HttpClient;

/**
 * i-doit
 *
 * SM-VIEW Connect module.
 *
 * @package    idoit
 * @subpackage smview_connect
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2022 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_module_shd_smview_connect extends isys_module implements AuthableInterface
{
    public const DISPLAY_IN_MAIN_MENU = true;
    public const DISPLAY_IN_SYSTEM_MENU = false;

    //use rewrite addon url
    public const MAIN_MENU_REWRITE_LINK = true;

    public const CONNECT_LOG_ENABLED = 'smview_connect.setting.log_enabled';
    public const CONNECT_LOG_LEVEL = 'smview_connect.setting.log_level';

    public const CONNECT_MODULE_LDAP = 'LDAP';
    public const CONNECT_MODULE_LDAP_ATTRIBUTES = 'smview_connect.connect.ldap_attributes';
    public const CONNECT_MODULE_LDAP_PROFILE_UI = 'C__SMVIEW_CONNECT__PROFILE';
    public const CONNECT_MODULE_LDAP_PROFILE = 'smview_connect.connect.ldap_profile';

    public const CONNECT_MODULE_TELEKOM = 'TELEKOM';
    public const CONNECT_MODULE_TELEKOM_ATTRIBUTES = 'smview_connect.connect.telekom_attributes';
    public const CONNECT_MODULE_TELEKOM_PROFILE_UI = 'C__SMVIEW_CONNECT__PROFILE';
    public const CONNECT_MODULE_TELEKOM_PROFILE = 'smview_connect.connect.telekom_profile';

    public const CONNECT_MODULE_BARAMUNDI = 'BARAMUNDI';
    public const CONNECT_MODULE_BARAMUNDI_ATTRIBUTES = 'smview_connect.connect.baramundi_attributes';
    public const CONNECT_MODULE_BARAMUNDI_PROFILE_UI = 'C__SMVIEW_CONNECT__PROFILE';
    public const CONNECT_MODULE_BARAMUNDI_PROFILE = 'smview_connect.connect.baramundi_profile';

    public const CONNECT_MODULE_AZURE = 'AZURE';
    public const CONNECT_MODULE_AZURE_ATTRIBUTES = 'smview_connect.connect.azure_attributes';
    public const CONNECT_MODULE_AZURE_PROFILE_UI = 'C__SMVIEW_CONNECT__PROFILE';
    public const CONNECT_MODULE_AZURE_PROFILE = 'smview_connect.connect.azure_profile';

    public const CONNECT_MODULE_VSPHERE = 'VSPHERE';
    public const CONNECT_MODULE_VSPHERE_ATTRIBUTES = 'smview_connect.connect.vsphere_attributes';
    public const CONNECT_MODULE_VSPHERE_PROFILE_UI = 'C__SMVIEW_CONNECT__PROFILE';
    public const CONNECT_MODULE_VSPHERE_PROFILE = 'smview_connect.connect.vsphere_profile';

    public const CONNECT_MODULE_AENEIS_SETTINGS = 'smview_connect.connect.aeneis_settings';

    public const CONNECT_MODULE_GCACHE = 'GCACHE';
    public const CONNECT_MODULE_GCACHE_ATTRIBUTES = 'smview_connect.connect.gcache_attributes';
    public const CONNECT_MODULE_GCACHE_PROFILE_UI = 'C__SMVIEW_CONNECT__PROFILE';
    public const CONNECT_MODULE_GCACHE_PROFILE = 'smview_connect.connect.gcache_profile';

    public const LOGGING_FIELD_MAPPING = [
        'enabled' => [
            'ui_key' => 'C__SMVIEW_CONNECT__LOGGING_ENABLED',
            'db_key' => self::CONNECT_LOG_ENABLED,
            'type'   => 'boolean',
        ],
        'level' => [
            'ui_key' => 'C__SMVIEW_CONNECT__LOG_LEVEL',
            'db_key' => self::CONNECT_LOG_LEVEL,
            'type'   => 'id',
        ],
    ];

    /**
     * @var bool
     */
    protected static $m_licenced = true;

    /**
     * Variable which holds the database component.
     *
     * @var isys_component_database|null
     */
    protected $m_db = null;

    /**
     * Variable which holds the template component.
     *
     * @var isys_component_template|null
     */
    protected $m_tpl = null;

    public static function getAdapters(): array
    {
        return [
            TelekomConfiguration::LICENSE_KEY => ['class' => TelekomConfiguration::class, 'enabled' => TelekomConfiguration::isEnabled(), 'licensed' => TelekomConfiguration::isLicensed(),],
            BaramundiConfiguration::LICENSE_KEY => ['class' => BaramundiConfiguration::class, 'enabled' => BaramundiConfiguration::isEnabled(), 'licensed' => BaramundiConfiguration::isLicensed(),],
            VsphereConfiguration::LICENSE_KEY => ['class' => VsphereConfiguration::class, 'enabled' => VsphereConfiguration::isEnabled(), 'licensed' => VsphereConfiguration::isLicensed(),],
            LdapConfiguration::LICENSE_KEY => ['class' => LdapConfiguration::class, 'enabled' => LdapConfiguration::isEnabled(), 'licensed' => LdapConfiguration::isLicensed(),],
            AzureConfiguration::LICENSE_KEY => ['class' => AzureConfiguration::class, 'enabled' => AzureConfiguration::isEnabled(), 'licensed' => AzureConfiguration::isLicensed(),],
            AeneisConfiguration::LICENSE_KEY => ['class' => AeneisConfiguration::class, 'enabled' => AeneisConfiguration::isEnabled(), 'licensed' => AeneisConfiguration::isLicensed(),],
            GcacheConfiguration::LICENSE_KEY => ['class' => GcacheConfiguration::class, 'enabled' => GcacheConfiguration::isEnabled(), 'licensed' => true,]
        ];
    }

    public function init(): isys_module_shd_smview_connect
    {
        $this->di();

        return $this;
    }

    /**
     * @return void
     * @throws Exception
     */
    private function di(): void
    {
        //create di services
        $moduleParameterBag = new Symfony\Component\DependencyInjection\ParameterBag\ParameterBag();
        $moduleParameterBag->set('smview_connect.cache_dir', self::getIdoitDir().'/temp/shd_smview_connect');
        $moduleParameterBag->set('smview_connect.api_cache_dir', self::getIdoitDir().'/temp/shd_smview_connect/api');

        isys_application::instance()->container->set('smview_connect.parameter', $moduleParameterBag);

        //http
        $proxyUrl = isys_tenantsettings::get(AbstractConfiguration::IDOIT_FIELD_MAPPING['proxy.url']['db_key'], null);
        $httpOptions = [];
        if($proxyUrl != null) {
            $httpOptions['proxy'] = $proxyUrl;
        }
        $http = HttpClient::create($httpOptions);

        isys_application::instance()->container->set('smview_connect.http_client', $http);

        $logger = new ConnectLogger();
        isys_application::instance()->container->set('smview_connect.logger', $logger);

        $idoitApiClient = new \SHD\Smview\Connect\Service\idoit\IdoitApiClient(
            $http,
            isys_tenantsettings::get(AbstractConfiguration::IDOIT_FIELD_MAPPING['idoit.url']['db_key'], null),
            isys_tenantsettings::get(AbstractConfiguration::IDOIT_FIELD_MAPPING['idoit.user']['db_key'], null),
            isys_tenantsettings::get(AbstractConfiguration::IDOIT_FIELD_MAPPING['idoit.password']['db_key'], null),
            isys_tenantsettings::get(AbstractConfiguration::IDOIT_FIELD_MAPPING['idoit.apikey']['db_key'], null),
        );
        isys_application::instance()->container->set('smview_connect.idoit_api_client', $idoitApiClient);

        //cache manager
        $cacheManager = new CacheManager();
        isys_application::instance()->container->set('smview_connect.cache', $cacheManager);

        //threading
        $threading = new \SHD\Smview\Connect\Service\Threading\ThreadDispatch($idoitApiClient, $logger);
        \isys_application::instance()->container->set('smview_connect.thread_dispatch', $threading);

        //value pipeline
        $objectService = new \SHD\Smview\Connect\Service\Import\ValuePipeline\ObjectService(
            isys_cmdb_dao_object::instance(isys_application::instance()->container->get('database')),
        );
        $categoryService = new \SHD\Smview\Connect\Service\Import\ValuePipeline\CategoryService();
        $dialogService = new \SHD\Smview\Connect\Service\Import\ValuePipeline\DialogService();

        $valuePipeline = new \SHD\Smview\Connect\Service\Import\ValuePipeline(
            $objectService,
            $categoryService,
            $dialogService,
        );
        isys_application::instance()->container->set('smview_connect.value_pipeline', $valuePipeline);

        $importManager = new \SHD\Smview\Connect\Service\Import\IdoitImportManager(
            isys_cmdb_dao_object::instance(isys_application::instance()->container->get('database')),
            $logger,
        );
        isys_application::instance()->container->set('smview_connect.import_manager', $importManager);

        AeneisConfiguration::registerDI();
        \SHD\Smview\Connect\Service\Azure\AzureConfiguration::registerDI();
        \SHD\Smview\Connect\Service\Vsphere\VsphereConfiguration::registerDI();
        \SHD\Smview\Connect\Service\Ldap\LdapConfiguration::registerDI();
        BaramundiConfiguration::registerDI();
        TelekomConfiguration::registerDI();
        GcacheConfiguration::registerDI();
    }

    public static function getParameter(string $parameter)
    {
        /** @var \Symfony\Component\DependencyInjection\ParameterBag\ParameterBag $bag */
        $bag = isys_application::instance()->container->get('smview_connect.parameter');

        return $bag->get($parameter);
    }

    /**
     * UGLY FORMSAVE HACK -> nötig um die navbar/edit/save funktionalität zu aktivieren (action parameter wird auf der isys_form benötigt, sonst geht edit button nicht; url muss nach alter form mit moduleId gefüllt sein)
     * //todo: ggf. andere lösung finden
     *
     * @return isys_module_shd_smview_connect
     * @throws Exception
     */
    public function start(): isys_module_shd_smview_connect
    {
        if (func_num_args() > 0) {
            /** @var isys_register $l_request */
            $l_request = func_get_arg(0);

            $lang = isys_application::instance()->language;

            //UGLY FORMSAVE HACK -> nötig um die navbar/edit/save funktionalität zu aktivieren (action parameter wird auf der isys_form benötigt, sonst geht edit button nicht; url muss nach alter form mit moduleId gefüllt sein)
            //todo ggf. anders machen
            if (isset($_GET['_controller']) && isset($_GET['_action'])) {
                $l_request->action = $_GET['_controller'] == null || $_GET['_controller'] == '' ? 'Main' : $_GET['_controller'];
                $l_request->method = $_GET['_action'] == null || $_GET['_action'] == '' ? null : $_GET['_action'];
            }

            $url = isys_helper_link::create_url([C__GET__MODULE_ID => $this->getModuleId(), '_controller' => $l_request->action, '_action' => $l_request->method, 'lang' => $lang]);

            isys_application::instance()->template
                ->assign('formAdditionalAction', "action=\"{$url}\"")
                ->assign('query_string', $url)
            ;
            ////////////////////////////////////

            return parent::start($l_request);
        } else {
            throw new RuntimeException('Module not compatible with i-doit ' . isys_application::instance()->info->get('version'));
        }
    }

    /**
     * Static method for retrieving the path, to the modules templates.
     *
     * @static
     * @return string
     */
    public static function get_tpl_dir(): string
    {
        return __DIR__ . '/templates/';
    }

    /**
     * Method for retrieving the www-path to the assets directory (needed for the frontend).
     *
     * @static
     * @return  string
     */
    public static function get_assets_www_dir(): string
    {
        return isys_application::instance()->www_path . 'src/classes/modules/shd_smview_connect/assets/';
    }

    /**
     * Represents the name, of the corresponding auth class - "false" means, there is none.
     *
     * @var string
     */
    public const AUTH_CLASS_NAME = 'isys_auth_module_smview_connect';

    /**
     * Method for retrieving the path to the module directory (needed for includes).
     *
     * @static
     * @return string
     */
    public static function get_module_dir(): string
    {
        return __DIR__ . DS;
    }

    /**
     * Method for retrieving the www-path (needed for the frontend).
     *
     * @static
     * @return  string
     */
    public static function get_www_dir(): string
    {
        return isys_application::instance()->www_path . 'src/classes/modules/shd_smview_connect/';
    }

    /**
     * Get API key of mandator.
     *
     * @return string
     * @throws Exception
     */
    public static function getApiKey(): string
    {
        $sysDb = isys_application::instance()->container->get('database_system');
        $session = isys_application::instance()->container->get('session');

        $mandatorId = (int)$session->get_mandator_id();
        $data = $sysDb->fetch_row_assoc($sysDb->query(
            <<<SQL
            SELECT isys_mandator__apikey apiKey FROM isys_mandator WHERE isys_mandator__id = {$mandatorId};
SQL
        ));

        return $data['apiKey'];
    }

    /**
     * Get all object types.
     *
     * @return array
     * @throws isys_exception_database
     */
    public static function getObjectTypes(): array
    {
        $lang = isys_application::instance()->container->get('language');
        $dao = new isys_cmdb_dao_object_type(isys_application::instance()->database);
        $res = $dao->retrieve("SELECT t.* FROM isys_obj_type t;");
        $object_types = [];

        while ($row = $res->get_row()) {
            $object_types[$row['isys_obj_type__id']] = $lang->get($row['isys_obj_type__title']);
        }

        return $object_types;
    }

    /**
     * Get translated strings to use with javascript.
     *
     * @return array
     * @throws Exception
     */
    public static function getJsTranslations(): array
    {
        // language templates
        $language_templates = [
            'LC__UNIVERSAL__COPY',
            'LC_UNIVERSAL__ACCEPT',
            'LC_UNIVERSAL__ABORT',
            'LC__UNIVERSAL__EDIT',
            'LC__UNIVERSAL__DETACH',
            'LC__UNIVERSAL__DELETE',
            'LC_UNIVERSAL__DELETE',
            'LC__UNIVERSAL__BUTTON_SAVE',
            'LC__UNIVERSAL__ADD',
            'LC__CMDB__OBJTYPE',
            'LC__UNIVERSAL__LOAD',
            'LC__UNIVERSAL__YES',
            'LC__UNIVERSAL__NO',
            'LC__CMDB__SMVIEW_CONNECT__MATCHING_TITLE',
            'LC__CMDB__SMVIEW_CONNECT__MATCHING_MATCHING',
            'LC__CMDB__SMVIEW_CONNECT__MATCHING_EDIT_ASSIGNMENT',
            'LC__CMDB__SMVIEW_CONNECT__MATCHING_DETACH_ASSIGNMENT',
            'LC__CMDB__SMVIEW_CONNECT__MATCHING_COLUMN_NAME',
            'LC__CMDB__SMVIEW_CONNECT__MATCHING_FIRST_LINE',
            'LC__CMDB__SMVIEW_CONNECT__MATCHING_ASSIGNMENT',
            'LC__CMDB__SMVIEW_CONNECT__MATCHING_NO_ASSIGNMENT',
            'LC__CMDB__SMVIEW_CONNECT__MATCHING_CATEGORY_PROPERTY',
            'LC__CMDB__SMVIEW_CONNECT__MATCHING_ADD_FIELD',
            'LC__CMDB__SMVIEW_CONNECT__MATCHING_MIN_MATCHING_COUNT',
            'LC__CMDB__SMVIEW_CONNECT__MATCHING_SAVE_PROFILE',
            'LC__CMDB__SMVIEW_CONNECT__MATCHING_ADD_NEW_REFERENCES',
            'LC__CMDB__SMVIEW_CONNECT__MATCHING_ADD_NEW_OBJECTS',
            'LC__CMDB__SMVIEW_CONNECT__MATCHING_ALL_MAPPINGS',
            'LC__CMDB__SMVIEW_CONNECT__MATCHING_SOURCE',
            'LC__CMDB__SMVIEW_CONNECT__MATCHING_TARGET',
            'LC__CMDB__SMVIEW_CONNECT__PROFILES',
            'LC__CMDB__SMVIEW_CONNECT__CONNECT_CREATE_OBJECTS_HINT',
            'LC__CMDB__SMVIEW_CONNECT__CONNECT_CREATE_MISSING_REFERENCES_HINT',
            'LC__CMDB__SMVIEW_CONNECT__CONNECT_BARAMUNDI_ORGUNIT_TREE',
            'LC__CMDB__SMVIEW_CONNECT__CONNECT_BARAMUNDI_SELECTED_ORGUNIT',
            'LC__CMDB__SMVIEW_CONNECT__CONNECT_BARAMUNDI_SET_ORGUNIT',
            'LC__CMDB__SMVIEW_CONNECT__CONNECT_SETTINGS_SETTINGS',
            'LC__CMDB__SMVIEW_CONNECT__CONNECT_BARAMUNDI_TREE_VIEW_ONLY_EDIT_MODE',
            'LC__CMDB__SMVIEW_CONNECT__PROFILE_ACTIVE',
            'LC__CMDB__SMVIEW_CONNECT__PROFILE_ACTIVE_HINT',
            'LC__CMDB__SMVIEW_CONNECT__PROFILE_NEW',
            'LC__CMDB__SMVIEW_CONNECT__PROFILE_NAME',
            'LC__CMDB__SMVIEW_CONNECT__DATA_MAPPING',
            'LC__CMDB__CLICK_TO_HIDE',
            'LC__CMDB__CLICK_TO_REVEAL',
            'LC__CMDB__SMVIEW_CONNECT__DISCOVERY_TITLE',
            'LC__CMDB__SMVIEW_CONNECT__START_DISCOVERY',
            'LC__CMDB__SMVIEW_CONNECT__CONNECT_VSPHERE_USERNAME',
            'LC__CMDB__SMVIEW_CONNECT__CONNECT_VSPHERE_API_BASE',
            'LC__CMDB__SMVIEW_CONNECT__CONNECT_VSPHERE_PASSWORD',
            'LC__CMDB__SMVIEW_CONNECT__TELEPHONE_RATE',
            'LC__CMDB__SMVIEW_CONNECT__MATCHING_CAT_COMPARE_ASSIGNMENT',
            'LC__CMDB__SMVIEW_CONNECT__API_EXTERNAL_STATUS',
            'LC__CMDB__SMVIEW_CONNECT__API_ONLINE',
            'LC__CMDB__SMVIEW_CONNECT__API_NOT_REACHABLE',
            'LC__CMDB__SMVIEW_CONNECT__VALUE_PIPELINE',
            'LC__CMDB__SMVIEW_CONNECT__VALUE_PIPELINE_FUNCTIONS',
            'LC__CMDB__SMVIEW_CONNECT__CONNECT_CSV',
            'LC__CMDB__SMVIEW_CONNECT__CSV_HEADER',
            'LC__CMDB__SMVIEW_CONNECT__CSV_READ',
            'LC__CMDB__SMVIEW_CONNECT__CSV_IMPORT',
            'LC__CMDB__SMVIEW_CONNECT__PROFILE_SELECT',
            'LC__CMDB__SMVIEW_CONNECT__CRON_INVALID',
            'LC__CMDB__SMVIEW_CONNECT__ADD_PIPELINE_ELEMENT',
            'LC__CMDB__SMVIEW_CONNECT__TOGGLE_PIPELINE_EDIT',
            'LC__CMDB__SMVIEW_CONNECT__DELETE_PIPELINE_ELEMENT',
            'LC__CMDB__SMVIEW_CONNECT__MOVE_PIPELINE_ELEMENT',
        ];

        // translate
        $language = [];

        /** @var isys_component_template_language_manager $lang */
        $lang = isys_application::instance()->container->get('language');
        foreach ($language_templates as $l) {
            $language[$l] = $lang->get($l);
        }
        return $language;
    }

    /**
     * @return int
     */
    public function getModuleId(): int
    {
        return $this->m_data['id'];
    }

    /**
     * Checks if App is in Dev Mode
     *
     * @return bool
     */
    public static function isDevMode(): bool
    {
        return file_exists(__DIR__.'/../../../../dev');
    }

    public static function getAppDir(): string
    {
        $l = isys_helper_link::get_base();
        if(substr($l, -1) == '/') {
            return substr($l, 0, -1);
        }

        return $l;
    }

    public static function getIdoitDir(): string
    {
        return __DIR__.'/../../../..';
    }

    public static function getAuth(): isys_auth
    {
        return IdoitAuth::instance();
    }
}
