<?php

/**
 * i-doit
 *
 * Class autoloader.
 *
 * @package    idoit
 * @subpackage smview_connect
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2022 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_module_shd_smview_connect_autoload extends isys_module_manager_autoload
{
    /**
     * Autoloader
     *
     * @param  string $className
     * @return boolean
     */
    public static function init($className): bool
    {
        $addOnPath = '/src/classes/modules/shd_smview_connect/';
        $class_loader_paths = array(
            '/api/model/',
            '/api/model/connect/',
            '/handler/',
            '/handler/ajax/',
            '/cmdb/dao/',
            '/cmdb/dao/global/',
            '/cmdb/ui/',
            '/cmdb/ui/global/',
            '/install/',
            '/cmdb/dao/list/',
            '/'
        );

        //dynamic loading of classes
        foreach ($class_loader_paths as $clp) {
            $l_path = $clp . $className . '.class.php';
            if (! file_exists(__DIR__ . $l_path)) {
                continue;
            }

            if (parent::include_file($addOnPath . $l_path)) {
                isys_cache::keyvalue()->ns('autoload')->set($className, $addOnPath . $l_path);
                return true;
            }
        }

        $l_classes = [
            'isys_ajax_handler_connect_connect'                                           => __DIR__ . '/handler/ajax/isys_ajax_handler_connect_connect.class.php',
            'ShdPgp'                                                                      => '/pgp/ShdPgp.class.php',
        ];

//        // include composer's autoloader
//        $vendorDir = dirname(__FILE__) . '/vendor/';
//        if (file_exists($vendorDir . 'autoload.php')) {
//            include_once($vendorDir . 'autoload.php');
//        }
//        unset($vendorDir);

        if (isset($l_classes[$className])) {
            if (parent::include_file($addOnPath . $l_classes[$className])) {
                isys_cache::keyvalue()->ns('autoload')->set($className, $addOnPath . $l_classes[$className]);
                return true;
            }
        }
        return false;
    }
}
