<?php

use idoit\Module\Api\ApiEnvironmentShell;
use idoit\Module\Api\Session\SessionIdSession;
use SHD\Smview\Connect\Service\Rest\RestController;

include __DIR__ . '/../api/src/ApiEnvironmentShell.php';

function restErrorOutput($message, $errorCode = 500, $data = null)
{
    http_response_code($errorCode);
    header('Content-Type: application/json; charset=utf-8');

    echo json_encode([
        'error' => $message,
        'code' => $errorCode,
        'data' => $data,
    ]);
}

// Create environment shell.
$apiEnvironmentShell = new ApiEnvironmentShell($idoitRootDirectory);

// Setup boot error handler.
$apiEnvironmentShell->injectBootErrorHandler(function (\Exception $e) {
    /**
     * Boot sequence exited with errors
     *
     * There is no guarantee that autoloading and other
     * fundamental application services were loaded correctly.
     *
     * Therefore do not access them to prevent further errors.
     */
    restErrorOutput('i-doit System Error: ' . $e->getMessage());
});

// Setup error handler.
$apiEnvironmentShell->injectErrorHandler(function (\Exception $e) {
    if($e instanceof \SHD\Smview\Connect\Service\Rest\RestApiException) {
        restErrorOutput('i-doit System Error: ' . $e->getMessage(), 500, $e->getContext());
    } else {
        restErrorOutput('i-doit System Error: ' . $e->getMessage());
    }
});

// Setup executable.
$apiEnvironmentShell->injectExecutable(function (ApiEnvironmentShell $apiShell) {
    // Check whether api init file exists.
    if (!file_exists($apiShell->getIdoitRootDirectoryPath() . '/src/classes/modules/api/init.php')) {
        throw new Exception('Api Module is not available.');
    }

    //module initialisieren
    require_once $apiShell->getIdoitRootDirectoryPath() . '/src/classes/modules/api/init.php';
    require_once $apiShell->getIdoitRootDirectoryPath() . '/src/classes/modules/shd_smview_connect/init.php';

    if (!class_exists(RestController::class)) {
        throw new Exception('Error: i-doit is unavailable.');
    }

    $request = \Symfony\Component\HttpFoundation\Request::createFromGlobals();

    //credentials holen
    if(!$request->headers->has('X-TOKEN')) {
        $apiKey = $request->headers->get('X-APIKEY');
        $username = $request->headers->get('X-USERNAME');
        $password = $request->headers->get('X-PASSWORD');
    } else {
        list($apiKey, $username, $password) = explode(':', $request->headers->get('X-TOKEN'));
    }

    $sessionId = $request->headers->get(isys_core::HTTP_RPCAuthSession) ?? null;

    $restController = RestController::instance($apiKey, $username, $password, $sessionId);

    //bootstrapping ist irgendwie scheisse.... hier nochmal dafür sorgen dass der tenant passt
    if (isys_application::instance()->tenant === null) {
        $tenantData = isys_application::instance()->container->get('session')->get_mandator_data();

        if (is_array($tenantData) && count($tenantData)) {
            isys_application::instance()->tenant = new isys_tenant(
                $tenantData['isys_mandator__title'],
                $tenantData['isys_mandator__description'],
                $tenantData['isys_mandator__id'],
                $tenantData['isys_mandator__db_name'],
                $tenantData['isys_mandator__dir_cache']
            );
        }
    }

    //modul zum drölften mal laden
    isys_module_manager::instance()->module_loader(true);

    $module = new isys_module_shd_smview_connect();
    $module->init();
    /////////

    //jetzt gehts los -> rest URI holen
    $q = $request->get('q') ?? null;

    if(!$q) throw new Exception('no query was sent');

    // Run all requests.
    $response = $restController->runRest($request);

    http_response_code(200);
    header('Content-Type: application/json; charset=utf-8');

    // Expose sessionId by sending x-rpc-header.
    if ($restController->getSession() instanceof SessionIdSession) {
        isys_core::send_header(isys_core::HTTP_RPCAuthSession, $restController->getSession()
            ->getSessionId());
    }

    echo $response->toJson();
});

/**
 * Run executable in shell.
 */
try {
    $apiEnvironmentShell->run();
} catch (\Exception $e) {
    // An shell error has occured.
    if($e instanceof \SHD\Smview\Connect\Service\Rest\RestApiException) {
        restErrorOutput('i-doit System Error: ' . $e->getMessage(), 500, $e->getContext());
    } else {
        restErrorOutput('i-doit System Error: ' . $e->getMessage());
    }
}
