<?php

namespace idoit\Module\shd_smview_connect\Console\Command;

use idoit\Console\Command\AbstractCommand;
use isys_application;
use isys_exception_database;
use isys_exception_general;
use SHD\Smview\Connect\DAO\ProfileAssignmentDAO;
use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\Domain\Gcache\GcacheEntryMapping;
use SHD\Smview\Connect\Service\CsvImporterInterface;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Gcache\Importer as GcacheImporter;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputDefinition;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

class CSVListCommand extends AbstractCommand
{
    protected static $defaultName = 'connect:csv:list';

    /**
     * @param InputInterface $input
     * @param OutputInterface $output
     * @return int
     * @throws isys_exception_database
     */
    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $io = new SymfonyStyle($input, $output);

        $profiles = DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, \isys_module_shd_smview_connect::CONNECT_MODULE_GCACHE)) ?? [];

        foreach ($profiles as $profile) {
            $io->writeln(sprintf('ID: %s, Name: %s, Active: %s', $profile['id'], $profile['title'], $profile['active'] ? 'true' : 'false'));
        }

        return Command::SUCCESS;
    }

    public function getCommandName(): string
    {
        return self::$defaultName;
    }

    public function getCommandDescription(): string
    {
        return 'list csv importers';
    }

    public function getCommandDefinition(): InputDefinition
    {
        return new InputDefinition();
    }

    public function isConfigurable(): bool
    {
        return false;
    }

    public function getCommandUsages(): array
    {
        return [];
    }
}
