<?php

namespace idoit\Module\shd_smview_connect\Console\Command;

use idoit\Console\Command\AbstractCommand;
use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\ImporterInterface;
use isys_application;
use isys_exception_database;
use isys_module_shd_smview_connect;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputDefinition;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

class ImportAllCommand extends AbstractCommand
{
    protected static $defaultName = 'connect:import-all';

    public function __construct(string $name = null)
    {
        parent::__construct($name);
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $importers = self::getImporters();
        $io = new SymfonyStyle($input, $output);

        $io->section('Import');

        foreach ($importers as $importer) {
            if(!$importer->isLicensed()) {
                $io->warning(sprintf('importer %s is not licensed', $importer->getName()));
                continue;
            }

            if(!$importer->isEnabled()) {
                $io->warning(sprintf('importer %s is not enabled', $importer->getName()));
                continue;
            }

            $io->text(sprintf('starting import for importer %s (profile %s)', $importer->getName(), $importer->getProfileId()));
            $importer->import();
            $io->text(sprintf('finished import for importer %s (profile %s)', $importer->getName(), $importer->getProfileId()));
        }

        return Command::SUCCESS;
    }

    /**
     * @return ImporterInterface[]
     * @throws isys_exception_database
     */
    public static function getImporters(): array
    {
        $importers = [];

        $ldapProfiles = DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, isys_module_shd_smview_connect::CONNECT_MODULE_LDAP));

        foreach ($ldapProfiles as $profile) {
            if(isys_application::instance()->container->has('smview_connect.ldap.profile_'.$profile['id'].'.importer')) {
                $importers[] = isys_application::instance()->container->get('smview_connect.ldap.profile_'.$profile['id'].'.importer');
            }
        }

        if(isys_application::instance()->container->has('smview_connect.telekom.contract_importer')) {
            $importers[] = isys_application::instance()->container->get('smview_connect.telekom.contract_importer');
        }

        if(isys_application::instance()->container->has('smview_connect.baramundi.endpoint_importer')) {
            $importers[] = isys_application::instance()->container->get('smview_connect.baramundi.endpoint_importer');
        }

        $vsphereProfiles = DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, isys_module_shd_smview_connect::CONNECT_MODULE_VSPHERE));

        foreach ($vsphereProfiles as $profile) {
            if(isys_application::instance()->container->has('smview_connect.vsphere.profile_'.$profile['id'].'.importer')) {
                $importers[] = isys_application::instance()->container->get('smview_connect.vsphere.profile_'.$profile['id'].'.vm_importer');
            }
        }

        return $importers;
    }

    public function getCommandName(): string
    {
        return self::$defaultName;
    }

    public function getCommandDescription(): string
    {
        return 'Import all';
    }

    public function getCommandDefinition(): InputDefinition
    {
        return new InputDefinition();
    }

    public function isConfigurable(): bool
    {
        return false;
    }

    public function getCommandUsages(): array
    {
        return [];
    }
}
