<?php

namespace idoit\Module\shd_smview_connect\Console\Command;

use idoit\Console\Command\AbstractCommand;
use SHD\Smview\Connect\Service\ImporterInterface;
use isys_exception_database;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputDefinition;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

class ImportCommand extends AbstractCommand
{
    protected static $defaultName = 'connect:import';

    /**
     * @param InputInterface $input
     * @param OutputInterface $output
     * @return int
     * @throws isys_exception_database
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $importers = ImportAllCommand::getImporters();

        $io = new SymfonyStyle($input, $output);

        $importerName = $input->getArgument('importer');
        $profileId = $input->getArgument('profile_id');

        $io->section(sprintf('Import %s', $importerName));

        /** @var ImporterInterface $importer */
        $importer = array_values(array_filter($importers, function ($i) use ($importerName, $profileId) {
            return $i->getName() === $importerName && ($i->getProfileId() == $profileId || $i->getProfileId() == null);
        }))[0] ?? null;

        if(!$importer) {
            $io->error(sprintf('importer %s (profile %s) was not found', $importerName, $profileId));

            return Command::FAILURE;
        }

        if(!$importer->isLicensed()) {
            $io->warning(sprintf('importer %s is not licensed', $importer->getName()));
            return Command::FAILURE;
        }

        if(!$importer->isEnabled()) {
            $io->warning(sprintf('importer %s is not enabled', $importer->getName()));
            return Command::FAILURE;
        }

        $io->text(sprintf('starting import for importer %s', $importer->getName()));
        $timeStart = time();
        $importer->import();
        $timeEnd = time();
        $io->text(sprintf('finished import for importer %s (profile %s) in %s seconds (%s minutes)', $importer->getName(), $profileId, $timeEnd - $timeStart, ($timeEnd - $timeStart) / 60));

        return Command::SUCCESS;
    }

    public function getCommandName(): string
    {
        return self::$defaultName;
    }

    public function getCommandDescription(): string
    {
        return 'Start import of a scpecified importer';
    }

    public function getCommandDefinition(): InputDefinition
    {
        $id = new InputDefinition();
        $id->addArgument(new InputArgument('importer', InputArgument::REQUIRED));
        $id->addArgument(new InputArgument('profile_id', InputArgument::REQUIRED));

        return $id;
    }

    public function isConfigurable(): bool
    {
        return false;
    }

    public function getCommandUsages(): array
    {
        return [];
    }
}
