<?php

namespace idoit\Module\shd_mview_connect\Console\Command;

use Exception;
use idoit\Console\Command\AbstractCommand;
use SHD\Smview\Connect\DAO\ProfileAssignmentDAO;
use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\DAO\ProfileObjectTypesDAO;
use SHD\Smview\Connect\DAO\ProfileSettingsDAO;
use SHD\Smview\Connect\Service\Baramundi\BaramundiConfiguration;
use isys_exception_dao;
use isys_exception_database;
use isys_exception_general;
use isys_module_shd_smview_connect;
use Symfony\Component\Console\Input\InputDefinition;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

class TestDataGeneratorCommand extends AbstractCommand
{
    public function getCommandName(): string
    {
        return 'smview_connect_test_data_generator';
    }

    public function getCommandDescription(): string
    {
        return 'Command to generate some SM-VIEW connect test data.';
    }

    public function getCommandDefinition(): InputDefinition
    {
        return new InputDefinition();
    }

    public function isConfigurable(): bool
    {
        return false;
    }

    public function getCommandUsages()
    {
        // TODO: Implement getCommandUsages() method.
    }

    /**
     * @param InputInterface $input
     * @param OutputInterface $output
     * @return int
     * @throws isys_exception_dao
     * @throws isys_exception_database
     * @throws isys_exception_general
     */
    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $this->addBaramundiFixtures();

        return 0;
    }

    /**
     * @return void
     * @throws isys_exception_dao
     * @throws isys_exception_database
     * @throws isys_exception_general
     * @throws Exception
     */
    private function addBaramundiFixtures()
    {
        BaramundiConfiguration::registerDI();

        $profileDao = ProfileDAO::getInstance();
        $assignmentDao = ProfileAssignmentDAO::getInstance();
        $setingsDao = ProfileSettingsDAO::getInstance();
        $typeDao = ProfileObjectTypesDAO::getInstance();

        $profileId = $profileDao->createProfile(isys_module_shd_smview_connect::CONNECT_MODULE_BARAMUNDI, 'DEMO', '', true, true, 1);

        $setingsDao->updateSettings($profileId, [
            [
                'key' => BaramundiConfiguration::SETTING_BARAMUNDI_URL,
                'value' => 'https://shdsepm.shd-online.de/bConnect/v1.0',
            ],
            [
                'key' => BaramundiConfiguration::SETTING_BARAMUNDI_USERNAME,
                'value' => 'brmservice@shd-online.de',
            ],
        ]);

        $typeDao->updateTypeRelations($profileId, [[
            'property_key' => BaramundiConfiguration::PROP_PC_TYPE_ID,
            'object_type_id' => 10,
        ]]);

        $assignmentDao->updateProfileAssignments($profileId, ProfileAssignmentDAO::ASSIGNMENT_TYPE_MATCHING, [[
            'cat_property' => 'C__CATG__GLOBAL-title',
            'source_field' => BaramundiConfiguration::ATTRIBUTE_BARAMUNDI_ENDPOINT_TITLE,
            'meta1' => BaramundiConfiguration::PROP_PC_TYPE_ID,
        ]]);
    }
}
