<?php

namespace SHD\Smview\Connect\DAO;

use Exception;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use isys_application;
use isys_component_dao_result;
use isys_exception_dao;
use isys_exception_database;
use isys_module_dao;

class CustomFunctionDAO extends isys_module_dao
{
    /**
     * This DAO.
     *
     * @var CustomFunctionDAO
     */
    private static $__instance;

    /**
     * Table name of this DAO.
     *
     * @var string
     */
    private $_table_name = 'shd_smview_connect_custom_functions';

    /**
     * Get instance.
     *
     * @return CustomFunctionDAO
     * @throws Exception
     */
    public static function getInstance(): CustomFunctionDAO
    {
        if (!self::$__instance) {
            self::$__instance = new self(isys_application::instance()->container->get('database'));
        }
        return self::$__instance;
    }

    /**
     * @param string|null $fn
     * @return isys_component_dao_result
     * @throws isys_exception_database
     */
    public function get_data(string $fn = null)
    {
        $where = ' WHERE TRUE ';
        if ($fn) {
            $where .= " AND {$this->_table_name}__fn = {$this->convert_sql_text($fn)} ";
        }

        return $this->retrieve(
            <<<SQL
SELECT
       {$this->_table_name}__id `id`,
       {$this->_table_name}__fn `key`,
       {$this->_table_name}__code `value`
FROM $this->_table_name as cf
$where;
SQL
        );
    }

    /**
     * Update/Create Functions and delete any that are not there anymore
     *
     * @param array $definitions
     * @return bool
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function saveDefinitions(array $definitions): bool
    {
        error_reporting(E_ALL);

        $_defs = DaoUtil::allRows($this->get_data());
        $ids = array_map(function ($def) {
            return $def['id'];
        }, $_defs);

        foreach ($definitions as $definition) {
            $id = $definition['id'];

            if(intval($id) < 0) {
                //create
                $this->update(
                    <<<SQL
insert into {$this->_table_name} ({$this->_table_name}__fn, {$this->_table_name}__code) values (
    {$this->convert_sql_text($definition['key'])},
   {$this->convert_sql_text($definition['value'])}
)
SQL
                ) && $this->apply_update();

                $id = $this->get_last_insert_id();
            } else {
                //update
                $this->update(
                    <<<SQL
update {$this->_table_name}
set {$this->_table_name}__fn = {$this->convert_sql_text($definition['key'])}, {$this->_table_name}__code = {$this->convert_sql_text($definition['value'])}
where {$this->_table_name}__id = {$this->convert_sql_id($id)}
SQL
                ) && $this->apply_update();
            }

            if(in_array($id, $ids)) {
                $i = array_search($id, $ids);
                unset($ids[$i]);
            }
        }

        $_ids = implode(', ', $ids);

        if(strlen($_ids) > 0) {
            $this->update(
                <<<SQL
    delete from {$this->_table_name} where {$this->_table_name}__id in ({$_ids})
    SQL
            ) && $this->apply_update();
        }

        return true;
    }
}
