<?php

namespace SHD\Smview\Connect\DAO;

use isys_application;

class DataMappingDAO extends \isys_module_dao
{
    /**
     * This instance.
     *
     * @var DataMappingDAO
     */
    private static $__instance;

    /**
     * Table name of this DAO.
     *
     * @var string
     */
    private $_table_name = 'shd_smview_connect_data_mapping';

    /**
     * Get instance.
     *
     * @return DataMappingDAO
     */
    public static function getInstance(): DataMappingDAO
    {
        if (!self::$__instance) {
            self::$__instance = new self(isys_application::instance()->database);
        }
        return self::$__instance;
    }

    public function get_data($type = null, $source = null)
    {
        $where = '';

        if($type) {
            $where = "WHERE {$this->_table_name}__type = {$this->convert_sql_text($type)}";

            if($source) {
                $where .= " and {$this->_table_name}__source = {$this->convert_sql_text($source)}";
            }
        }

        $sql = <<<SQL
SELECT
    {$this->_table_name}__id `id`,
    {$this->_table_name}__type `type`,
    {$this->_table_name}__source `source`,
    {$this->_table_name}__target `target`
FROM $this->_table_name
$where
SQL;

        return $this->retrieve($sql);
    }

    public function createMappings($mappings)
    {
        foreach ($mappings as $mapping) {
            $this->update(
                <<<SQL
INSERT INTO $this->_table_name(
        {$this->_table_name}__type,
        {$this->_table_name}__source,
        {$this->_table_name}__target
)
VALUES (
        {$this->convert_sql_text($mapping['type'])},
        {$this->convert_sql_text($mapping['source'])},
        {$this->convert_sql_text($mapping['target'])}
)
SQL
            ) && $this->apply_update();
        }

        return true;
    }

    public function updateMappings($mappings)
    {
        $this->deleteMappings();
        $this->createMappings($mappings);
    }

    public function deleteMappings()
    {
        return $this->update(
            <<<SQL
DELETE FROM $this->_table_name
SQL
        ) && $this->apply_update();
    }
}
