<?php

namespace SHD\Smview\Connect\DAO;

use Exception;
use isys_application;
use isys_component_dao_result;
use isys_exception_dao;
use isys_module_dao;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class ProfileDiscoveryDataDAO extends isys_module_dao
{
    public const DATA_TYPE_OBJECT_TYPE = 'object_type';

    /**
     * This DAO.
     *
     * @var ProfileDiscoveryDataDAO
     */
    private static $__instance;

    /**
     * Table name of this DAO.
     *
     * @var string
     */
    private string $_table_name = 'shd_smview_connect_profile_discovery';

    /**
     * Get instance.
     *
     * @return ProfileDiscoveryDataDAO
     * @throws Exception
     */
    public static function getInstance(): ProfileDiscoveryDataDAO
    {
        if (!self::$__instance) {
            self::$__instance = new self(isys_application::instance()->container->get('database'));
        }
        return self::$__instance;
    }

    public function get_data($profileId = null, $type = null, $key = null): isys_component_dao_result
    {
        $where = ' WHERE TRUE ';
        if ($profileId) {
            $where .= " AND {$this->_table_name}__profile__id = {$this->convert_sql_id($profileId)} ";
        }

        if ($key) {
            $where .= " AND {$this->_table_name}__key = {$this->convert_sql_text($key)} ";
        }

        if ($type) {
            $where .= " AND {$this->_table_name}__type = {$this->convert_sql_text($type)} ";
        }

        return $this->retrieve(
            <<<SQL
SELECT
       {$this->_table_name}__id `id`,
       {$this->_table_name}__profile__id `profile_id`,
       {$this->_table_name}__type `type`,
       {$this->_table_name}__key `key`,
       {$this->_table_name}__value `value`
FROM $this->_table_name as p
$where;
SQL
        );
    }

    /**
     * @param int $profileId
     * @param array $data
     * @return void
     * @throws isys_exception_dao
     */
    public function createDiscoveryData(int $profileId, array $data): void
    {
        foreach ($data as $disco) {
            $key = $disco['key'];
            $type = $disco['type'];
            $value = $disco['value'];

            $this->update(
                <<<SQL
INSERT INTO $this->_table_name (
    {$this->_table_name}__profile__id,
    {$this->_table_name}__type,
    {$this->_table_name}__key,
    {$this->_table_name}__value
)
VALUES (
    {$this->convert_sql_id($profileId)},
    {$this->convert_sql_text($type)},
    {$this->convert_sql_text($key)},
    {$this->convert_sql_text($value)}
)
SQL
            ) && $this->apply_update();
        }
    }

    /**
     * @param int $profileId
     * @param string|null $type
     * @return void
     * @throws isys_exception_dao
     */
    public function deleteDiscoveryData(int $profileId, ?string $type = null): void
    {
        $where = ' WHERE TRUE ';
        if ($profileId) {
            $where .= " AND {$this->_table_name}__profile__id = {$this->convert_sql_id($profileId)} ";
        }

        if ($type) {
            $where .= " AND {$this->_table_name}__type = {$this->convert_sql_text($type)} ";
        }

        $this->update(
            <<<SQL
DELETE FROM $this->_table_name
$where;
SQL
        ) && $this->apply_update();
    }

    /**
     * @param int $profileId
     * @param array $data
     * @param string|null $type
     * @return void
     * @throws isys_exception_dao
     */
    public function updateDiscoveryData(int $profileId, array $data, ?string $type = null): void
    {
        $this->deleteDiscoveryData($profileId, $type);
        $this->createDiscoveryData($profileId, $data);
    }
}
