<?php

namespace SHD\Smview\Connect\DAO;

use Exception;
use isys_application;
use isys_component_dao_result;
use isys_exception_dao;
use isys_module_dao;

/**
 * i-doit
 *
 * Profile Object Types DAO for SM-VIEW Connect.
 *
 * @package     i-doit
 * @subpackage  smview_connect
 * @author      Florian Weber <florian.weber@shd-online.de>
 * @copyright   2022 SHD System-Haus-Dresden GmbH
 * @license     http://www.i-doit.com/license
 */
class ProfileObjectTypesDAO extends isys_module_dao
{
    /**
     * This DAO.
     *
     * @var ProfileObjectTypesDAO
     */
    private static $__instance;

    /**
     * Table name of this DAO.
     *
     * @var string
     */
    private $_table_name = 'shd_smview_connect_profile_object_types';

    /**
     * Get instance.
     *
     * @return ProfileObjectTypesDAO
     * @throws Exception
     */
    public static function getInstance(): ProfileObjectTypesDAO
    {
        if (!self::$__instance) {
            self::$__instance = new self(isys_application::instance()->container->get('database'));
        }
        return self::$__instance;
    }

    public function get_data(int $profile_id = null, string $propertyKey = null): isys_component_dao_result
    {
        $where = ' WHERE TRUE ';
        if ($profile_id) {
            $where .= " AND {$this->_table_name}__profile__id = {$this->convert_sql_id($profile_id)} ";
        }

        if ($propertyKey) {
            $where .= " AND {$this->_table_name}__property_key = {$this->convert_sql_text($propertyKey)} ";
        }

        return $this->retrieve(
            <<<SQL
SELECT
       {$this->_table_name}__id `id`,
       {$this->_table_name}__profile__id `profile_id`,
       {$this->_table_name}__property_key `property_key`,
       {$this->_table_name}__isys_obj_type__id `object_type_id`
FROM $this->_table_name as p
$where;
SQL
        );
    }

    /**
     * @param int $profile_id
     * @param array $mappings
     * @return void
     * @throws isys_exception_dao
     */
    public function createTypeRelations(int $profile_id, array $mappings): void
    {
        foreach ($mappings as $mapping) {
            $propertyKey = $mapping['property_key'];
            $objTypeId = $mapping['object_type_id'];

            //todo: check if already exists

            $this->update(
                <<<SQL
INSERT INTO $this->_table_name (
    {$this->_table_name}__profile__id,
    {$this->_table_name}__property_key,
    {$this->_table_name}__isys_obj_type__id
)
VALUES (
    {$this->convert_sql_id($profile_id)},
    {$this->convert_sql_text($propertyKey)},
    {$this->convert_sql_id($objTypeId)}
)
SQL
            ) && $this->apply_update();
        }
    }

    /**
     * @param int $profile_id
     * @param array $mappings
     * @return void
     * @throws isys_exception_dao
     */
    public function updateTypeRelations(int $profile_id, array $mappings): void
    {
        $this->deleteTypeRelations($profile_id);
        $this->createTypeRelations($profile_id, $mappings);
    }

    /**
     * @param int $profile_id
     * @return void
     * @throws isys_exception_dao
     */
    public function deleteTypeRelations(int $profile_id): void
    {
        $this->update(
            <<<SQL
DELETE FROM $this->_table_name
WHERE {$this->_table_name}__profile__id = {$this->convert_sql_id($profile_id)}
SQL
        ) && $this->apply_update();
    }

    /**
     * Duplicate the object type settings to another profile
     *
     * @param int $currentProfileId
     * @param int $newProfileId
     * @return void
     * @throws isys_exception_dao
     */
    public function duplicate(int $currentProfileId, int $newProfileId)
    {
        $this->update(
                <<<SQL
insert into $this->_table_name (
    {$this->_table_name}__profile__id,
    {$this->_table_name}__property_key,
    {$this->_table_name}__isys_obj_type__id
)
select  {$this->convert_sql_id($newProfileId)} as {$this->_table_name}__profile__id
    ,   {$this->_table_name}__property_key
    ,   {$this->_table_name}__isys_obj_type__id
from    $this->_table_name
where   {$this->_table_name}__profile__id = {$this->convert_sql_id($currentProfileId)}
SQL
            ) && $this->apply_update();
    }
}
