<?php

namespace SHD\Smview\Connect\DAO;

use Exception;
use isys_application;
use isys_component_dao_result;
use isys_exception_dao;
use isys_exception_database;
use isys_module_dao;

/**
 * i-doit
 *
 * Profile Settings DAO for SM-VIEW Connect.
 *
 * @package     i-doit
 * @subpackage  smview_connect
 * @author      Florian Weber <florian.weber@shd-online.de>
 * @copyright   2023 SHD System-Haus-Dresden GmbH
 * @license     http://www.i-doit.com/license
 */
class ProfileSettingsDAO extends isys_module_dao
{
    /**
     * This DAO.
     *
     * @var ProfileObjectTypesDAO
     */
    private static $__instance;

    /**
     * Table name of this DAO.
     *
     * @var string
     */
    private $_table_name = 'shd_smview_connect_profile_settings';

    /**
     * Get instance.
     *
     * @return ProfileSettingsDAO
     * @throws Exception
     */
    public static function getInstance(): ProfileSettingsDAO
    {
        if (!self::$__instance) {
            self::$__instance = new self(isys_application::instance()->container->get('database'));
        }
        return self::$__instance;
    }

    /**
     * @param int|null $profile_id
     * @param string|null $settingKey
     * @return isys_component_dao_result
     * @throws isys_exception_database
     */
    public function get_data(int $profile_id = null, string $settingKey = null): isys_component_dao_result
    {
        $where = ' WHERE TRUE ';
        if ($profile_id) {
            $where .= " AND {$this->_table_name}__profile__id = {$this->convert_sql_id($profile_id)} ";
        }

        if ($settingKey) {
            $where .= " AND {$this->_table_name}__key = {$this->convert_sql_text($settingKey)} ";
        }

        return $this->retrieve(
            <<<SQL
SELECT
       {$this->_table_name}__id `id`,
       {$this->_table_name}__profile__id `profile_id`,
       {$this->_table_name}__key `key`,
       {$this->_table_name}__value `value`
FROM $this->_table_name as p
$where;
SQL
        );
    }

    /**
     * @param int $profile_id
     * @param array $settings
     * @return void
     * @throws isys_exception_dao
     */
    public function createSettings(int $profile_id, array $settings): void
    {
        foreach ($settings as $setting) {
            $key = $setting['key'];
            $value = $setting['value'];

            $this->update(
                <<<SQL
INSERT INTO $this->_table_name (
    {$this->_table_name}__profile__id,
    {$this->_table_name}__key,
    {$this->_table_name}__value
)
VALUES (
    {$this->convert_sql_id($profile_id)},
    {$this->convert_sql_text($key)},
    {$this->convert_sql_text($value)}
)
SQL
            ) && $this->apply_update();
        }
    }

    /**
     * @param int $profile_id
     * @param array $settings
     * @return void
     * @throws isys_exception_dao
     */
    public function updateSettings(int $profile_id, array $settings): void
    {
        $this->deleteSettings($profile_id);
        $this->createSettings($profile_id, $settings);
    }

    /**
     * @param int $profile_id
     * @return void
     * @throws isys_exception_dao
     */
    public function deleteSettings(int $profile_id): void
    {
        $this->update(
            <<<SQL
DELETE FROM $this->_table_name
WHERE {$this->_table_name}__profile__id = {$this->convert_sql_id($profile_id)}
SQL
        ) && $this->apply_update();
    }

    /**
     * Duplicate the settings to another profile
     *
     * @param int $currentProfileId
     * @param int $newProfileId
     * @return void
     * @throws isys_exception_dao
     */
    public function duplicate(int $currentProfileId, int $newProfileId)
    {
        $this->update(
            <<<SQL
insert into $this->_table_name (
    {$this->_table_name}__profile__id,
    {$this->_table_name}__key,
    {$this->_table_name}__value
)
select  {$this->convert_sql_id($newProfileId)} as {$this->_table_name}__profile__id
    ,   {$this->_table_name}__key
    ,   {$this->_table_name}__value
from    $this->_table_name
where   {$this->_table_name}__profile__id = {$this->convert_sql_id($currentProfileId)}
SQL
        ) && $this->apply_update();
    }
}
