<?php

namespace SHD\Smview\Connect\Domain\Azure\Type\Compute;

use SHD\Smview\Connect\Domain\Azure\AzureResource;
use SHD\Smview\Connect\Domain\Azure\GenericResourceType;
use SHD\Smview\Connect\Domain\Azure\idoit\AzureRelationCategoryData;
use SHD\Smview\Connect\Domain\idoit\HostAddressCategoryData;
use SHD\Smview\Connect\Domain\idoit\IdoitCategoryDataInterface;
use SHD\Smview\Connect\Domain\idoit\OperatingSystemCategoryData;
use SHD\Smview\Connect\Domain\idoit\TransferObjectInterface;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class VirtualMachinesType extends GenericResourceType
{
    /**
     * @param string $class
     * @param AzureResource $object
     * @return array<IdoitCategoryDataInterface>|null
     */
    public function getCategoryDataDefintition(string $class, TransferObjectInterface $object): ?array
    {
        switch ($class) {
            case OperatingSystemCategoryData::class:
                return [new OperatingSystemCategoryData([
                    OperatingSystemCategoryData::TITLE => $object->rawProperties->storageProfile->osDisk->osType,
                    OperatingSystemCategoryData::VERSION => $object->rawProperties->storageProfile->imageReference->exactVersion,
                ])];
            case HostAddressCategoryData::class:
                if(count($object->networkInterfaces) == 0) {
                    return null;
                }

                $data = [];
                foreach ($object->networkInterfaces as $networkInterface) {
                    foreach ($networkInterface->ipAddresses as $ipAddress) {
                        $cd = [
                            HostAddressCategoryData::IP => $ipAddress->rawProperties->ipAddress,
                        ];

                        if($ipAddress->rawProperties->publicIPAddressVersion == 'IPv4') {
                            $cd[HostAddressCategoryData::VERSION] = HostAddressCategoryData::IPV4;
                        } elseif($ipAddress->rawProperties->publicIPAddressVersion == 'IPv6') {
                            $cd[HostAddressCategoryData::VERSION] = HostAddressCategoryData::IPV6;
                        }

                        if(@$ipAddress->rawProperties->dnsSettings->fqdn != null) {
                            $cd[HostAddressCategoryData::HOSTNAME] = $ipAddress->rawProperties->dnsSettings->domainNameLabel;
                            $domain = str_replace(
                                $ipAddress->rawProperties->dnsSettings->domainNameLabel.'.',
                                '',
                                $ipAddress->rawProperties->dnsSettings->fqdn
                            );
                            $cd[HostAddressCategoryData::DOMAIN] = $domain;
                            $cd[HostAddressCategoryData::FQDN] = $ipAddress->rawProperties->dnsSettings->fqdn;
                        }

                        $data[] = new HostAddressCategoryData($cd);
                    }
                }

                return $data;
            case AzureRelationCategoryData::class:
                $data = [];

                foreach ($object->networkInterfaces as $networkInterface) {
                    $data[] = new AzureRelationCategoryData([
                        AzureRelationCategoryData::SYS_ID_REL => 'MSAZ_'.sha1($networkInterface->id),
                    ]);

                    foreach ($networkInterface->ipAddresses as $ipAddress) {
                        $data[] = new AzureRelationCategoryData([
                            AzureRelationCategoryData::SYS_ID_REL => 'MSAZ_'.sha1($ipAddress->id),
                        ]);
                    }
                }

                foreach ($object->disks as $disk) {
                    $data[] = new AzureRelationCategoryData([
                        AzureRelationCategoryData::SYS_ID_REL => 'MSAZ_'.sha1($disk->id),
                    ]);
                }

                return $data;
            default:
                return null;
        }
    }
}
