<?php

namespace SHD\Smview\Connect\Domain\Azure\idoit;

use Exception;
use SHD\Smview\Connect\Domain\idoit\IdoitCategoryData;
use SHD\Smview\Connect\Domain\idoit\IdoitCategoryDataInterface;
use SHD\Smview\Connect\Domain\idoit\IdoitObject;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use isys_exception_dao;
use isys_exception_database;
use isys_exception_validation;
use isys_import_handler_cmdb;
use isys_smview_connect_cmdb_dao_g_azure_metadata;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class AzureMetadataCategoryData extends IdoitCategoryData
{
    public const AZURE_ID = 'azure_id';
    public const AZURE_TYPE = 'type';
    public const AZURE_DETAILS = 'details';

    /**
     * @param IdoitObject $item
     * @return IdoitCategoryDataInterface
     * @throws isys_exception_validation
     * @throws isys_exception_dao
     * @throws isys_exception_database
     * @throws Exception
     */
    public function syncDataTo(IdoitObject $item): IdoitCategoryDataInterface
    {
        $dao = isys_smview_connect_cmdb_dao_g_azure_metadata::instance($this->getDatabase());

        $props = [
            self::AZURE_ID => ['value' => $this->data[self::AZURE_ID]],
            self::AZURE_TYPE => ['value' => $this->data[self::AZURE_TYPE]],
        ];

        $cid = DaoUtil::getSingleValueCategoryDataId($dao->table, $item->getId());

        if($cid > 0) {
            $mode = isys_import_handler_cmdb::C__UPDATE;
        } else {
            $mode = isys_import_handler_cmdb::C__CREATE;
        }

        $cid = $dao->sync([
            'data_id' => $cid,
            'properties' => $props,
        ], $item->getId(), $mode);

        //json daten speichern
        if(is_int($cid) && $cid > 0) {
            $dao->setDetails($cid, $this->data[self::AZURE_DETAILS]);
        }

        return $this;
    }
}
