<?php

namespace SHD\Smview\Connect\Domain\Telekom\idoit;

use SHD\Smview\Connect\Domain\idoit\IdoitCategoryData;
use SHD\Smview\Connect\Domain\idoit\IdoitCategoryDataInterface;
use SHD\Smview\Connect\Domain\idoit\IdoitObject;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use isys_exception_database;
use isys_exception_validation;
use isys_import_handler_cmdb;
use isys_smview_connect_cmdb_dao_g_telekom_sim_meta;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class TelekomSimMetadataCategoryData extends IdoitCategoryData
{
    public const SIM_ID = 'sim_id';
    public const CARD_TYPE = 'card_type';
    public const ESIM = 'esim';
    public const MODE = 'mode';
    public const RINGS = 'rings';
    public const LAST_SEEN = 'last_seen';

    /**
     * @param IdoitObject $item
     * @return IdoitCategoryDataInterface
     * @throws isys_exception_database
     * @throws isys_exception_validation
     */
    public function syncDataTo(IdoitObject $item): IdoitCategoryDataInterface
    {
        $dao = isys_smview_connect_cmdb_dao_g_telekom_sim_meta::instance($this->getDatabase());

        $props = [
            self::SIM_ID => ['value' => $this->data[self::SIM_ID]],
            self::CARD_TYPE => ['value' => $this->data[self::CARD_TYPE]],
            self::ESIM => ['value' => $this->data[self::ESIM]],
            self::MODE => ['value' => $this->data[self::MODE]],
            self::RINGS => ['value' => $this->data[self::RINGS]],
            self::LAST_SEEN => ['value' => $this->data[self::LAST_SEEN]],
        ];

        $cid = DaoUtil::getSingleValueCategoryDataId($dao->table, $item->getId());

        if($cid > 0) {
            $mode = isys_import_handler_cmdb::C__UPDATE;
        } else {
            $mode = isys_import_handler_cmdb::C__CREATE;
        }

        $dao->sync([
            'data_id' => $cid,
            'properties' => $props,
        ], $item->getId(), $mode);

        return $this;
    }
}
