<?php

namespace SHD\Smview\Connect\Domain\Vsphere\idoit;

use SHD\Smview\Connect\Domain\idoit\IdoitCategoryData;
use SHD\Smview\Connect\Domain\idoit\IdoitCategoryDataInterface;
use SHD\Smview\Connect\Domain\idoit\IdoitObject;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use isys_exception_database;
use isys_exception_validation;
use isys_import_handler_cmdb;
use isys_smview_connect_cmdb_dao_g_vsphere_vm_metadata;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class VsphereVmMetadataCategoryData extends IdoitCategoryData
{
    public const VM_ID = 'vm_id';
    public const NAME = 'name';
    public const OS = 'os';
    public const OS_FAMILY = 'os_family';

    /**
     * @param IdoitObject $item
     * @return IdoitCategoryDataInterface
     * @throws isys_exception_database
     * @throws isys_exception_validation
     */
    public function syncDataTo(IdoitObject $item): IdoitCategoryDataInterface
    {
        $dao = isys_smview_connect_cmdb_dao_g_vsphere_vm_metadata::instance($this->getDatabase());

        $props = [
            self::VM_ID => ['value' => $this->data[self::VM_ID]],
            self::NAME => ['value' => $this->data[self::NAME]],
            self::OS => ['value' => $this->data[self::OS]],
            self::OS_FAMILY => ['value' => $this->data[self::OS_FAMILY]],
        ];

        $cid = DaoUtil::getSingleValueCategoryDataId($dao->table, $item->getId());

        if($cid > 0) {
            $mode = isys_import_handler_cmdb::C__UPDATE;
        } else {
            $mode = isys_import_handler_cmdb::C__CREATE;
        }

        $dao->sync([
            'data_id' => $cid,
            'properties' => $props,
        ], $item->getId(), $mode);

        return $this;
    }
}
