<?php

namespace SHD\Smview\Connect\Domain\idoit;

use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Mapping\Definition\ApplicationMappingDefinition;
use SHD\Smview\Connect\Service\Mapping\Definition\PersonMappingDefinition;
use SHD\Smview\Connect\Service\Mapping\Mapper;
use SHD\Smview\Connect\Service\Mapping\Transformer\ApplicationMappingTransformer;
use SHD\Smview\Connect\Service\Mapping\Transformer\PersonMappingTransformer;
use isys_cmdb_dao_category_g_application;
use isys_cmdb_dao_object;
use isys_exception_dao;
use isys_exception_database;
use isys_exception_general;
use isys_import_handler_cmdb;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class ApplicationCategoryData extends IdoitCategoryData
{
    public const APP_NAME = 'app_name';
    public const VERSION = 'version';

    /**
     * @param IdoitObject $item
     * @return IdoitCategoryDataInterface
     * @throws isys_exception_dao
     * @throws isys_exception_database
     * @throws isys_exception_general
     */
    public function syncDataTo(IdoitObject $item): IdoitCategoryDataInterface
    {
        $dao = isys_cmdb_dao_object::instance($this->getDatabase());
        $daoApp = isys_cmdb_dao_category_g_application::instance($this->getDatabase());

        /** @var ApplicationMappingDefinition $app */
        $app = (new Mapper(ApplicationMappingTransformer::class))->get(
            new ApplicationMappingDefinition($this->data[self::APP_NAME]),
        );

        $idoitApp = DaoUtil::allRows($dao->get_object($app->appId, false, null, 2))[0] ?? null;

        if(!$idoitApp) {
            // damit apps überhaupt gesynced werden müssen die in der cmdb bereits existieren, es macht also keinen sinn hier eine anzulegen
            return $this;
        }

        $daoApp->set_object_id($item->getId());

        $catdata = new \isys_array([]);
        $daoApp->category_data($catdata, C__RECORD_STATUS__NORMAL);

        $importType = isys_import_handler_cmdb::C__CREATE;
        $dataId = null;

        $dataExists = false;
        if(count(DaoUtil::allRows($daoApp->get_data(null, $item->getId()))) > 0) {
            $dataExists = true;
        }

        if($dataExists) {
            foreach ($catdata as $key => $entry) {
                if($entry['application']->m_id == $app->appId && $entry['assigned_version']->m_value == $this->data[self::VERSION]) {
                    $dataId = $key;
                    $importType = isys_import_handler_cmdb::C__UPDATE;

                    break;
                }
            }
        }

        //data
        $daoApp->sync([
            'data_id' => $dataId,
            'properties' => [
                'application' => ['value' => $app->appId],
                'assigned_version' => ['value' => $this->data[self::VERSION]],
            ],
        ], $item->getId(), $importType);

        return $this;
    }
}
