<?php

namespace SHD\Smview\Connect\Domain\idoit;

use Exception;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Mapping\Definition\PersonMappingDefinition;
use SHD\Smview\Connect\Service\Mapping\Mapper;
use SHD\Smview\Connect\Service\Mapping\Transformer\PersonMappingTransformer;
use isys_cmdb_dao_category_g_contact;
use isys_import_handler_cmdb;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class ContactCategoryData extends IdoitCategoryData
{
    public const EMAIL = 'email';

    /**
     * @param IdoitObject $item
     * @return IdoitCategoryDataInterface
     * @throws Exception
     */
    public function syncDataTo(IdoitObject $item): IdoitCategoryDataInterface
    {
        $dao = isys_cmdb_dao_category_g_contact::instance($this->getDatabase());
        $dao->set_object_id($item->getId());

        /** @var PersonMappingDefinition $person */
        $person = (new Mapper(PersonMappingTransformer::class))->get(
            new PersonMappingDefinition($this->data[self::EMAIL]),
        );

        if($person->personId == null) {
            return $this;
        }

        $catdata = new \isys_array([]);
        $dao->category_data($catdata, C__RECORD_STATUS__NORMAL);

        $importType = isys_import_handler_cmdb::C__CREATE;
        $dataId = null;

        $dataExists = false;
        if(count(DaoUtil::allRows($dao->get_data(null, $item->getId()))) > 0) {
            $dataExists = true;
        }

        if($dataExists) {
            $compValue = $person->personId;
            $compProp = 'contact';

            foreach ($catdata as $key => $entry) {
                if($entry[$compProp]->m_id == $compValue) {
                    $dataId = $key;
                    $importType = isys_import_handler_cmdb::C__UPDATE;

                    break;
                }
            }
        }

        $props = [];
        $cp = $dao->get_properties();

        if($dataId > 0 && $importType == isys_import_handler_cmdb::C__UPDATE) {
            //props mit daten angereichert holen
            //catdata enthält nur felder die schon werte haben, somit werden neue nicht gefüllt -> props holen und ggf werte anhängen
            $_data = [];
            foreach ($cp as $p => $info) {
                $_data[$p] = $catdata->get($dataId)[$p] ?? null;
            }

            //alles durchgehen und an der richtigen stelle mit neuem wert updaten
            foreach ($_data as $_prop_key => $_prop_value) {
                if($_prop_value->m_id != null && $cp[$_prop_key]['data']['type'] != 'text') {
                    $_val = $_prop_value->m_id;
                } else {
                    $_val = $_prop_value->m_value ?? null;
                }

                if(is_object($cp[$_prop_key])) { //ab mindestens idoit 26 ist das hier kein array mehr
                    if($cp[$_prop_key]->getData()->getType() == 'int' && count($cp[$_prop_key]->getData()->getReferences()) == 0) {
                        $_val = $_val == 'Yes' ? 1 : 0;
                    }
                } else {
                    if($cp[$_prop_key]['data']['type'] == 'int' && !array_key_exists('references', $cp[$_prop_key]['data'])) {
                        $_val = $_val == 'Yes' ? 1 : 0;
                    }
                }

                $props[$_prop_key] = ['value' => $_val];
            }

            $props['primary'] = ['value' => 1];
            $props['contact'] = ['value' => $person->personId];
        } else {
            $props['primary'] = ['value' => 1];
            $props['contact'] = ['value' => $person->personId];
            $props['role'] = ['value' => null];
        }

        $dao->sync([
            'data_id' => $dataId,
            'properties' => $props,
        ], $item->getId(), $importType);

        return $this;
    }
}
