<?php

namespace SHD\Smview\Connect\Domain\idoit;

use Exception;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use isys_cmdb_dao_category_g_ip;
use isys_exception_cmdb;
use isys_exception_dao;
use isys_exception_database;
use isys_exception_general;
use isys_import_handler_cmdb;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class HostAddressCategoryData extends IdoitCategoryData
{
    public const IPV4 = 'IPV4';
    public const IPV6 = 'IPV6';
    public const IP = 'ipv4_address';
    public const VERSION = 'version';
    public const HOSTNAME = 'hostname';
    public const DOMAIN = 'domain';
    public const FQDN = 'fqdn';
    public const ACTIVE = 'active';

    /**
     * @param IdoitObject $item
     * @return IdoitCategoryDataInterface
     * @throws isys_exception_general
     * @throws isys_exception_cmdb
     * @throws isys_exception_dao
     */
    public function syncDataTo(IdoitObject $item): IdoitCategoryDataInterface
    {
        //todo: implement ipv6 handling
        if($this->data[self::VERSION] == self::IPV6) {
            return $this;
        }

        $dao = isys_cmdb_dao_category_g_ip::instance($this->getDatabase());
        $dao->set_object_id($item->getId());

        $catdata = new \isys_array([]);
        $dao->category_data($catdata, C__RECORD_STATUS__NORMAL);

        $importType = isys_import_handler_cmdb::C__CREATE;
        $dataId = null;

        $dataExists = false;
        if(count(DaoUtil::allRows($dao->get_data(null, $item->getId()))) > 0) {
            $dataExists = true;
        }

        if($dataExists) {
            $compValue = $this->data[self::IP];
            $compProp = self::IP;

            if(is_null($compValue)) {
                $compValue = $this->data[self::HOSTNAME];
                $compProp = self::HOSTNAME;
            }

            foreach ($catdata as $key => $entry) {
                if($entry[$compProp] == $compValue) {
                    $dataId = $key;
                    $importType = isys_import_handler_cmdb::C__UPDATE;

                    break;
                }
            }
        }

        $props = [];
        $cp = $dao->get_properties();

        if($dataId > 0 && $importType == isys_import_handler_cmdb::C__UPDATE) {
            //props mit daten angereichert holen
            //catdata enthält nur felder die schon werte haben, somit werden neue nicht gefüllt -> props holen und ggf werte anhängen
            $_data = [];
            foreach ($cp as $p => $info) {
                $_data[$p] = $catdata->get($dataId)[$p] ?? null;
            }

            //alles durchgehen und an der richtigen stelle mit neuem wert updaten
            foreach ($_data as $_prop_key => $_prop_value) {
                if($_prop_value->m_id != null && $cp[$_prop_key]['data']['type'] != 'text') {
                    $_val = $_prop_value->m_id;
                } else {
                    $_val = $_prop_value->m_value ?? null;
                }

                if(is_object($cp[$_prop_key])) { //ab mindestens idoit 26 ist das hier kein array mehr
                    if($cp[$_prop_key]->getData()->getType() == 'int' && count($cp[$_prop_key]->getData()->getReferences()) == 0) {
                        $_val = $_val == 'Yes' ? 1 : 0;
                    }
                } else {
                    if($cp[$_prop_key]['data']['type'] == 'int' && !array_key_exists('references', $cp[$_prop_key]['data'])) {
                        $_val = $_val == 'Yes' ? 1 : 0;
                    }
                }

                $props[$_prop_key] = ['value' => $_val];
            }
        }

        $props[self::IP] = $this->data[self::IP] ? ['value' => $this->data[self::IP]] : $props[self::IP];
        $props[self::HOSTNAME] = $this->data[self::HOSTNAME] ? ['value' => $this->data[self::HOSTNAME]] : $props[self::HOSTNAME];
        $props[self::DOMAIN] = $this->data[self::DOMAIN] ? ['value' => $this->data[self::DOMAIN]] : $props[self::DOMAIN];

        if(!$this->data[self::ACTIVE]) $this->data[self::ACTIVE] = 1;
        $props[self::ACTIVE] = $this->data[self::ACTIVE] ? ['value' => $this->data[self::ACTIVE]] : $props[self::ACTIVE];

        //create oder update
        $dao->sync([
            'data_id' => $dataId,
            'properties' => $props,
        ], $item->getId(), $importType);

        return $this;
    }
}
