<?php

namespace SHD\Smview\Connect\Domain\idoit;

use Exception;
use isys_application;
use isys_component_dao;
use isys_component_database;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
abstract class IdoitCategoryData implements IdoitCategoryDataInterface
{
    /**
     * @var array
     */
    protected array $data;

    /**
     * @var isys_component_dao|null
     */
    protected ?isys_component_dao $dao;

    public function __construct(array $data, ?isys_component_dao $dao = null)
    {
        $this->data = $data;
        $this->dao = $dao;
    }

    /**
     * @return array
     */
    public function getData(): array
    {
        return $this->data;
    }

    /**
     * @return isys_component_database
     * @throws Exception
     */
    public function getDatabase(): isys_component_database
    {
        return isys_application::instance()->container->get('database');
    }

    protected function extractFilledData(array $propdef): array
    {
        $props = [];

        foreach ($propdef as $propKey => $valueKey) {
            if($this->data[$valueKey] == '' || $this->data[$valueKey] == null) {
                continue;
            }

            $props[$propKey] = ['value' => $this->data[$valueKey]];
        }

        return $props;
    }
}
