<?php

namespace SHD\Smview\Connect\Domain\idoit;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class IdoitImportCollection
{
    /**
     * @var array<IdoitObject>
     */
    private array $objects = [];

    /**
     * @param IdoitObject $object
     * @return $this
     */
    public function addObject(IdoitObject $object): IdoitImportCollection
    {
        if(in_array($object, $this->objects)) {
            return $this;
        }

        $this->objects[] = $object;

        return $this;
    }

    /**
     * @return array<IdoitObject>
     */
    public function getObjects(): array
    {
        return $this->objects;
    }

    /**
     * Export the Collection info as an array
     * @return array
     */
    public function export(): array
    {
        $obs = [];

        foreach($this->objects as $object) {
            $ob = [
                'title' => $object->getTitle(),
                'sys_id' => $object->getSysId(),
                'type_id' => $object->getTypeId(),
                'source' => $object->getSourceObject(),
            ];

            foreach ($object->getCategoryStack() as $stack) {
                /** @var IdoitCategoryData $stackItem */
                foreach ($stack as $stackItem) {
                    $ob['items'][] = [
                        'type' => get_class($stackItem),
                        'data' => $stackItem->getData(),
                    ];
                }
            }

            $obs[] = $ob;
        }

        return $obs;
    }
}
