<?php

namespace SHD\Smview\Connect\Domain\idoit;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class IdoitObject
{
    /**
     * @var int
     */
    private int $id;

    /**
     * @var string
     */
    private string $title;

    /**
     * @var string
     */
    private string $sysId;

    /**
     * @var int
     */
    private int $typeId;

    /**
     * @var array<array<IdoitCategoryDataInterface>>
     */
    private array $categoryStack = [];

    /**
     * @var TransferObjectInterface
     */
    private TransferObjectInterface $sourceObject;

    /**
     * @var array<DynamicCategoryData>
     */
    private array $dynamicCategoryStack = [];

    /**
     * @var array
     */
    private array $sourceAssignments = [];

    /**
     * @param string $title
     * @param string $sysId
     * @param int $typeId
     */
    public function __construct(string $title, string $sysId, int $typeId)
    {
        $this->title = $title;
        $this->sysId = $sysId;
        $this->typeId = $typeId;
    }

    /**
     * @return int
     */
    public function getId(): int
    {
        return $this->id;
    }

    /**
     * @param int $id
     * @return $this
     */
    public function setId(int $id): IdoitObject
    {
        $this->id = $id;

        return $this;
    }

    /**
     * @return string
     */
    public function getTitle(): string
    {
        return $this->title;
    }

    /**
     * @return string
     */
    public function getSysId(): string
    {
        return $this->sysId;
    }

    /**
     * @return int
     */
    public function getTypeId(): int
    {
        return $this->typeId;
    }

    /**
     * @return array<IdoitCategoryDataInterface>
     */
    public function getCategoryStack(): array
    {
        return $this->categoryStack;
    }

    /**
     * @param array<IdoitCategoryDataInterface> $value
     * @return IdoitObject
     */
    public function addCategoryData(array $value): IdoitObject
    {
        $this->categoryStack[] = $value;

        return $this;
    }

    /**
     * @return TransferObjectInterface
     */
    public function getSourceObject(): TransferObjectInterface
    {
        return $this->sourceObject;
    }

    /**
     * @param TransferObjectInterface $object
     * @return $this
     */
    public function setSourceObject(TransferObjectInterface $object): IdoitObject
    {
        $this->sourceObject = $object;

        return $this;
    }

    /**
     * @param DynamicCategoryData $data
     * @return IdoitObject
     */
    public function addDynamicCategoryData(DynamicCategoryData $data): IdoitObject
    {
        $this->dynamicCategoryStack[] = $data;

        return $this;
    }

    /**
     * @return DynamicCategoryData[]
     */
    public function getDynamicCategoryStack(): array
    {
        return $this->dynamicCategoryStack;
    }

    /**
     * @param string $key
     * @param mixed|null $value
     * @return $this
     */
    public function addSourceAssignment(string $key, $value): IdoitObject
    {
        $this->sourceAssignments[$key] = $value;

        return $this;
    }

    /**
     * @param string $key
     * @return mixed|null
     */
    public function getSourceAssignment(string $key)
    {
        return @$this->sourceAssignments[$key] ?? null;
    }

    /**
     * @return array
     */
    public function getSourceAssignments(): array
    {
        return $this->sourceAssignments;
    }
}
