<?php

namespace SHD\Smview\Connect\Domain\idoit;

use SHD\Smview\Connect\Service\DB\DaoUtil;
use isys_cmdb_dao_category_g_cards;
use isys_exception_database;
use isys_exception_validation;
use isys_import_handler_cmdb;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class IdoitSimCardCategoryData extends IdoitCategoryData
{
    public const TITLE = 'title';
    public const CARD_NUMBER = 'card_no';
    public const PIN1 = 'pin';
    public const PIN2 = 'pin2';
    public const PUK1 = 'puk';
    public const PUK2 = 'puk2';
    public const SERIAL = 'serial';
    public const DESCRIPTION = 'description';
    public const ASSIGNED_MOBILE_ID = 'assigned_mobile';

    public const TABLE = 'isys_catg_cards_list';

    /**
     * @param IdoitObject $item
     * @return IdoitCategoryDataInterface
     * @throws isys_exception_database
     * @throws isys_exception_validation
     */
    public function syncDataTo(IdoitObject $item): IdoitCategoryDataInterface
    {
        $dao = isys_cmdb_dao_category_g_cards::instance($this->getDatabase());
        //alle listeneinträge holen
        $listData = DaoUtil::allRows($dao->get_data(null, $item->getId()));

        $importType = isys_import_handler_cmdb::C__CREATE;
        $dataId = null;

        //den konkreten listeneintrag holen sollte einer da sein (wenn ja dann mode update)
        if(count($listData) > 0) {
            $entry = array_values(array_filter($listData, function ($ccd) {
                return $ccd['isys_catg_cards_list__card_number'] == $this->data[self::CARD_NUMBER];
            }))[0] ?? null;

            if(!is_null($entry)) {
                $dataId = $entry['isys_catg_cards_list__id'];
                $importType = isys_import_handler_cmdb::C__UPDATE;
            }
        }

        //erstmal die eingegebenen werte sammeln
        $values = [
            self::TITLE => $this->data[self::TITLE] ?? null,
            self::CARD_NUMBER => $this->data[self::CARD_NUMBER] ?? null,
            self::PIN1 => $this->data[self::PIN1] ?? null,
            self::PIN2 => $this->data[self::PIN2] ?? null,
            self::PUK1 => $this->data[self::PUK1] ?? null,
            self::PUK2 => $this->data[self::PUK2] ?? null,
            self::SERIAL => $this->data[self::SERIAL] ?? null,
            self::DESCRIPTION => $this->data[self::DESCRIPTION] ?? null,
            self::ASSIGNED_MOBILE_ID => $this->data[self::ASSIGNED_MOBILE_ID] ?? null,
        ];
        $props = [];

        //alles durchgehen -> wenn null werte da sind dann ggf. mit bestandsdaten ergänzen, ansonsten wird ggf ein feld genullt was eigentlich date hat (wenn bei der dateneingabe in die class etwas vergessen wird)
        foreach ($values as $key => $value) {
            if(is_null($value)) {
                //daten aus bestand holen oder null lassen
                switch ($key) {
                    case self::ASSIGNED_MOBILE_ID:
                        $value = $entry['isys_connection__isys_obj__id'] ?? null;
                        break;

                    default:
                        $field = DaoUtil::getFieldNameFromProperty($key, $dao->get_properties());
                        $value = $c[$field] ?? null;
                        break;
                }
            }

            $props[$key] = ['value' => $value];
        }

        //create oder update
        $dao->sync([
            'data_id' => $dataId,
            'properties' => $props,
        ], $item->getId(), $importType);

        return $this;
    }
}
