<?php

namespace SHD\Smview\Connect\Domain\idoit;

use Exception;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Mapping\Definition\ModelMappingDefinition;
use SHD\Smview\Connect\Service\Mapping\Mapper;
use SHD\Smview\Connect\Service\Mapping\Transformer\ModelMappingTransformer;
use isys_cmdb_dao_category_g_model;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class ModelCategoryData extends IdoitCategoryData
{
    public const MODEL = 'model';
    public const MANUFACTURER = 'manufacturer';
    public const SERIAL = 'serial';

    /**
     * @param IdoitObject $item
     * @return IdoitCategoryDataInterface
     * @throws Exception
     */
    public function syncDataTo(IdoitObject $item): IdoitCategoryDataInterface
    {
        $dao = isys_cmdb_dao_category_g_model::instance($this->getDatabase());

        /** @var ModelMappingDefinition $model */
        $model = (new Mapper(ModelMappingTransformer::class))->get(
            new ModelMappingDefinition(
                $this->data[self::MODEL],
                $this->data[self::MANUFACTURER],
            )
        );

        //wenn daten am model erfasst wurden die nicht aus baramundi kommen dann müssen die hier manuell nochmal geholt werden,
        //sonst werden die mit null überschrieben wenn sync() ausgeführt wird
        $idoitModelListData = DaoUtil::allRows($dao->get_data(null, $item->getId(), null, null, 2))[0] ?? null;

        $props = [
            'title' => ['value' => $model->titleId],
            'manufacturer' => ['value' => $model->manufacturerId],
            'serial' => ['value' => $this->data[self::SERIAL]],
        ];

        if($idoitModelListData != null) {
            $props = array_merge($props, [
                'productid' => ['value' => $idoitModelListData['isys_catg_model_list__productid']],
                'firmware' => ['value' => $idoitModelListData['isys_catg_model_list__firmware']],
                'description' => ['value' => $idoitModelListData['isys_catg_model_list__description']],
                'service_tag' => ['value' => $idoitModelListData['isys_catg_model_list__service_tag']],
            ]);
        }

        $dao->sync([
            'properties' => $props,
        ], $item->getId());

        return $this;
    }
}
