<?php

namespace SHD\Smview\Connect\Domain\idoit;

use Exception;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Mapping\Definition\OSMappingDefinition;
use SHD\Smview\Connect\Service\Mapping\Mapper;
use SHD\Smview\Connect\Service\Mapping\Transformer\OSMappingTransformer;
use isys_cmdb_dao_category_g_operating_system;
use isys_exception_database;
use isys_import_handler_cmdb;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class OperatingSystemCategoryData extends IdoitCategoryData
{
    public const TITLE = 'title';
    public const VERSION = 'version';

    /**
     * @param IdoitObject $item
     * @return IdoitCategoryDataInterface
     * @throws isys_exception_database
     * @throws Exception
     */
    public function syncDataTo(IdoitObject $item): IdoitCategoryDataInterface
    {
        $dao = isys_cmdb_dao_category_g_operating_system::instance($this->getDatabase());

        /** @var OSMappingDefinition $os */
        $os = (new Mapper(OSMappingTransformer::class))->get(
            new OSMappingDefinition(
                $this->data[self::TITLE],
            )
        );

        if($os->applicationId <= 0) {
            return $this;
        }

        $dao->set_object_id($item->getId());

        $catdata = new \isys_array([]);
        $dao->category_data($catdata, C__RECORD_STATUS__NORMAL);

        $importType = isys_import_handler_cmdb::C__CREATE;
        $dataId = null;

        $dataExists = false;
        if(count(DaoUtil::allRows($dao->get_data(null, $item->getId()))) > 0) {
            $dataExists = true;
        }

        if($dataExists) {
            foreach ($catdata as $key => $entry) {
                if($entry['application']->m_id == $os->applicationId && $entry['assigned_version'] == $this->data[self::VERSION]) {
                    $dataId = $key;
                    $importType = isys_import_handler_cmdb::C__UPDATE;

                    break;
                }
            }
        }

        $props = [];
        $cp = $dao->get_properties();

        if($dataId > 0 && $importType == isys_import_handler_cmdb::C__UPDATE) {
            //props mit daten angereichert holen
            //catdata enthält nur felder die schon werte haben, somit werden neue nicht gefüllt -> props holen und ggf werte anhängen
            $_data = [];
            foreach ($cp as $p => $info) {
                $_data[$p] = $catdata->get($dataId)[$p] ?? null;
            }

            //alles durchgehen und an der richtigen stelle mit neuem wert updaten
            foreach ($_data as $_prop_key => $_prop_value) {
                if($_prop_value->m_id != null && $cp[$_prop_key]['data']['type'] != 'text') {
                    $_val = $_prop_value->m_id;
                } else {
                    $_val = $_prop_value->m_value ?? null;
                }

                if(is_object($cp[$_prop_key])) { //ab mindestens idoit 26 ist das hier kein array mehr
                    if($cp[$_prop_key]->getData()->getType() == 'int' && count($cp[$_prop_key]->getData()->getReferences()) == 0) {
                        $_val = $_val == 'Yes' ? 1 : 0;
                    }
                } else {
                    if($cp[$_prop_key]['data']['type'] == 'int' && !array_key_exists('references', $cp[$_prop_key]['data'])) {
                        $_val = $_val == 'Yes' ? 1 : 0;
                    }
                }

                $props[$_prop_key] = ['value' => $_val];
            }
        }

        $props['application'] = ['value' => $os->applicationId];
        $props['assigned_version'] = ['value' => $this->data[self::VERSION]];

        $dao->sync([
            'data_id' => $dataId,
            'properties' => $props,
        ], $item->getId(), $importType);

        return $this;
    }
}
