<?php

namespace SHD\Smview\Connect\Service\Aeneis;

use SHD\Smview\Connect\Service\AbstractConfiguration;
use isys_application;
use isys_tenantsettings;

/**
 * Aeneis configuration class.
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class AeneisConfiguration extends AbstractConfiguration
{
    public const LICENSE_KEY = 'aeneis';


    public const SETTING_AENEIS_HOST_URL = 'aeneis.host_url';
    public const SETTING_AENEIS_PORTAL_NAME = 'aeneis.portal_name';
    public const SETTING_AENEIS_USER_NAME = 'aeneis.user_name';
    public const SETTING_AENEIS_PASSWORD = 'aeneis.password';
    public const SETTING_AENEIS_PROCESS_DIRECTORY = 'aeneis.process.directory';
    public const SETTING_AENEIS_PROCESS_GUID = 'aeneis.process.guid';
    public const SETTING_AENEIS_PROCESS_OBJECT_TYPE = 'aeneis.process.object_type';
    public const SETTING_AENEIS_IT_SERVICE_DIRECTORY = 'aeneis.it_service.directory';
    public const SETTING_AENEIS_IT_SERVICE_GUID = 'aeneis.it_service.guid';
    public const SETTING_AENEIS_IT_SERVICE_OBJECT_TYPE = 'aeneis.it_service.object_type';
    public const SETTING_AENEIS_EMPLOYEE_DIRECTORY = 'aeneis.employee.directory';
    public const SETTING_AENEIS_EMPLOYEE_GUID = 'aeneis.employee.guid';
    public const SETTING_AENEIS_EMPLOYEE_OBJECT_TYPE = 'aeneis.employee.object_type';

    public const AENEIS_FIELD_MAPPING = [
        self::SETTING_AENEIS_HOST_URL => [
            'ui_key'      => 'C__AENEIS__HOST_URL',
            'db_key'      => 'smview_connect.connect.aeneis_host_url',
            'placeholder' => 'https://URL:PORT',
            'type'        => 'string',
        ],
        self::SETTING_AENEIS_PORTAL_NAME => [
            'ui_key'      => 'C__AENEIS__PORTAL_NAME',
            'db_key'      => 'smview_connect.connect.aeneis_portal_name',
            'type'        => 'string',
        ],
        self::SETTING_AENEIS_USER_NAME => [
            'ui_key'      => 'C__AENEIS__USERNAME',
            'db_key'      => 'smview_connect.connect.aeneis_username',
            'type'        => 'string',
        ],
        self::SETTING_AENEIS_PASSWORD => [
            'ui_key'      => 'C__AENEIS__PASSWORD',
            'db_key'      => 'smview_connect.connect.aeneis_password',
            'type'        => 'password',
        ],
        self::SETTING_AENEIS_PROCESS_DIRECTORY => [
            'ui_key'      => 'C__AENEIS__GUID_PROCESS_DIRECTORY',
            'db_key'      => 'smview_connect.connect.aeneis_process.directory',
            'type'        => 'string',
        ],
        self::SETTING_AENEIS_PROCESS_GUID => [
            'ui_key'      => 'C__AENEIS__GUID_PROCESS',
            'db_key'      => 'smview_connect.connect.aeneis_process.guid',
            'type'        => 'string',
        ],
        self::SETTING_AENEIS_PROCESS_OBJECT_TYPE => [
            'ui_key'      => 'C__AENEIS__GUID_PROCESS_OBJECT_TYPE',
            'db_key'      => 'smview_connect.connect.aeneis_process.object_type',
            'type'        => 'id',
        ],
        self::SETTING_AENEIS_IT_SERVICE_DIRECTORY => [
            'ui_key'      => 'C__AENEIS__GUID_IT_SERVICE_DIRECTORY',
            'db_key'      => 'smview_connect.connect.aeneis_it_service.directory',
            'type'        => 'string',
        ],
        self::SETTING_AENEIS_IT_SERVICE_GUID => [
            'ui_key'      => 'C__AENEIS__GUID_IT_SERVICE',
            'db_key'      => 'smview_connect.connect.aeneis_it_service.guid',
            'type'        => 'string',
        ],
        self::SETTING_AENEIS_IT_SERVICE_OBJECT_TYPE => [
            'ui_key'      => 'C__AENEIS__GUID_IT_SERVICE_OBJECT_TYPE',
            'db_key'      => 'smview_connect.connect.aeneis_it_service.object_type',
            'type'        => 'id',
        ],
        self::SETTING_AENEIS_EMPLOYEE_DIRECTORY => [
            'ui_key'      => 'C__AENEIS__GUID_EMPLOYEE_DIRECTORY',
            'db_key'      => 'smview_connect.connect.aeneis_employee.directory',
            'type'        => 'string',
        ],
        self::SETTING_AENEIS_EMPLOYEE_GUID => [
            'ui_key'      => 'C__AENEIS__GUID_EMPLOYEE',
            'db_key'      => 'smview_connect.connect.aeneis_employee.guid',
            'type'        => 'string',
        ],
        self::SETTING_AENEIS_EMPLOYEE_OBJECT_TYPE => [
            'ui_key'      => 'C__AENEIS__GUID_EMPLOYEE_OBJECT_TYPE',
            'db_key'      => 'smview_connect.connect.aeneis_employee.object_type',
            'type'        => 'id',
        ],
    ];

    public static function registerDI(): void
    {
        if(!self::isEnabled()) return;

        $http = isys_application::instance()->container->get('smview_connect.http_client');

        $aeneisConfig = new AeneisConfiguration();
        isys_application::instance()->container->set('smview_connect.aeneis.config', $aeneisConfig);

        $aeneisImporter = new Importer(
            $http,
            $aeneisConfig,
        );
        isys_application::instance()->container->set('smview_connect.aeneis.importer', $aeneisImporter);

        $aeneisExporter = new Exporter(
            $http,
            $aeneisConfig,
        );
        isys_application::instance()->container->set('smview_connect.aeneis.exporter', $aeneisExporter);
    }

    /**
     * Get prepared rules for display in UI.
     *
     * @return array
     */
    public function getRules(): array
    {
        return self::get_rules_by_mapping(self::AENEIS_FIELD_MAPPING);
    }

    /**
     * @param string $key
     * @param $default
     * @return string|null
     */
    public function getConfig(string $key, $default = null): ?string
    {
        $config = self::AENEIS_FIELD_MAPPING[$key];
        $dbKey = $config['db_key'];
        return isys_tenantsettings::get($dbKey, $default);
    }

    /**
     * Save configuration to database.
     *
     * @param array $postedData
     */
    public function save_config(array $postedData)
    {
        self::save_config_by_mapping($postedData, self::AENEIS_FIELD_MAPPING);
    }

    /**
     * Export configuration to filesystem (not implemented).
     */
    public function export_config()
    {
    }
}
