<?php

namespace SHD\Smview\Connect\Service\Auth;

use isys_auth;
use isys_auth_dao;
use isys_auth_interface;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class IdoitAuth extends isys_auth implements isys_auth_interface
{
    /**
     * @var self
     */
    private static $m_instance = null;

    /**
     * Retrieve singleton instance of authorization class.
     *
     * @return self
     */
    public static function instance(): ?IdoitAuth
    {
        // If the DAO has not been loaded yet, we initialize it now.
        if (self::$m_dao === null) {
            global $g_comp_database;

            self::$m_dao = new isys_auth_dao($g_comp_database);
        }

        if (self::$m_instance === null) {
            self::$m_instance = new self;
        }

        return self::$m_instance;
    }

    /**
     * Method for returning the available auth-methods. This will be used for the GUI.
     *
     * @return  array
     */
    public function get_auth_methods(): array
    {
        return [
            'view'   => [
                'title'  => 'SHD SM-VIEW Connect',
                'type'   => 'boolean',
                'rights' => [
                    isys_auth::VIEW
                ]
            ]
        ];
    }

    /**
     * Get ID of related module.
     *
     * @return  integer
     */
    public function get_module_id(): int
    {
        return defined_or_default('C__MODULE__SHD_SMVIEW_CONNECT');
    }

    /**
     * Get title of related module.
     *
     * @return  string
     */
    public function get_module_title(): string
    {
        return "LC__CMDB__SMVIEW_CONNECT__TREE_TITLE";
    }
}
