<?php

namespace SHD\Smview\Connect\Service\Auth;

use Exception;
use isys_module_shd_smview_connect;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
abstract class TokenAuthenticator implements TokenAuthenticatorInterface
{
    /**
     * @param TokenInterface $token
     * @return TokenAuthenticatorInterface
     */
    protected function persist(TokenInterface $token): TokenAuthenticatorInterface
    {
        $path = isys_module_shd_smview_connect::getIdoitDir().'/temp/shd_smview_connect/token/'.$token->getName();
        $file = $token->getId().'.token';

        if(!is_dir($path)) {
            mkdir($path, 0777, true);
        }

        file_put_contents($path.'/'.$file, serialize($token));

        return $this;
    }

    /**
     * @param string $name
     * @param string $id
     * @return TokenInterface
     * @throws Exception
     */
    public static function restore(string $name, string $id): TokenInterface
    {
        $path = isys_module_shd_smview_connect::getIdoitDir().'/temp/shd_smview_connect/token/'.$name;
        $file = $id.'.token';

        if(!file_exists($path.'/'.$file)) {
            throw new Exception(sprintf('no token file %s found', $path.'/'.$file));
        }

        return unserialize(file_get_contents($path.'/'.$file));
    }

    public static function remove(string $name, string $id)
    {
        $path = isys_module_shd_smview_connect::getIdoitDir().'/temp/shd_smview_connect/token/'.$name;
        $file = $id.'.token';

        if(!file_exists($path.'/'.$file)) {
            throw new Exception(sprintf('no token file %s found', $path.'/'.$file));
        }

        unlink($path.'/'.$file);
    }
}
