<?php

namespace SHD\Smview\Connect\Service\Azure\Auth;

use Exception;
use SHD\Smview\Connect\Service\Auth\TokenAuthenticator;
use SHD\Smview\Connect\Service\Auth\TokenInterface;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class AzureAuthenticator extends TokenAuthenticator
{
    private HttpClientInterface $httpClient;
    private string $clientId;
    private string $tenantId;
    private string $clientSecret;

    private ?TokenInterface $token = null;

    /**
     * @param HttpClientInterface $httpClient
     * @param string $clientId
     * @param string $tenantId
     * @param string $clientSecret
     */
    public function __construct(HttpClientInterface $httpClient, string $clientId, string $tenantId, string $clientSecret)
    {
        $this->httpClient = $httpClient;
        $this->clientId = $clientId;
        $this->tenantId = $tenantId;
        $this->clientSecret = $clientSecret;

        try {
            $this->token = self::restore(AzureToken::TOKEN_NAME, $clientId);
        } catch (Exception $e) {

        }
    }

    /**
     * @return TokenInterface
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     */
    public function getToken(): TokenInterface
    {
        if(!$this->token || $this->token->isExpired()) {
            $this->token = $this->requestToken();
        }

        return $this->token;
    }

    /**
     * @return TokenInterface
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws TransportExceptionInterface
     */
    public function requestToken(): TokenInterface
    {
        $url = sprintf('https://login.microsoftonline.com/%s/oauth2/token', $this->tenantId);

        $response = $this->httpClient->request('POST', $url, [
            'body' => [
                'grant_type' => 'client_credentials',
                'client_id' => $this->clientId,
                'client_secret' => $this->clientSecret,
                'resource' => 'https://management.azure.com/',
            ]
        ]);
        $content = $response->getContent();
        $token = AzureToken::fromJson($content);
        $token->setId($this->clientId);

        $this->persist($token);

        return $token;
    }

    /**
     * @throws Exception
     */
    public function refreshToken(TokenInterface $token): TokenInterface
    {
        throw new Exception('refresh is not supported on azure api');
    }
}
