<?php

namespace SHD\Smview\Connect\Service\Azure\Auth;

use DateTime;
use SHD\Smview\Connect\Service\Auth\TokenInterface;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class AzureToken implements TokenInterface
{
    public string $accessToken;
    public int $expiresIn;
    public DateTime $expiresOn;
    private string $id;

    public const TOKEN_NAME = 'azure_token';

    public static function fromJson(string $json): TokenInterface
    {
        $data = json_decode($json);

        $token = new self();
        $token->accessToken = $data->access_token;
        $token->expiresIn = $data->expires_in;

        $token->expiresOn = (new DateTime())->modify(sprintf('+%s seconds', $token->expiresIn));

        return $token;
    }

    public function isExpired(): bool
    {
        return $this->expiresOn < (new DateTime());
    }

    public function getName(): string
    {
        return self::TOKEN_NAME;
    }

    public function getId(): string
    {
        return $this->id;
    }

    public function setId(string $id): TokenInterface
    {
        $this->id = $id;

        return $this;
    }

    public function getAccessToken(): string
    {
        return $this->accessToken;
    }
}
