<?php

namespace SHD\Smview\Connect\Service\Azure;

use SHD\Smview\Connect\Domain\Azure\AzureResourceTypeInterface;
use SHD\Smview\Connect\Domain\Azure\GenericResourceType;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class AzureResourceTypeFactory
{
    public static function fromTypeLiteral($type): AzureResourceTypeInterface
    {
        $matches = [];
        preg_match('/Microsoft\.([a-z]+)\/([a-z]+)\/*([a-z]+)*/i', $type, $matches);

        if(count($matches) == 0) {
            return new GenericResourceType($type);
        }

        $ns = ucfirst($matches[1]);
        $cls1 = ucfirst($matches[2]);
        $cls2 = ucfirst($matches[3]) ?? null;

        $baseNs = 'idoit\\Module\\Smviewconnect\\Domain\\Azure\\Type';

        if($cls2 == null) {
            $class = $baseNs.'\\'.$ns.'\\'.$cls1.'Type';
        } else {
            $class = $baseNs.'\\'.$ns.'\\'.$cls1.$cls2.'Type';
        }

        if(!class_exists($class)) {
            return new GenericResourceType($type);
        }

        $obj = (new $class($type)) ?? null;

        if(!$obj instanceof AzureResourceTypeInterface) {
            return new GenericResourceType($type);
        }

        return $obj;
    }
}
