<?php

namespace SHD\Smview\Connect\Service\Azure\Discovery;

use SHD\Smview\Connect\DAO\ProfileDiscoveryDataDAO;
use SHD\Smview\Connect\Domain\Azure\AzureResourceTypeInterface;
use SHD\Smview\Connect\Domain\Azure\Exception\AzureCallFailedException;
use SHD\Smview\Connect\Service\Azure\AzureClient;
use SHD\Smview\Connect\Service\Azure\AzureResourceTypeFactory;
use SHD\Smview\Connect\Service\DiscoveryAnalyzer;
use isys_exception_dao;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class ResourceAnalyzer extends DiscoveryAnalyzer
{
    /**
     * @var AzureClient
     */
    private AzureClient $azureClient;

    /**
     * @var ProfileDiscoveryDataDAO
     */
    private ProfileDiscoveryDataDAO $dao;

    /**
     * @var int
     */
    private int $profileId;

    public function __construct(AzureClient $azureClient, ProfileDiscoveryDataDAO $dao, int $profileId)
    {
        $this->azureClient = $azureClient;
        $this->dao = $dao;
        $this->profileId = $profileId;
    }

    /**
     * Start the Azure resource discovery process
     *
     * @return void
     * @throws AzureCallFailedException|isys_exception_dao
     */
    public function discover(): void
    {
        //get raw resources data
        try {
            $_resources = $this->azureClient->getSubscriptionResourcesList();
        } catch (\Throwable $e) {
            throw new AzureCallFailedException($e->getMessage());
        }

        //extract types
        /** @var array<AzureResourceTypeInterface> $discoveredTypes */
        $discoveredTypes = array_map(function ($t) {
            return AzureResourceTypeFactory::fromTypeLiteral($t);
        }, array_unique(array_map(function ($d) {
            return $d->type;
        }, $_resources)));

        $data = array_map(function ($dt) {
            return [
                'type' => ProfileDiscoveryDataDAO::DATA_TYPE_OBJECT_TYPE,
                'key' => $dt->getName(),
                'value' => null,
            ];
        }, array_filter($discoveredTypes, function ($dt) {
            return $dt->useForImport();
        }));

        //persist discovery
        $this->dao->updateDiscoveryData($this->profileId, $data, ProfileDiscoveryDataDAO::DATA_TYPE_OBJECT_TYPE);
    }
}
