<?php

namespace SHD\Smview\Connect\Service\Azure\Import;

use SHD\Smview\Connect\DAO\ProfileDiscoveryDataDAO;
use SHD\Smview\Connect\Service\Azure\AzureClient;
use SHD\Smview\Connect\Service\Azure\AzureConfiguration;
use SHD\Smview\Connect\Service\Import\IdoitImportManager;
use SHD\Smview\Connect\Service\ImporterInterface;
use SHD\Smview\Connect\Service\ShdLicense;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
abstract class AzureImporter implements ImporterInterface
{
    protected AzureClient $azureClient;

    protected IdoitImportManager $idoitImportManager;

    protected ProfileDiscoveryDataDAO $discoveryDataDao;

    protected int $profileId;

    protected AzureConfiguration $configuration;

    public function __construct(AzureClient $azureClient, IdoitImportManager $idoitImportManager, ProfileDiscoveryDataDAO $discoveryDataDao, int $profileId, AzureConfiguration $configuration)
    {
        $this->azureClient = $azureClient;
        $this->idoitImportManager = $idoitImportManager;
        $this->discoveryDataDao = $discoveryDataDao;
        $this->profileId = $profileId;
        $this->configuration = $configuration;
    }

    public function status(?int $profileId = null): bool
    {
        //todo: azure status
        return true;
    }

    public function isLicensed(): bool
    {
        return $this->configuration::isLicensed();
    }

    public function getName(): string
    {
        return 'azure';
    }

    public function isEnabled(): bool
    {
        return AzureConfiguration::isEnabled();
    }

    public function getProfileId(): ?int
    {
        return $this->profileId;
    }
}
