<?php

namespace SHD\Smview\Connect\Service\Cache;

use DateTime;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class CacheArray implements CacheInterface
{
    /**
     * @var string
     */
    private $key;

    /**
     * @var array
     */
    private $data;

    /**
     * @var DateTime|null
     */
    private $invalidateBy = null;

    /**
     * @var string
     */
    private $cacheDir;

    public function __construct(string $key, string $cacheDir)
    {
        $this->key = $key;
        $this->cacheDir = $cacheDir;
    }

    public function get(): array
    {
        return $this->data;
    }

    /**
     * @param array $data
     * @param DateTime|null $invalidateBy
     * @return CacheArray
     */
    public function set($data, ?DateTime $invalidateBy = null): CacheInterface
    {
        $this->data = $data;
        $this->invalidateBy = $invalidateBy;

        return $this;
    }

    /**
     * @return bool
     */
    public function isValid(): bool
    {
        if(!$this->hasCache()) {
            return false;
        }

        if($this->invalidateBy >= (new DateTime()) || $this->invalidateBy == null) {
            return true;
        }

        return false;
    }

    /**
     * @return bool
     */
    public function hasCache(): bool
    {
        return file_exists($this->cacheDir.'/'.$this->key.'.cache');
    }

    public function getKey(): string
    {
        return $this->key;
    }

    public function getCacheDir(): string
    {
        return $this->cacheDir;
    }

    public function getInvalidateBy(): ?DateTime
    {
        return $this->invalidateBy;
    }
}
