<?php

namespace SHD\Smview\Connect\Service;

use DateTime;
use SHD\Smview\Connect\Service\Cache\CacheInterface;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class CacheManager
{
    public function getCaches(string $dir): array
    {
        $files = scandir($dir);
        $caches = [];

        foreach ($files as $file) {
            $cache = unserialize(file_get_contents($dir.'/'.$file)) ?? null;

            if(!$cache instanceof CacheInterface) {
                continue;
            }

            $caches[] = $cache;
        }

        return $caches;
    }

    public function persist(CacheInterface $cache): CacheManager
    {
        if(!is_dir($cache->getCacheDir())) {
            mkdir($cache->getCacheDir(), 0777, true);
        }

        file_put_contents($cache->getCacheDir().'/'.$cache->getKey().'.cache', serialize($cache));

        return $this;
    }

    public function retreive(string $key, string $cacheDir): ?CacheInterface
    {
        if(!file_exists($cacheDir.'/'.$key.'.cache')) {
            return null;
        }

        return unserialize(file_get_contents($cacheDir.'/'.$key.'.cache')) ?? null;
    }

    public function invalidate(CacheInterface $cache): void
    {
        $cache = $this->retreive($cache->getKey(), $cache->getCacheDir());

        if(!$cache) {
            return;
        }

        $cache->set(null, new DateTime());

        $this->persist($cache);
    }

//    public function warmup(CacheInterface $cache, array $parameters)
//    {
//        $classes = array_filter(get_declared_classes(), function ($className) {
//            return is_subclass_of($className, 'shd_api_model_cmdb_cachable');
//        });
//
//        $class = array_values(array_filter($classes, function ($className) use ($cache) {
//            return $className::supports($cache->getKey());
//        }))[0] ?? null;
//
//        if(!$class) {
//            return;
//        }
//
//        $data = $class::retreiveData($parameters);
//        $cache->set($data);
//        $this->persist($cache);
//    }
}
