<?php

namespace SHD\Smview\Connect\Service;

use isys_exception_general;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class ConfigurationSettingsKey
{
    /**
     * @var string
     */
    public $key;

    /**
     * @var string
     */
    public $label;

    /**
     * @var string
     */
    public $typeName;

    /**
     * @var bool
     */
    public $visible = true;

    /**
     * @var array|string|null
     */
    public $value = null;

    public array $tags = [];

    public const TYPE_TEXT = 'text';
    public const TYPE_PASSWORD = 'password';
    public const TYPE_BOOL = 'bool';
    public const TYPE_SELECT = 'select';

    public const SELECT_ENUM_KEY = 'key';
    public const SELECT_ENUM_TEXT = 'text';
    public const TYPE_CRON = 'cron';

    /**
     * @param string $key
     * @param string $label
     * @param string $typeName
     * @param bool $visible
     * @param $value
     * @param array $tags
     * @throws isys_exception_general
     */
    public function __construct(string $key, string $label, string $typeName, bool $visible = true, $value = null, array $tags = [])
    {
        $this->key = $key;
        $this->label = $label;
        $this->typeName = $typeName;
        $this->visible = $visible;
        $this->value = $value;

        if($typeName == self::TYPE_SELECT && !is_array($value)) {
            throw new isys_exception_general(sprintf('you have to provide a value array for the type %s', $typeName));
        }
    }

    /**
     * @param string $key
     * @param string $label
     * @param string $typeName
     * @param bool $visible
     * @param $value
     * @param array $tags
     * @return ConfigurationSettingsKey
     * @throws isys_exception_general
     */
    public static function set(string $key, string $label, string $typeName, bool $visible = true, $value = null, array $tags = []): ConfigurationSettingsKey
    {
        return new self($key, $label, $typeName, $visible, $value, $tags);
    }

    public function toArray(): array
    {
        $data = [
            'key' => $this->key,
            'label' => $this->label,
            'typeName' => $this->typeName,
            'visible' => $this->visible,
            'tags' => $this->tags,
        ];

        if(is_array($this->value)) {
            $data['value'] = json_encode($this->value);
        } else {
            $data['value'] = $this->value;
        }

        return $data;
    }
}
