<?php

namespace SHD\Smview\Connect\Service;

use idoit\Component\Logger;
use isys_module_shd_smview_connect;
use isys_tenantsettings;
use Monolog\Formatter\LineFormatter;
use Monolog\Handler\StreamHandler;
use Monolog\Logger as LoggerAlias;

class ConnectLogger
{
    /**
     * Logger instance
     *
     * @var Logger
     */
    protected $logger;

    public function __construct()
    {
        global $g_absdir;

        // setup logger
        $logEnabled = isys_tenantsettings::get(isys_module_shd_smview_connect::CONNECT_LOG_ENABLED, true);
        $logLevel = isys_tenantsettings::get(isys_module_shd_smview_connect::CONNECT_LOG_LEVEL, LoggerAlias::WARNING);

        $logPath = $g_absdir . '/log/shd_smview_connect_' . date('Y-m-d') . '.log';

        $logHandlers = [];
        if ($logEnabled) {
            $logHandler = new StreamHandler($logPath, $logLevel);
            $logHandler->setFormatter(new LineFormatter(null, null, true, true));
            $logHandlers[] = $logHandler;
        }
        $this->logger = new Logger('smview_connect', $logHandlers);
    }

    /**
     * Get log file of the past days.
     *
     * @param int $pastDays
     * @return array
     */
    public function getLogs(int $pastDays = 1): array
    {
        global $g_absdir;

        $logs = [];
        for ($i = 0; $i < $pastDays; $i++) {
            $offset = time() - ($i * 86400);
            $date = date('Y-m-d', $offset);
            $logPath = $g_absdir . '/log/shd_smview_connect_' . $date . '.log';
            $rawLog = file_get_contents($logPath);
            $logLines = [];
            if ($rawLog !== false) {
                $logLines = explode("\n", $rawLog);
            }
            $logs[$date] = $logLines;
        }
        return $logs;
    }

    /**
     * @param string $message
     * @param string|null $logPart
     * @param int $logLevel
     * @return void
     */
    public function log(string $message, string $logPart = null, int $logLevel = LoggerAlias::INFO): void
    {
        if(!(!$logPart)) {
            $message = sprintf('[%s] %s', $logPart, $message);
        }

        $this->logger->log($logLevel, $message);
    }
}
