<?php

namespace SHD\Smview\Connect\Service;

use Cron\CronExpression;
use DateTime;
use Exception;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class Cron
{
    /**
     * Gibt eine Liste von crontab werten heraus die als standardwerte benutzt werden können
     *
     * @return string[]
     */
    public static function getDefaultCronList(): array
    {
        return [
            'LC__CMDB__SMVIEW_CONNECT__CRON__EVERY_6_HOURS' => '0 */6 * * *',
            'LC__CMDB__SMVIEW_CONNECT__CRON__EVERY_12_HOURS' => '0 */12 * * * ',
            'LC__CMDB__SMVIEW_CONNECT__CRON__6AM' => '0 6 * * * ',
            'LC__CMDB__SMVIEW_CONNECT__CRON__NOON' => '0 12 * * * ',
            'LC__CMDB__SMVIEW_CONNECT__CRON__EVERY_HOUR' => '0 * * * *',
            'LC__CMDB__SMVIEW_CONNECT__CRON__6PM' => '0 18 * * *',
        ];
    }

    /**
     * @throws Exception
     */
    public static function getDateTimeFromCron(string $cron): DateTime
    {
        return (new CronExpression($cron))->getNextRunDate();
    }

    /**
     * Default crontab wenn kein wert konfiguriert wurde
     *
     * @return string
     */
    public static function getDefaultCronForNotConfigured(): string
    {
        return self::getDefaultCronList()['LC__CMDB__SMVIEW_CONNECT__CRON__EVERY_6_HOURS'];
    }
}
