<?php

namespace SHD\Smview\Connect\Service\DB;

use Exception;
use Generator;
use idoit\Component\Property\Property;
use isys_application;
use isys_cmdb_dao_category;
use isys_cmdb_dao_category_g_custom_fields;
use isys_cmdb_dao_category_global;
use isys_cmdb_dao_object;
use isys_component_dao;
use isys_component_dao_result;
use isys_component_database;
use isys_exception_dao;
use isys_exception_database;
use isys_cmdb_dao_category_g_global;
use isys_exception_database_mysql;
use isys_exception_general;
use isys_exception_validation;
use isys_import_handler_cmdb;
use Webmozart\Assert\Assert;
use MySQL\Error\Server as MySQLServerErrors;

final class DaoUtil
{
    /**
     * @param isys_component_dao_result $result
     * @return Generator
     */
    public static function rows(isys_component_dao_result $result): Generator
    {
        while ($row = $result->get_row()) {
            yield $row;
        }
    }

    /**
     * @param isys_component_dao_result $result
     * @return array
     */
    public static function allRows(isys_component_dao_result $result): array
    {
        $data = [];

        foreach(self::rows($result) as $row) {
            $data[] = $row;
        }

        return $data;
    }

    public static function iterator(isys_component_dao_result $result, callable $fn): Generator
    {
        while ($row = $result->get_row()) {
            yield $fn($row);
        }
    }

    /**
     * @throws isys_exception_database
     */
    public static function getCatgoryFieldValue(isys_cmdb_dao_category_global $dao, array $obj, string $constant, string $key)
    {
        if($dao instanceof isys_cmdb_dao_category_g_custom_fields) {
            $data = DaoUtil::allRows($dao->get_data(
                1,
                $obj['isys_obj__id'],
                "and isysgui_catg_custom__const = '".$constant."' and concat(isys_catg_custom_fields_list__field_type, '_', isys_catg_custom_fields_list__field_key) = '".$key."'"
            ));

            return $data[0]['isys_catg_custom_fields_list__field_content'] ?? null;
        } else {
            if($constant == 'C__CATG__GLOBAL' && $key) {
                return $obj['isys_obj__title'];
            }

            $data = DaoUtil::allRows($dao->get_data_by_property(1, $obj['isys_obj__id'], $key));

            return $data[0][$key] ?? null;
        }
    }

    /**
     * @param array $arr
     * @param mixed $key
     * @param mixed $value
     * @return mixed|null
     */
    public static function pluck(array $arr, $key, $value)
    {
        return array_values(array_filter($arr, function ($a) use ($key, $value) {
            if(is_object($a)) {
                return $a->$key == $value;
            } else {
                return $a[$key] == $value;
            }
        }))[0] ?? null;
    }

    public static function where(array $arr, $key, $value): array
    {
        return array_values(array_filter($arr, function ($a) use ($key, $value) {
            if(is_object($a)) {
                return $a->$key == $value;
            } else {
                return $a[$key] == $value;
            }
        })) ?? [];
    }

    /**
     * @param string $catType
     * @param string $catConst
     * @param $id
     * @return isys_cmdb_dao_category_g_custom_fields|isys_component_dao
     * @throws Exception
     */
    public static function categoryDaoFactory(string $catType, string $catConst, $id)
    {
        /** @var isys_component_database $db */
        $db = isys_application::instance()->container->get('database');

        if ($catType === 'catgID') { //globale cat
            $d = new isys_cmdb_dao_category_g_global($db);
            $className = DaoUtil::allRows($d->retrieve(
                <<<SQL
SELECT `isysgui_catg__class_name` FROM `isysgui_catg` WHERE `isysgui_catg__const` = {$d->convert_sql_text($catConst)};
SQL
            ))[0]['isysgui_catg__class_name'] ?? null;

            if(!$className) {
                throw new Exception(sprintf('cannot get category dao classname for constant %s', $catConst));
            }

            $i = new $className($db);
        } elseif ($catType === 'catsID') { //cats
            $d = new isys_cmdb_dao_category_g_global($db);
            $className = DaoUtil::allRows($d->retrieve(
                <<<SQL
SELECT `isysgui_cats__class_name` FROM `isysgui_cats` WHERE `isysgui_cats__const` = {$d->convert_sql_text($catConst)};
SQL
            ))[0]['isysgui_cats__class_name'] ?? null;

            if(!$className) {
                throw new Exception(sprintf('cannot get category dao classname for constant %s', $catConst));
            }

            $i = new $className($db);
        } else { //custom cat
            $i = new isys_cmdb_dao_category_g_custom_fields($db);
            $i->set_catg_custom_id($id);
        }

        return $i;
    }

    /**
     * @param int $typeId
     * @param int|null $statusId
     * @param array $categoryAssignments
     * @return isys_component_dao_result
     * @throws isys_exception_database
     */
    public static function getObjectsByTypeAndCategoryFields(int $typeId, ?int $statusId, array $categoryAssignments): isys_component_dao_result
    {
        /** @var isys_component_database $db */
        $db = isys_application::instance()->container->get('database');
        $dao = isys_cmdb_dao_object::instance($db);

        $query = <<<SQL
SELECT isys_obj.*, isys_obj_type__title, isys_obj_type__isys_obj_type_group__id
FROM isys_obj
INNER JOIN isys_obj_type ON isys_obj__isys_obj_type__id = isys_obj_type__id
WHERE TRUE AND isys_obj__isys_obj_type__id = {$dao->convert_sql_id($typeId)}
SQL;

        if($statusId != null) {
            $query .= PHP_EOL.<<<SQL
AND isys_obj__status = {$dao->convert_sql_id($statusId)}
SQL;
        }

        foreach ($categoryAssignments as $c) {
            $assignment = $c['assignment'];
            $comparator = $c['comparator'];
            $value = (is_string($c['value'])) ? $dao->convert_sql_text($c['value']) : $dao->convert_sql_int($c['value']);

            if($assignment['dao'] instanceof isys_cmdb_dao_category_g_custom_fields) {
                $query .= PHP_EOL.<<<SQL
and $value $comparator (
	select isys_catg_custom_fields_list.isys_catg_custom_fields_list__field_content
    from isys_catg_custom_fields_list
    where isys_catg_custom_fields_list.isys_catg_custom_fields_list__isys_obj__id = isys_obj.isys_obj__id
    and concat(isys_catg_custom_fields_list.isys_catg_custom_fields_list__field_type, {$dao->convert_sql_text('_')}, isys_catg_custom_fields_list.isys_catg_custom_fields_list__field_key) = {$dao->convert_sql_text($assignment['property_key'])}
)
SQL;
            } else {
                /** @var isys_cmdb_dao_category $cdao */
                $cdao = $assignment['dao'];
                $props = $cdao->get_properties();
                $prop = $props[$assignment['property_key']];
                $field = $prop['data']['field'];
                $table = $cdao->get_table();

                $query .= PHP_EOL.<<<SQL
and $value $comparator (
	select $field
    from $table
    where {$table}__isys_obj__id = isys_obj.isys_obj__id
    limit 1
)
SQL;
            }
        }

        $query .= PHP_EOL.<<<SQL
ORDER BY isys_obj__title;
SQL;


        return $dao->retrieve($query);
    }

    /**
     * Check if a condition is true and only the execute the callable
     *
     * @param bool $condition
     * @param callable $callable
     * @param null $default
     * @return mixed|null
     */
    public static function guard(bool $condition, callable $callable, $default = null)
    {
        if(!$condition) {
            return $default;
        }

        return $callable();
    }

    /**
     * @param string $table
     * @param int $objId
     * @return int
     * @throws isys_exception_database
     */
    public static function getSingleValueCategoryDataId(string $table, int $objId): int
    {
        /** @var isys_component_database $db */
        $db = isys_application::instance()->container->get('database');
        $dao = isys_cmdb_dao_object::instance($db);

        $q = sprintf('
            select %s__id id
            from %s
            where %s__isys_obj__id = %s limit 1
        ', $table, $table, $table, $dao->convert_sql_id($objId));

        return self::allRows($dao->retrieve($q))[0]['id'] ?? 0;
    }

    /**
     * @param $table
     * @param int $objId
     * @return array
     * @throws isys_exception_database
     */
    public static function getMultiValueCategoryData($table, int $objId): array
    {
        /** @var isys_component_database $db */
        $db = isys_application::instance()->container->get('database');
        $dao = isys_cmdb_dao_object::instance($db);

        $q = sprintf('
            select *
            from %s
            where %s__isys_obj__id = %s
        ', $table, $table, $dao->convert_sql_id($objId));

        return self::allRows($dao->retrieve($q));
    }

    /**
     * Get the DB Field Name from a given Property
     *
     * @param string $prop          The property name
     * @param array $properties     The Properies array
     * @return string|null          get the name or null if it wasnt found
     */
    public static function getFieldNameFromProperty(string $prop, array $properties): ?string
    {
        $property = $properties[$prop] ?? null;

        if(!$property) {
            return null;
        }

        if($property instanceof Property) {
            return $property->getData()->getField();
        }

        return $property['data']['field'] ?? null;
    }

    /**
     * Write a value to any category
     * automatically finds out if it is a single or multi valu category
     *
     * @param array $assignment the assignment array, must have: ['dao', 'property_key'] -> target field
     * @param $value
     * @param array $idoitObj the object with the category
     * @param string|null $compProp name of the comparison property (only needed if target cat is multi value)
     * @param array|null $compAssignment the assignment of the comparison category/prop, must have ['dao', 'property_key', 'category_const']
     * @param string|null $multiValueUniqeIdentifier
     * @return void
     * @throws isys_exception_database
     * @throws isys_exception_general
     * @throws isys_exception_validation
     */
    public static function writeDynamicCategoryData(array $assignment, $value, array $idoitObj, ?string $compProp = null, ?array $compAssignment = null, ?string $multiValueUniqeIdentifier = null)
    {
        /** @var isys_cmdb_dao_category $dao */
        $dao = $assignment['dao'];

        if($dao->is_multivalued()) {
            Assert::notEmpty($compAssignment, 'compAssignment is mandatory on a multi value category');

            //wenn die cat id des vergleichs-quell-felds == cat id des zielfelds -> dann wert nicht aus übergeordneten objekt holen sondern aus cat tabelle
            //zusätzlich: wenn oberes eintritt dann ist das vergleichs-ziel-feld vermutlich nicht angegeben (wenn doch dann nehmen)
            //-> wenn leer dann dasselbe feld wie quelle nehmen als ziel-vergleich
            if($compAssignment['category_const'] == $assignment['category_const'] && !$compProp) {
                $compProp = $compAssignment['property_key'];
            }

            Assert::notEmpty($compProp, 'compProp is mandatory on a multi value category');
        }

        $dao->set_object_id($idoitObj['isys_obj__id']);

        $catdata = new \isys_array([]);

        try { //temporärer try, wird ggf durch idoit update unnötig, siehe idoit ticket #23725 https://help.i-doit.com/hc/de/requests/23725
            $dao->category_data($catdata, C__RECORD_STATUS__NORMAL);
        } catch (\TypeError $e) {
        }

        $importType = isys_import_handler_cmdb::C__CREATE;
        $dataId = null;

        //wir bekommen das objekt auf das wir uns beziehen in $idoitObj
        //jetzt müssen wir uns anhand $compAssignment noch die kategorie und das feld wie den feldinhalt zusammensuchen -> danach vergleichen
        $compValue = null;
        if($compAssignment != null) {
            $compValue = DaoUtil::getCatgoryFieldValue($compAssignment['dao'], $idoitObj, $compAssignment['category_const'], $compAssignment['property_key']);
        }

        $dataExists = false;

        if($dao instanceof isys_cmdb_dao_category_g_custom_fields) {
            if(count($catdata->toArray()) > 0) {
                $dataExists = true;
            }
        } else {
            if(count(DaoUtil::allRows($dao->get_data(null, $idoitObj['isys_obj__id']))) > 0) {
                $dataExists = true;
            }
        }

        //den konkreten listeneintrag holen sollte einer da sein (wenn ja dann mode update)
        if($dataExists) {
            if($dao->is_multivalued()) { //wenn single value dann ist nur ein eintrag relevant
                //wichtig: wenn eine multi-value cat in $compAssignment angegeben wurde dann abbrechen
                if ($compAssignment['dao']->is_multivalued() && !$multiValueUniqeIdentifier) {
                    throw new isys_exception_general('passing a multi value category as $compAssignment without having a $multiValueUniqeIdentifier is not possible');
                }

                if ($compAssignment['dao']->is_multivalued() && $multiValueUniqeIdentifier) {
                    $compValue = $multiValueUniqeIdentifier;
                }

                foreach ($catdata as $key => $entry) {
                    if($entry[$compProp] == $compValue) {
                        $dataId = $key;
                        $importType = isys_import_handler_cmdb::C__UPDATE;

                        break;
                    }
                }
            } else {
                $dataId = array_keys($catdata->toArray())[0];
                $importType = isys_import_handler_cmdb::C__UPDATE;
            }
        }

        //value check
        if($value instanceof \DateTime) {
            $value = $value->format('d.m.Y H:i:s');
        }

        //wenn update dann müssen die vorhanden daten erstmal ausgelesen werden und dann nur das feld geupdated werden was verändert werden soll
        if($dataId > 0 && $importType == isys_import_handler_cmdb::C__UPDATE) {
            //props mit daten angereichert holen
            //catdata enthält nur felder die schon werte haben, somit werden neue nicht gefüllt -> props holen und ggf werte anhängen
            $_data = [];
            foreach ($dao->get_properties() as $p => $info) {
                $_data[$p] = $catdata->get($dataId)[$p] ?? null;
            }

            $props = [];

            //alles durchgehen und an der richtigen stelle mit neuem wert updaten
            foreach ($_data as $_prop_key => $_prop_value) {
                $_prop = $dao->get_properties()[$_prop_key];

                if($_prop_value->m_id != null && $_prop['data']['type'] != 'text') {
                    $_val = $_prop_value->m_id;
                } else {
                    $_val = $_prop_value->m_value ?? null;
                }

                //wenn es eine custom cat ist und hier ein dialog+ feld ankommt dann ist der m_id wert nicht da...
                //hier also in dem fall nochmal durch das mapping jagen
                if($dao instanceof isys_cmdb_dao_category_g_custom_fields && !is_numeric($_val) && !is_null($_val)) {
                    if($_prop['info']['type'] == 'dialog_plus') {
                        //ident suchen...
                        $dpident = $_prop["ui"]["params"]["identifier"] ?? null;
                        $_val = self::getCustomDialogPlusId($dpident, $_val);
                    }
                }

                if(is_object($_prop)) { //ab mindestens idoit 26 ist das hier kein array mehr
                    if($_prop->getData()->getType() == 'int' && count($_prop->getData()->getReferences() ?? []) == 0) {
                        $_val = $_val == 'Yes' ? 1 : 0;
                    }
                } else {
                    if($_prop['data']['type'] == 'int' && !array_key_exists('references', $_prop['data'])) {
                        $_val = $_val == 'Yes' ? 1 : 0;
                    }
                }

                if($_prop_key == $assignment['property_key']) {
                    $_val = $value;
                }

                $props[$_prop_key] = ['value' => $_val];
            }
        } else {
            $props[$assignment['property_key']] = ['value' => $value];

            //wenn neu angelegt dann muss auch das vergleichsfeld gefüllt werden
            if($compProp != null && $compValue != null) {
                $props[$compProp] = ['value' => $compValue];
            }
        }

        $retryCounter = 0;
        $maxRetry = 5;

        $syncFn = function () use ($dataId, $props, $idoitObj, $importType, $dao) {
            $dao->sync([
                'data_id' => $dataId,
                'properties' => $props,
            ], $idoitObj['isys_obj__id'], $importType);
        };

        try {
            $syncFn();
        } catch (isys_exception_database_mysql $dbe) {
            isys_application::instance()->container->get('logger')->emergency('['.__FILE__.'] '.$dbe->getMessage());

            if ($dbe->getCode() === MySQLServerErrors::ER_LOCK_DEADLOCK && $retryCounter <= $maxRetry) {
                usleep(500);
                $retryCounter++;
                $syncFn();
            }
        }
    }

    /**
     * @param string $identifier
     * @param string $title
     * @return int|null
     * @throws isys_exception_database
     */
    public static function getCustomDialogPlusId(string $identifier, string $title): ?int
    {
        /** @var isys_component_database $db */
        $db = isys_application::instance()->container->get('database');
        $dao = isys_cmdb_dao_object::instance($db);

        return DaoUtil::allRows($dao->retrieve(
            <<<SQL
SELECT `isys_dialog_plus_custom__id` `id`
FROM `isys_dialog_plus_custom`
where `isys_dialog_plus_custom__status` = 2
    and `isys_dialog_plus_custom__identifier` = {$dao->convert_sql_text($identifier)}
    and `isys_dialog_plus_custom__title` = {$dao->convert_sql_text($title)}
SQL
        ))[0]['id'] ?? null;
    }

    /**
     * @param string $identifier
     * @param string $title
     * @return int|null
     * @throws isys_exception_dao
     */
    public static function createCustomDialogPlusId(string $identifier, string $title): ?int
    {
        /** @var isys_component_database $db */
        $db = isys_application::instance()->container->get('database');
        $dao = isys_cmdb_dao_object::instance($db);

        $dao->update(<<<SQL
INSERT INTO `isys_dialog_plus_custom`(`isys_dialog_plus_custom__identifier`, `isys_dialog_plus_custom__status`, `isys_dialog_plus_custom__title`) VALUES ({$dao->convert_sql_text($identifier)}, 2, {$dao->convert_sql_text($title)})
SQL) && $dao->apply_update();

        return $dao->get_last_insert_id();
    }

    /**
     * @param string $const
     * @param bool $g
     * @return string|null
     * @throws isys_exception_database
     */
    public static function getCategorySourceTable(string $const, bool $g = true): ?string
    {
        /** @var isys_component_database $db */
        $db = isys_application::instance()->container->get('database');
        $dao = isys_cmdb_dao_object::instance($db);

        $t = 'isysgui_catg';
        if(!$g) {
            $t = 'isysgui_cats';
        }

        $result = self::allRows($dao->retrieve("select ".$t."__source_table tbl from ".$t." where ".$t."__const = ".$dao->convert_sql_text($const)));

        return $result[0]['tbl'] ?? null;
    }
}
