<?php

namespace SHD\Smview\Connect\Service\Gcache;

use DateTime;
use isys_exception_dao;
use League\Csv\Exception;
use League\Csv\Reader;
use League\Csv\SyntaxError;
use League\Csv\UnavailableStream;
use ReflectionException;
use ReflectionProperty;
use SHD\Smview\Connect\DAO\GcacheDAO;
use SHD\Smview\Connect\Domain\Gcache\GcacheEntry;
use SHD\Smview\Connect\Domain\Gcache\GcacheEntryMapping;
use SHD\Smview\Connect\Service\AbstractConfiguration;
use SHD\Smview\Connect\Service\CsvEntryMappingInterface;
use SHD\Smview\Connect\Service\CsvImporterInterface;

class Importer implements CsvImporterInterface
{
    /**
     * @param string $filename
     * @param GcacheEntryMapping $mapping
     * @param string|null $delimiter
     * @return array<GcacheEntry>
     * @throws Exception
     * @throws ReflectionException
     */
    public function read(string $filename, CsvEntryMappingInterface $mapping, ?string $delimiter = null): array
    {
        $csv = Reader::createFromPath($filename, 'r');
        $csv->setHeaderOffset(0);

        if(!(!$delimiter)) {
            $csv->setDelimiter($delimiter);
        }

        $header = $csv->getHeader();

        if(!$header) {
            throw new \Exception('csv has to have a header row');
        }

        $data = [];

        foreach ($csv->getRecords() as $record) {
            $entry = new GcacheEntry();

            foreach (get_object_vars($mapping) as $mapVar => $field) {
                $val = @$record[$field] ?? null;

                $rp = new ReflectionProperty(GcacheEntry::class, $mapVar);

                if($rp->getType()->getName() == 'DateTime' && $val != null) {
                    $val = new DateTime($val);
                }

                if($rp->getType()->getName() == 'int' && $val != null) {
                    $val = intval($val);
                }

                $entry->{$mapVar} = $val;
            }

            $data[] = $entry;
        }

        return $data;
    }

    /**
     * @param string $filename
     * @param CsvEntryMappingInterface $mapping
     * @param string|null $delimiter
     * @return void
     * @throws Exception
     * @throws SyntaxError
     * @throws UnavailableStream
     * @throws isys_exception_dao|ReflectionException
     */
    public function import(string $filename, CsvEntryMappingInterface $mapping, ?string $delimiter = null): void
    {
        $data = $this->read($filename, $mapping, $delimiter);
        $dao = GcacheDAO::getInstance();

        $dao->clear();
        $dao->addData($data);
    }

    /**
     * @return GcacheConfiguration
     */
    public function getConfig(): AbstractConfiguration
    {
        return new GcacheConfiguration();
    }
}
