<?php

namespace SHD\Smview\Connect\Service\Ldap;

use SHD\Smview\Connect\Service\ConnectLogger;
use SHD\Smview\Connect\Service\Import\AttributeEmittingImporterInterface;
use SHD\Smview\Connect\Service\Import\IdoitImportManager;
use SHD\Smview\Connect\Service\Import\ValuePipeline;
use SHD\Smview\Connect\Service\ImporterInterface;
use isys_exception_database;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
abstract class Importer implements ImporterInterface, AttributeEmittingImporterInterface, LdapImporterInterface
{
    protected LdapClient $client;
    protected IdoitImportManager $idoitImportManager;
    protected LdapConfiguration $configuration;
    protected int $profileId;
    protected ConnectLogger $logger;
    protected ValuePipeline $valuePipeline;

    public function __construct(LdapClient $client, IdoitImportManager $idoitImportManager, LdapConfiguration $configuration, int $profileId, ConnectLogger $logger, ValuePipeline $valuePipeline)
    {
        $this->client = $client;
        $this->idoitImportManager = $idoitImportManager;
        $this->configuration = $configuration;
        $this->profileId = $profileId;
        $this->logger = $logger;
        $this->valuePipeline = $valuePipeline;
    }

    public function getName(): string
    {
        return 'ldap';
    }

    public function isLicensed(): bool
    {
        return $this->configuration::isLicensed();
    }

    public function isEnabled(): bool
    {
        return LdapConfiguration::isEnabled();
    }

    public function getProfileId(): ?int
    {
        return $this->profileId;
    }

    /**
     * @return string|null
     */
    public function getCustomSettingFilter(): ?string
    {
        try {
            return $this->configuration->getSetting($this->profileId, $this->configuration::SETTING_LDAP_FILTER);
        } catch (isys_exception_database $e) {
            return null;
        }
    }
}
