<?php

namespace SHD\Smview\Connect\Service\Mapping\Transformer;

use SHD\Smview\Connect\DAO\DataMappingDAO;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Mapping\Definition\ApplicationMappingDefinition;
use SHD\Smview\Connect\Service\Mapping\Mapper;
use SHD\Smview\Connect\Service\Mapping\MappingDefinitionInterface;
use SHD\Smview\Connect\Service\Mapping\MappingTransformerInterface;
use isys_application;
use isys_cmdb_dao_object;
use isys_component_database;
use isys_exception_database;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class ApplicationMappingTransformer implements MappingTransformerInterface
{
    public const APPLICATION = 'application';

    /**
     * @param ApplicationMappingDefinition $defintion
     * @param string $mode
     * @param array $parameters
     * @return ApplicationMappingDefinition
     * @throws isys_exception_database
     */
    public function handle(MappingDefinitionInterface $defintion, string $mode = Mapper::MODE_DEFAULT, array $parameters = []): MappingDefinitionInterface
    {
        $defintion->appId = $this->getAppId($this->map(self::APPLICATION, $defintion->title));

        return $defintion;
    }

    public function map(string $type, string $str): string
    {
        $mapping = DaoUtil::allRows(DataMappingDAO::getInstance()->get_data($type, $str))[0] ?? null;

        if($mapping != null) {
            return $mapping['target'];
        }

        return $str;
    }

    /**
     * @param string $title
     * @return int|null
     * @throws isys_exception_database
     */
    private function getAppId(string $title): ?int
    {
        /** @var isys_component_database $db */
        $db = isys_application::instance()->container->get('database');
        $dao = isys_cmdb_dao_object::instance($db);

        $const = 'C__CATG__GLOBAL';
        $cat = $dao->get_cat_by_const($const);
        $daoCat = DaoUtil::categoryDaoFactory('catgID', $const, $cat['id']);

        $assignment['property_key'] = 'title';
        $assignment['category'] = $cat;
        $assignment['dao'] = $daoCat;

        $obj = DaoUtil::allRows(DaoUtil::getObjectsByTypeAndCategoryFields(C__OBJTYPE__APPLICATION, 2, [[
            'assignment' => $assignment,
            'comparator' => '=',
            'value' => $title,
        ]]))[0] ?? null;

        if(!$obj) {
            return null;
        }

        return $obj['isys_obj__id'];
    }
}
