<?php

namespace SHD\Smview\Connect\Service\Mapping\Transformer;

use SHD\Smview\Connect\DAO\DataMappingDAO;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Mapping\Definition\CostCenterMappingDefinition;
use SHD\Smview\Connect\Service\Mapping\Definition\NetworkProviderMappingDefinition;
use SHD\Smview\Connect\Service\Mapping\Mapper;
use SHD\Smview\Connect\Service\Mapping\MappingDefinitionInterface;
use SHD\Smview\Connect\Service\Mapping\Transformer;
use isys_application;
use isys_cmdb_dao_object;
use isys_component_database;
use isys_exception_dao;
use isys_exception_database;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class CostCenterMappingTransformer extends Transformer
{
    public const COST_CENTER = 'cost_center';

    /**
     * @param CostCenterMappingDefinition $defintion
     * @param string $mode
     * @param array $parameters
     * @return MappingDefinitionInterface
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    public function handle(MappingDefinitionInterface $defintion, string $mode = Mapper::MODE_DEFAULT, array $parameters = []): MappingDefinitionInterface
    {
        $defintion->costCenterId = ($mode == Mapper::MODE_CUSTOM_DIALOGPLUS)
            ? $this->getOrCreateCustomDialogPlusValue($parameters['identifier'], $this->map(self::COST_CENTER, $defintion->number))
            : $this->getOrCreateCostCenterId($this->map(self::COST_CENTER, $defintion->number));

        return $defintion;
    }

    public function map(string $type, string $str): string
    {
        $mapping = DaoUtil::allRows(DataMappingDAO::getInstance()->get_data($type, $str))[0] ?? null;

        if($mapping != null) {
            return $mapping['target'];
        }

        return $str;
    }

    /**
     * @param string $number
     * @return int
     * @throws isys_exception_dao
     * @throws isys_exception_database
     */
    private function getOrCreateCostCenterId(string $number): int
    {
        /** @var isys_component_database $db */
        $db = isys_application::instance()->container->get('database');
        $dao = isys_cmdb_dao_object::instance($db);

        $id = DaoUtil::allRows($dao->retrieve(
            <<<SQL
SELECT `isys_account__id` `id` FROM `isys_account` WHERE `isys_account__title` = {$dao->convert_sql_text($number)}
SQL
        ))[0]['id'] ?? null;

        if(!$id) { //create
            $dao->update(<<<SQL
INSERT INTO `isys_account`(`isys_account__title`, `isys_account__status`) VALUES ({$dao->convert_sql_text($number)}, 2)
SQL) && $dao->apply_update();
            $id = $dao->get_last_insert_id();
        }

        return $id;
    }
}
