<?php

namespace SHD\Smview\Connect\Service\Mapping\Transformer;

use SHD\Smview\Connect\DAO\DataMappingDAO;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Mapping\Definition\ModelMappingDefinition;
use SHD\Smview\Connect\Service\Mapping\Mapper;
use SHD\Smview\Connect\Service\Mapping\MappingDefinitionInterface;
use SHD\Smview\Connect\Service\Mapping\MappingTransformerInterface;
use isys_application;
use isys_cmdb_dao_object;
use isys_component_database;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class ModelMappingTransformer implements MappingTransformerInterface
{
    public const MANUFACTURER = 'manufacturer';
    public const MODEL = 'model';

    /**
     * @param ModelMappingDefinition $defintion
     * @param string $mode
     * @param array $parameters
     * @return ModelMappingDefinition
     */
    public function handle(MappingDefinitionInterface $defintion, string $mode = Mapper::MODE_DEFAULT, array $parameters = []): MappingDefinitionInterface
    {
        $defintion->manufacturerId = $this->getOrCreateModelManufacturerId($this->map(self::MANUFACTURER, $defintion->manufacturer));
        $defintion->titleId = $this->getOrCreateModelTitleId($this->map(self::MODEL, $defintion->title), $defintion->manufacturerId);

        return $defintion;
    }

    public function map(string $type, string $str): string
    {
        $mapping = DaoUtil::allRows(DataMappingDAO::getInstance()->get_data($type, $str))[0] ?? null;

        if($mapping != null) {
            return $mapping['target'];
        }

        return $str;
    }

    private function getOrCreateModelTitleId(string $title, int $manufacturerId): int
    {
        /** @var isys_component_database $db */
        $db = isys_application::instance()->container->get('database');
        $dao = isys_cmdb_dao_object::instance($db);

        $id = DaoUtil::allRows($dao->retrieve(
            <<<SQL
SELECT `isys_model_title__id` `id` FROM `isys_model_title` WHERE `isys_model_title__title` = {$dao->convert_sql_text($title)} and `isys_model_title__isys_model_manufacturer__id` = {$dao->convert_sql_id($manufacturerId)}
SQL
        ))[0]['id'] ?? null;

        if(!$id) { //create
            $dao->update(<<<SQL
INSERT INTO `isys_model_title`(`isys_model_title__title`, `isys_model_title__status`, `isys_model_title__isys_model_manufacturer__id`) VALUES ({$dao->convert_sql_text($title)}, {$dao->convert_sql_id(2)}, {$dao->convert_sql_id($manufacturerId)})
SQL) && $dao->apply_update();
            $id = $dao->get_last_insert_id();
        }

        return $id;
    }

    private function getOrCreateModelManufacturerId(string $title): int
    {
        /** @var isys_component_database $db */
        $db = isys_application::instance()->container->get('database');
        $dao = isys_cmdb_dao_object::instance($db);

        $id = DaoUtil::allRows($dao->retrieve(
            <<<SQL
SELECT `isys_model_manufacturer__id` `id` FROM `isys_model_manufacturer` WHERE `isys_model_manufacturer__title` = {$dao->convert_sql_text($title)}
SQL
        ))[0]['id'] ?? null;

        if(!$id) { //create
            $dao->update(<<<SQL
INSERT INTO `isys_model_manufacturer`(`isys_model_manufacturer__title`, `isys_model_manufacturer__status`) VALUES ({$dao->convert_sql_text($title)}, 2)
SQL) && $dao->apply_update();
            $id = $dao->get_last_insert_id();
        }

        return $id;
    }
}
