<?php

namespace SHD\Smview\Connect\Service\Mapping\Transformer;

use SHD\Smview\Connect\DAO\DataMappingDAO;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Mapping\Definition\OSMappingDefinition;
use SHD\Smview\Connect\Service\Mapping\Mapper;
use SHD\Smview\Connect\Service\Mapping\MappingDefinitionInterface;
use SHD\Smview\Connect\Service\Mapping\MappingTransformerInterface;
use isys_application;
use isys_cmdb_dao_object;
use isys_component_database;
use isys_exception_database;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class OSMappingTransformer implements MappingTransformerInterface
{
    public const OS = 'os';

    /**
     * @param OSMappingDefinition $defintion
     * @param string $mode
     * @param array $parameters
     * @return OSMappingDefinition
     * @throws isys_exception_database
     */
    public function handle(MappingDefinitionInterface $defintion, string $mode = Mapper::MODE_DEFAULT, array $parameters = []): MappingDefinitionInterface
    {
        $defintion->applicationId = $this->getOrCreateApplicationId($this->map(self::OS, $defintion->name));

        return $defintion;
    }

    public function map(string $type, string $str): string
    {
        $mapping = DaoUtil::allRows(DataMappingDAO::getInstance()->get_data($type, $str))[0] ?? null;

        if($mapping != null) {
            return $mapping['target'];
        }

        return $str;
    }

    /**
     * @param string $title
     * @return int
     * @throws isys_exception_database
     */
    private function getOrCreateApplicationId(string $title): int
    {
        /** @var isys_component_database $db */
        $db = isys_application::instance()->container->get('database');
        $dao = isys_cmdb_dao_object::instance($db);
        $objTypeId = defined_or_default('C__OBJTYPE__OPERATING_SYSTEM');

        $app = DaoUtil::allRows($dao->get_objects_by_type_id($objTypeId, 2, null, $title))[0] ?? null;

        if(!$app) {
            return $dao->create_object($title, $objTypeId);
        }

        return $app['isys_obj__id'];
    }
}
