<?php

namespace SHD\Smview\Connect\Service\Rest;

use Exception;

class InstallManager
{
    public const FRONT_CONTROLLER_PATH = '/src/rest.php';
    public const FRONT_CONTROLLER_TEMPLATE_PATH = '/src/classes/modules/shd_smview_connect/rest_index_template.php';
    public const HTACCESS_PATH = '/.htaccess';

    public static function installFrontController(): void
    {
        copy(self::getFrontControllerTemplatePath(), self::getFrontControllerPath());
    }

    public static function hasFrontController(): bool
    {
        return file_exists(self::getFrontControllerPath());
    }

    public static function isFrontControllerUpToDate(): bool
    {
        return sha1_file(self::getFrontControllerPath()) === sha1_file(self::getFrontControllerTemplatePath());
    }

    /**
     * @return string
     */
    public static function getFrontControllerPath(): string
    {
        return \isys_module_shd_smview_connect::getIdoitDir().self::FRONT_CONTROLLER_PATH;
    }

    /**
     * @return string
     */
    public static function getFrontControllerTemplatePath(): string
    {
        return \isys_module_shd_smview_connect::getIdoitDir().self::FRONT_CONTROLLER_TEMPLATE_PATH;
    }

    /**
     * @return bool
     */
    public static function hasHtaccessExemption(): bool
    {
        $ht = file_get_contents(self::getHtaccessPath());

        $out = [];
        preg_match_all('/<Files "rest.php">.*\n.*Require all granted.*\n.*<\/Files>/', $ht, $out);

        return count(@$out[0]) == 1;
    }

    /**
     * @return void
     * @throws Exception
     */
    public static function installHtaccessExemption(): void
    {
        $ht = file_get_contents(self::getHtaccessPath());

        $out = [];
        preg_match_all('/<Files "jsonrpc.php">.*\n.*Require all granted.*\n.*<\/Files>/', $ht, $out);

        if(count(@$out[0]) != 1) {
            throw new Exception(sprintf('block %s not found or multiple matches in %s', $b, self::getHtaccessPath()));
        }

        $ht = preg_replace('/<Files "jsonrpc.php">.*\n.*Require all granted.*\n.*<\/Files>/', '<Files "jsonrpc.php">'.PHP_EOL.'Require all granted'.PHP_EOL.'</Files>'.PHP_EOL.PHP_EOL.'<Files "rest.php">'.PHP_EOL.'Require all granted'.PHP_EOL.'</Files>', $ht);

        file_put_contents(self::getHtaccessPath(), $ht);
    }

    /**
     * @return string
     */
    public static function getHtaccessPath(): string
    {
        return \isys_module_shd_smview_connect::getIdoitDir().self::HTACCESS_PATH;
    }
}
