<?php

namespace SHD\Smview\Connect\Service\Rest;

use SHD\Smview\Connect\Service\ConnectLogger;
use SHD\Smview\Connect\Service\idoit\IdoitApiClient;
use Monolog\Logger;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Throwable;

class RestCategory implements CrudClientInterface
{
    private IdoitApiClient $apiClient;

    private ConnectLogger $logger;

    public function __construct(IdoitApiClient $apiClient, ConnectLogger $logger)
    {
        $this->apiClient = $apiClient;
        $this->logger = $logger;
    }

    /**
     * @throws RestApiException
     */
    public function collection(Request $request, ?string $constant = null)
    {
        throw new RestApiException(sprintf('listing categories is not implemented'));
    }

    /**
     * @param Request $request
     * @param string $constant
     * @param int $objectId
     * @return RestApiResponse
     * @throws ClientExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ServerExceptionInterface
     * @throws Throwable
     * @throws TransportExceptionInterface
     */
    public function object(Request $request, string $constant, int $objectId): RestApiResponse
    {
        /** @var \isys_component_database $db */
        $db = \isys_application::instance()->container->get('database');
        $dao = \isys_cmdb_dao_object::instance($db);

        $cat = $dao->get_cat_by_const($constant);

        $parameters = [];

        $parameters['objID'] = $objectId;

        if(stripos($constant, 'C__CATG') !== false) {
            $parameters['catgID'] = $cat['id'];
        } else {
            $parameters['catsID'] = $cat['id'];
        }

        try {
            $response = $this->apiClient->post('cmdb.categories.read', $parameters);
            $content = json_decode($response->getContent());
            $category = $content->result;

            return new RestApiResponse($category);
        } catch (Throwable $exception) {
            $this->logger->log($exception->getMessage(), 'Rest Wrapper', Logger::ERROR);

            throw $exception;
        }
    }

    /**
     * @throws RestApiException
     */
    public function create(Request $request)
    {
        throw new RestApiException(sprintf('creating categories is not implemented'));
    }

    /**
     * @throws RestApiException
     */
    public function update(Request $request)
    {
        throw new RestApiException(sprintf('updating categories is not implemented'));
    }

    /**
     * @throws RestApiException
     */
    public function delete(Request $request)
    {
        throw new RestApiException(sprintf('deleting categories is not implemented'));
    }
}
