<?php

namespace SHD\Smview\Connect\Service\Telekom\Auth;

use DateTime;
use SHD\Smview\Connect\Service\Auth\TokenInterface;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class TelekomToken implements TokenInterface
{
    /**
     * @var string
     */
    public $accessToken;

    /**
     * @var int
     */
    public $expiresIn;

    /**
     * @var int
     */
    public $refreshExpiresIn;

    /**
     * @var string
     */
    public $refreshToken;

    /**
     * @var string
     */
    public $tokenType;

    /**
     * @var string
     */
    public $sessionState;

    /**
     * @var string
     */
    public $scope;

    /**
     * @var DateTime
     */
    public $expiresOn;

    /**
     * @var DateTime
     */
    public $refreshExpiresOn;

    /**
     * @var int
     */
    public $profileId;

    /**
     * @return bool
     */
    public function isExpired(): bool
    {
        return $this->expiresOn < (new DateTime());
    }

    /**
     * @return bool
     */
    public function isRefeshExpired(): bool
    {
        return $this->refreshExpiresOn < (new DateTime());
    }

    /**
     * @param string $json
     * @return TokenInterface
     */
    public static function fromJson(string $json): TokenInterface
    {
        $data = json_decode($json);

        $token = new self();
        $token->accessToken = $data->access_token;
        $token->refreshToken = $data->refresh_token;
        $token->expiresIn = $data->expires_in;
        $token->refreshExpiresIn = $data->refresh_expires_in;
        $token->tokenType = $data->token_type;
        $token->sessionState = $data->session_state;
        $token->scope = $data->scope;

        $token->expiresOn = (new DateTime())->modify(sprintf('+%s seconds', $token->expiresIn));
        $token->refreshExpiresOn = (new DateTime())->modify(sprintf('+%s seconds', $token->refreshExpiresIn));

        return $token;
    }

    /**
     * @param int $profileId
     * @return $this
     */
    public function setProfileId(int $profileId): TelekomToken
    {
        $this->profileId = $profileId;

        return $this;
    }

    public function setId(string $id): TokenInterface
    {
        $this->setProfileId(intval($id));

        return $this;
    }

    public function getName(): string
    {
        return 'telekom_token';
    }

    public function getId(): string
    {
        return $this->profileId;
    }

    public function getAccessToken(): string
    {
        return $this->accessToken;
    }
}
