<?php

namespace SHD\Smview\Connect\Service;

use Exception;
use SHD\Smview\Connect\Service\Aeneis\AeneisConfiguration;
use SHD\Smview\Connect\Service\Azure\AzureConfiguration;
use SHD\Smview\Connect\Service\Baramundi\BaramundiConfiguration;
use SHD\Smview\Connect\Service\Gcache\GcacheConfiguration;
use SHD\Smview\Connect\Service\Ldap\LdapConfiguration;
use SHD\Smview\Connect\Service\Telekom\TelekomConfiguration;
use SHD\Smview\Connect\Service\Vsphere\VsphereConfiguration;
use idoit\Tree\Node;
use isys_application;
use isys_component_tree;
use isys_module_shd_smview_connect;
use isys_register;

/**
 * Tree Generator
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Florian Weber <florian.weber@shd-online.de>
 * @copyright  2022 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class TreeGenerator
{
    public const TREE_MODE_DEFAULT = 'default';

    /**
     * Generates the menu tree
     *
     * @param string $treeMode
     * @param isys_register $p_request
     * @param isys_application $p_application
     * @param isys_component_tree $p_tree
     * @return Node
     * @throws Exception
     */
    public static function getTree(string $treeMode, isys_register $p_request, isys_application $p_application, isys_component_tree $p_tree): Node
    {
        $language = $p_application->container->get('language');
        $license = new ShdLicense();

        switch($treeMode) {
            case self::TREE_MODE_DEFAULT: {
                //root
                $tree = new Node(
                    $language->get('LC__CMDB__SMVIEW_CONNECT__TREE_TITLE'),
                    $p_request->get('BASE').'shd_smview_connect',
                    isys_module_shd_smview_connect::get_www_dir().'theme/img/icons/smview-connect.png'
                );

                //ldap adapter
                if (LdapConfiguration::isLicensed() && LdapConfiguration::isEnabled()) {
                    $tree->add(new Node(
                        $language->get("LC__CMDB__SMVIEW_CONNECT__ADAPTER_LDAP"),
                        $p_request->get('BASE').'shd_smview_connect/ldap/settings',
                        isys_module_shd_smview_connect::get_www_dir().'theme/img/icons/ldap.png'
                    ));
                }

                //vsphere adapter
                if (VsphereConfiguration::isLicensed() && VsphereConfiguration::isEnabled()) {
                    $tree->add(new Node(
                        $language->get("LC__CMDB__SMVIEW_CONNECT__ADAPTER_VSPHERE"),
                        $p_request->get('BASE').'shd_smview_connect/vsphere/settings',
                        isys_module_shd_smview_connect::get_www_dir().'theme/img/icons/vsphere.png'
                    ));
                }

                //telekom adapter
                if (TelekomConfiguration::isLicensed() && TelekomConfiguration::isEnabled()) {
                    $telekomNode = new Node(
                        $language->get("LC__CMDB__SMVIEW_CONNECT__ADAPTER_TELEKOM"),
                        $p_request->get('BASE').'shd_smview_connect/telekom/settings',
                        isys_module_shd_smview_connect::get_www_dir().'theme/img/icons/telekom.png'
                    );

                    $telekomNode->add(new Node(
                        $language->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_CERTIFICATE_UPLOAD'),
                        $p_request->get('BASE').'shd_smview_connect/telekom/certUpload',
                        isys_application::instance()->www_path.'images/icons/silk/database_copy.png'
                    ));

                    $telekomNode->add(new Node(
                        $language->get('LC__CMDB__SMVIEW_CONNECT__CONNECT_TELEKOM_TOKEN'),
                        $p_request->get('BASE').'shd_smview_connect/telekom/token',
                        isys_application::instance()->www_path.'images/icons/silk/database_copy.png'
                    ));

                    if(isys_module_shd_smview_connect::isDevMode()) {
                        $telekomNode->add(new Node(
                            'DEV Get Data',
                            $p_request->get('BASE') . 'shd_smview_connect/telekom/devGetData',
                            isys_application::instance()->www_path . 'images/icons/silk/arrow_down.png'
                        ));
                    }

                    $tree->add($telekomNode);
                }

                //baramundi adapter
                if (BaramundiConfiguration::isLicensed() && BaramundiConfiguration::isEnabled()) {
                    $tree->add(new Node(
                        $language->get("LC__CMDB__SMVIEW_CONNECT__ADAPTER_BARAMUNDI"),
                        $p_request->get('BASE').'shd_smview_connect/main/adapterBaramundi',
                        isys_module_shd_smview_connect::get_www_dir().'theme/img/icons/baramundi.png'
                    ));
                }

                //logging node
                $tree->add(new Node(
                    $language->get('LC__CMDB__SMVIEW_CONNECT__LOGGING'),
                    $p_request->get('BASE').'shd_smview_connect/main/logging',
                    isys_application::instance()->www_path.'images/icons/silk/book_open.png'
                ));

                //azure node
                if (AzureConfiguration::isLicensed() && AzureConfiguration::isEnabled()) {
                    $azureNode = new Node(
                        $language->get('LC__CMDB__SMVIEW_CONNECT__AZURE'),
                        $p_request->get('BASE').'shd_smview_connect/azure',
                        isys_module_shd_smview_connect::get_www_dir().'theme/img/icons/azure.png'
                    );

                    $azureNode->add(new Node(
                        $language->get('LC__CMDB__SMVIEW_CONNECT__AZURE_DISCOVERY'),
                        $p_request->get('BASE').'shd_smview_connect/azure/discovery',
                        isys_application::instance()->www_path.'images/icons/silk/chart_graph_blue.png'
                    ));

                    $azureNode->add(new Node(
                        $language->get('LC__CMDB__SMVIEW_CONNECT__AZURE_IMPORT'),
                        $p_request->get('BASE').'shd_smview_connect/azure/import',
                        isys_application::instance()->www_path.'images/icons/silk/add.png'
                    ));

                    $tree->add($azureNode);
                }

                //management node
                $managementNode = new Node(
                    $language->get('LC__CMDB__SMVIEW_CONNECT__MANAGEMENT'),
                    '#',
                    isys_application::instance()->www_path.'images/icons/silk/wrench.png'
                );

                //cache node
                $managementNode->add(new Node(
                    $language->get('LC__CMDB__SMVIEW_CONNECT__CACHE'),
                    $p_request->get('BASE').'shd_smview_connect/main/cacheOverview',
                    isys_application::instance()->www_path.'images/icons/silk/arrow_in_longer.png'
                ));

                if(isys_module_shd_smview_connect::isDevMode()) {
                    //rest api
                    $managementNode->add(new Node(
                        $language->get('LC__CMDB__SMVIEW_CONNECT__REST'),
                        $p_request->get('BASE') . 'shd_smview_connect/rest/settings',
                        isys_application::instance()->www_path . 'images/icons/silk/package.png'
                    ));
                }

                //license node
                $licenseNode = new Node(
                    $language->get('LC__CMDB__SMVIEW_CONNECT__LICENSE'),
                    $p_request->get('BASE').'shd_smview_connect/main/license',
                    isys_module_shd_smview_connect::get_www_dir().'theme/img/icons/license.png'
                );

                // display license generator only if a valid private key is installed
                if ($license->can_create_licenses()) {
                    $licenseNode->add(new Node(
                        $language->get("LC__CMDB__SMVIEW_CONNECT__LICENSE_GENERATOR"),
                        $p_request->get('BASE').'shd_smview_connect/main/licenseGenerator',
                        isys_module_shd_smview_connect::get_www_dir().'theme/img/icons/license.png'
                    ));
                }

                $licenseNode->add(new Node(
                    $language->get("LC__CMDB__SMVIEW_CONNECT__LICENSE_UPLOAD"),
                    $p_request->get('BASE').'shd_smview_connect/main/licenseUpload',
                    isys_application::instance()->www_path.'images/icons/silk/database_copy.png'
                ));

                $managementNode->add($licenseNode);

                //value pipeline
                if(isys_module_shd_smview_connect::isDevMode()) {
                    $vpNode = new Node(
                        $language->get('LC__CMDB__SMVIEW_CONNECT__VALUE_PIPELINE'),
                        $p_request->get('BASE') . 'shd_smview_connect/valuepipeline/functions',
                        isys_application::instance()->www_path . 'images/icons/silk/table.png'
                    );

                    $vpNode->add(new Node(
                        $language->get("LC__CMDB__SMVIEW_CONNECT__VALUE_PIPELINE_FUNCTIONS"),
                        $p_request->get('BASE') . 'shd_smview_connect/valuepipeline/functions',
                        isys_application::instance()->www_path . 'images/icons/silk/textfield.png'
                    ));

                    $managementNode->add($vpNode);
                }

                $tree->add($managementNode);

                //aeneis node
                if (AeneisConfiguration::isLicensed() && AeneisConfiguration::isEnabled()) {
                    $aeneisNode = new Node(
                        $language->get('LC__CMDB__SMVIEW_CONNECT__AENEIS_TITLE'),
                        '#',
                        isys_module_shd_smview_connect::get_www_dir().'theme/img/icons/aeneis.png'
                    );

                    //aeneis import
                    $aeneisNode->add(new Node(
                        $language->get('LC__CMDB__SMVIEW_CONNECT__AENEIS_IMPORT_TITLE'),
                        $p_request->get('BASE') . 'shd_smview_connect/aeneis/import',
                        isys_application::instance()->www_path.'images/icons/silk/arrow_left.png'
                    ));

                    //aeneis export
                    $aeneisNode->add(new Node(
                        $language->get('LC__CMDB__SMVIEW_CONNECT__AENEIS_EXPORT_TITLE'),
                        $p_request->get('BASE').'shd_smview_connect/aeneis/export',
                        isys_application::instance()->www_path.'images/icons/silk/arrow_right.png'
                    ));

                    //aeneis config
                    $aeneisNode->add(new Node(
                        $language->get('LC__CMDB__SMVIEW_CONNECT__AENEIS_CONFIG_TITLE'),
                        $p_request->get('BASE').'shd_smview_connect/aeneis/config',
                        isys_application::instance()->www_path.'images/icons/silk/cog.png'
                    ));

                    $tree->add($aeneisNode);
                }

                //gcache node
                if(GcacheConfiguration::isEnabled()) {
                    $gcacheNode = new Node(
                        $language->get('LC__CMDB__SMVIEW_CONNECT__GCACHE_TITLE'),
                        $p_request->get('BASE') . 'shd_smview_connect/gcache/index',
                        isys_module_shd_smview_connect::get_www_dir().'theme/img/icons/gcache.png'
                    );

                    $gcacheNode->add(new Node(
                        $language->get('LC__CMDB__SMVIEW_CONNECT__CSV_READ'),
                        $p_request->get('BASE') . 'shd_smview_connect/gcache/index',
                        isys_application::instance()->www_path.'images/icons/silk/zoom.png'
                    ));

                    $gcacheNode->add(new Node(
                        $language->get('LC__CMDB__SMVIEW_CONNECT__CSV_IMPORT'),
                        $p_request->get('BASE') . 'shd_smview_connect/gcache/import',
                        isys_application::instance()->www_path.'images/icons/silk/disk.png'
                    ));

                    $tree->add($gcacheNode);
                }

                return $tree;
            }
            default: throw new Exception(sprintf('tree mode %s not implemented', $treeMode));
        }
    }
}
