<?php

namespace SHD\Smview\Connect\Service\Vsphere\Import;

use SHD\Smview\Connect\Service\Import\AttributeEmittingImporterInterface;
use SHD\Smview\Connect\Service\Import\IdoitImportManager;
use SHD\Smview\Connect\Service\ImporterInterface;
use SHD\Smview\Connect\Service\Vsphere\VsphereClient;
use SHD\Smview\Connect\Service\Vsphere\VsphereConfiguration;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
abstract class VsphereImporter implements ImporterInterface, AttributeEmittingImporterInterface
{
    /**
     * @var VsphereClient
     */
    protected VsphereClient $vsphereClient;

    /**
     * @var IdoitImportManager
     */
    protected IdoitImportManager $idoitImportManager;

    /**
     * @var int
     */
    protected int $profileId;

    /**
     * @var VsphereConfiguration
     */
    protected VsphereConfiguration $configuration;

    /**
     * @var int
     */
    protected int $limit;

    /**
     * @var string|null
     */
    protected ?string $foldersList;

    /**
     * @param VsphereClient $vsphereClient
     * @param IdoitImportManager $idoitImportManager
     * @param VsphereConfiguration $configuration
     * @param int $profileId
     * @param int $limit
     * @param string|null $foldersList
     */
    public function __construct(VsphereClient $vsphereClient, IdoitImportManager $idoitImportManager, VsphereConfiguration $configuration, int $profileId, int $limit = 0, ?string $foldersList = null)
    {
        $this->vsphereClient = $vsphereClient;
        $this->idoitImportManager = $idoitImportManager;
        $this->profileId = $profileId;
        $this->configuration = $configuration;
        $this->limit = $limit;
        $this->foldersList = $foldersList;
    }

    public function status(?int $profileId = null): bool
    {
        return $this->vsphereClient->isAuthenticatedAndAccessible();
    }

    public function isLicensed(): bool
    {
        return $this->configuration::isLicensed();
    }

    public function getName(): string
    {
        return 'vsphere';
    }

    public function isEnabled(): bool
    {
        return VsphereConfiguration::isEnabled();
    }

    public function getProfileId(): ?int
    {
        return $this->profileId;
    }

    public function getAttributes(): array
    {
        return [];
    }
}
