<?php

namespace SHD\Smview\Connect\Service\idoit;

use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;
use Symfony\Contracts\HttpClient\ResponseInterface;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class IdoitApiClient
{
    private HttpClientInterface $httpClient;
    private ?string $idoitApiUrl;
    private ?string $idoitUsername;
    private ?string $idoitPassword;
    private ?string $idoitApiKey;

    public function __construct(HttpClientInterface $httpClient, ?string $idoitApiUrl, ?string $idoitUsername, ?string $idoitPassword, ?string $idoitApiKey)
    {
        $this->httpClient = $httpClient;
        $this->idoitApiUrl = $idoitApiUrl.'src/jsonrpc.php';
        $this->idoitUsername = $idoitUsername;
        $this->idoitPassword = $idoitPassword;
        $this->idoitApiKey = $idoitApiKey;
    }

    /**
     * @param string $method
     * @param array $parameters
     * @param array $headers
     * @return ResponseInterface
     * @throws TransportExceptionInterface
     */
    public function post(string $method, array $parameters = [], array $headers = []): ResponseInterface
    {
        $parameters = array_merge($parameters, [
            'apikey' => $this->idoitApiKey,
        ]);

        $headers = array_merge($headers, [
            'Content-Type' => 'application/json',
        ]);

        $body = json_encode([
            'id' => 1,
            'jsonrpc' => '2.0',
            'version' => '2.0',
            'method' => $method,
            'params' => $parameters,
        ]);

        return $this->httpClient->request('POST', $this->idoitApiUrl, [
            'query' => [
            ],
            'auth_basic' => [$this->idoitUsername, $this->idoitPassword],
            'body' => $body,
            'headers' => $headers,
        ]);
    }

    public function getClient(): HttpClientInterface
    {
        return $this->httpClient;
    }
}
