<?php

namespace idoit\Module\Shdsmviewconnect\Controller;

use Exception;
use idoit\Controller\Base as BaseController;
use idoit\Model\Dao\Base as BaseDao;
use Monolog\Logger;
use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\Service\ConnectLogger;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\TreeGenerator;
use idoit\Tree\Node;
use isys_application;
use isys_auth;
use isys_auth_auth;
use isys_component_dao;
use isys_component_template_navbar;
use isys_component_tree;
use isys_controller;
use isys_exception_database;
use isys_helper_link;
use isys_module;
use isys_module_shd_smview_connect;
use isys_register;
use isys_tenantsettings;
use SHD\Smview\DB\Schema;

/**
 * Abstract Addon Controller
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Florian Weber <florian.weber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
abstract class AddonController extends BaseController implements isys_controller
{
    protected ?string $profileString = null;
    protected ?string $moduleString = null;

    /**
     * @var isys_component_template_navbar
     */
    protected $navbar;

    public function __construct(isys_module $p_module)
    {
    }

    /**
     * @param isys_application $p_application
     * @return BaseDao|isys_component_dao
     * @throws Exception
     */
    public function dao(isys_application $p_application)
    {
        return BaseDao::instance($p_application->container->get('database'));
    }

    /**
     * Will be executed before every action
     *
     * @param isys_register $request
     * @param BaseController $controller
     * @return void
     * @throws isys_exception_database
     */
    public function pre(isys_register $request, BaseController $controller)
    {
        $this->navbar = isys_component_template_navbar::getInstance();
        $canEdit = isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');

        if (
            $_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__EDIT') ||
            $_POST[C__CMDB__GET__EDITMODE] &&
            $_POST[C__CMDB__GET__EDITMODE] == defined_or_default('C__EDITMODE__ON')
        ) {
            $this->navbar
                ->set_selected($canEdit, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
                ->set_active($canEdit, defined_or_default('C__NAVBAR_BUTTON__SAVE'))
                ->set_active($canEdit, defined_or_default('C__NAVBAR_BUTTON__CANCEL'))
            ;
        } else {
            $this->navbar
                ->set_active($canEdit, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
                ->set_visible($canEdit, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
            ;
        }

        //wenn noch kein profil ausgewählt wurde dann automaitsch default holen
        if(!is_null($this->profileString) && !is_null($this->moduleString)) {
            $profile_id = isys_tenantsettings::get($this->profileString, null);
            if (!$profile_id) {
                $profiles = DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, $this->moduleString)) ?? [];

                if (count($profiles) > 0) {
                    isys_tenantsettings::set($this->profileString, $profiles[0]['id']);
                }
            }
        }

        isys_application::instance()->template
            ->assign('ajax_url', isys_helper_link::create_url([C__GET__AJAX => 1, C__GET__AJAX_CALL => 'connect_connect']))
            ->assign('spinner_url', isys_module_shd_smview_connect::getAppDir() . '/images/ajax-loading.gif')
        ;
    }

    /**
     * @param string $profileString
     * @param string $moduleString
     * @return $this
     */
    protected function setup(string $profileString, string $moduleString): AddonController
    {
        $this->profileString = $profileString;
        $this->moduleString = $moduleString;

        return $this;
    }

    /**
     * @param isys_register $p_request
     * @param isys_application $p_application
     * @param isys_component_tree $p_tree
     * @return Node
     * @throws Exception
     */
    public function tree(isys_register $p_request, isys_application $p_application, isys_component_tree $p_tree): Node
    {
        return TreeGenerator::getTree(TreeGenerator::TREE_MODE_DEFAULT, $p_request, $p_application, $p_tree);
    }

    /**
     * Check DB Schema
     *
     * @param array $modules
     * @return bool
     * @throws isys_exception_database
     */
    protected function checkDbSchema(array $modules): bool
    {
        $schema = Schema::load(isys_application::instance()->container->get('database'), isys_module_shd_smview_connect::get_module_dir().'/src/addon/DB/schema.yml');
        $message = $schema->check($modules);

        if(!is_null($message)) {
            /** @var ConnectLogger $logger */
            $logger = isys_application::instance()->container->get('smview_connect.logger');
            $logger->log($message, 'Telekom', Logger::ERROR);

            throw new Exception($message);
        }

        return true;
    }
}
