<?php

namespace idoit\Module\Shdsmviewconnect\Controller;

use Exception;
use SHD\Smview\Connect\Service\Aeneis\AeneisConfiguration;
use SHD\Smview\Connect\Service\TreeGenerator;
use idoit\Module\Shdsmviewconnect\View\Aeneis\ConfigView;
use idoit\Module\Shdsmviewconnect\View\Aeneis\ExportView;
use idoit\Module\Shdsmviewconnect\View\Aeneis\ImportView;
use idoit\Module\Shdsmviewconnect\View\Aeneis\IndexView;
use idoit\View\Renderable;
use isys_application;
use isys_auth;
use isys_auth_auth;
use isys_component_dao;
use isys_component_template_navbar;
use isys_component_tree;
use isys_controller;
use isys_helper_link;
use isys_module;
use isys_module_shd_smview_connect;
use isys_register;
use idoit\Controller\Base as BaseController;
use idoit\Model\Dao\Base as BaseDao;
use idoit\Tree\Node;

/**
 * Aeneis Controller
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Steve Gröber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class Aeneis extends BaseController implements isys_controller
{
    /**
     * @var isys_component_template_navbar
     */
    private $navbar;

    public function __construct(isys_module $p_module)
    {
    }

    /**
     * @param isys_application $p_application
     * @return BaseDao|isys_component_dao
     * @throws Exception
     */
    public function dao(isys_application $p_application)
    {
        return BaseDao::instance($p_application->container->get('database'));
    }

    /**
     * @route   /shd_smview_connect
     * @route   /shd_smview_connect/main
     * @route   /shd_smview_connect/main/handle  But will result in error, because different second parameter.
     *
     * @param isys_register $p_request
     * @param isys_application $p_application
     * @return Renderable
     * @throws Exception
     */
    public function handle(isys_register $p_request, isys_application $p_application): Renderable
    {
        return $this->index($p_request);
    }

    /**
     * @param isys_register $p_request
     * @param isys_application $p_application
     * @param isys_component_tree $p_tree
     * @return Node
     * @throws Exception
     */
    public function tree(isys_register $p_request, isys_application $p_application, isys_component_tree $p_tree): Node
    {
        return TreeGenerator::getTree(TreeGenerator::TREE_MODE_DEFAULT, $p_request, $p_application, $p_tree);
    }

    /**
     * Will be executed before every action
     *
     * @param isys_register $request
     * @param BaseController $controller
     * @return void
     */
    public function pre(isys_register $request, BaseController $controller)
    {
        $this->navbar = isys_component_template_navbar::getInstance();
        $canEdit = isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');

        if (
            $_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__EDIT') ||
            $_POST[C__CMDB__GET__EDITMODE] &&
            $_POST[C__CMDB__GET__EDITMODE] == defined_or_default('C__EDITMODE__ON')
        ) {
            $this->navbar
                ->set_selected($canEdit, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
                ->set_active($canEdit, defined_or_default('C__NAVBAR_BUTTON__SAVE'))
                ->set_active($canEdit, defined_or_default('C__NAVBAR_BUTTON__CANCEL'))
            ;
        } else {
            $this->navbar
                ->set_active($canEdit, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
                ->set_visible($canEdit, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
            ;
        }

        isys_application::instance()->template
            ->assign('ajax_url', isys_helper_link::create_url([C__GET__AJAX => 1, C__GET__AJAX_CALL => 'connect_connect']))
            ->assign('spinner_url', isys_module_shd_smview_connect::getAppDir() . '/images/ajax-loading.gif')
        ;
    }

    /**
     * @route /shd_smview_connect/aeneis/index
     *
     * @param isys_register $request
     * @return Renderable
     * @throws Exception
     */
    public function index(isys_register $request): Renderable
    {
        return new IndexView($request);
    }

    /**
     * @route /shd_smview_connect/aeneis/config
     *
     * @param isys_register $request
     * @return Renderable
     * @throws Exception
     */
    public function config(isys_register $request): Renderable
    {
        if ($_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__SAVE')) {
            /** @var AeneisConfiguration */
            $config = isys_application::instance()->container->get('smview_connect.aeneis.config');
            $config->save_config($_POST);
        }

        $this->navbar->set_save_mode('formsubmit');
        return new ConfigView($request);
    }

    /**
     * @route /shd_smview_connect/aeneis/import
     *
     * @param isys_register $request
     * @return Renderable
     * @throws Exception
     */
    public function import(isys_register $request): Renderable
    {
        $this->navbar->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'));
        return new ImportView($request);
    }

    /**
     * @route /shd_smview_connect/aeneis/export
     *
     * @param isys_register $request
     * @return Renderable
     * @throws Exception
     */
    public function export(isys_register $request): Renderable
    {
        $this->navbar->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'));
        return new ExportView($request);
    }
}
