<?php

namespace idoit\Module\Shdsmviewconnect\Controller;

use Exception;
use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\Service\Azure\AzureConfiguration;
use SHD\Smview\Connect\Service\Azure\Import\ResourceImporter;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\TreeGenerator;
use idoit\Module\Shdsmviewconnect\View\Azure\DiscoveryView;
use idoit\Module\Shdsmviewconnect\View\Azure\ImportView;
use idoit\Module\Shdsmviewconnect\View\Azure\IndexView;
use idoit\Tree\Node;
use idoit\View\Renderable;
use isys_application;
use isys_component_tree;
use isys_module;
use isys_module_shd_smview_connect;
use isys_register;

/**
 * Azure Controller
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Florian Weber <florian.weber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class Azure extends AddonController
{
    public function __construct(isys_module $p_module)
    {
        parent::__construct($p_module);

        $this->setup(isys_module_shd_smview_connect::CONNECT_MODULE_AZURE_PROFILE, isys_module_shd_smview_connect::CONNECT_MODULE_AZURE);
    }

    /**
     * @route   /shd_smview_connect
     * @route   /shd_smview_connect/azure
     * @route   /shd_smview_connect/azure/handle  But will result in error, because different second parameter.
     *
     * @param isys_register $p_request
     * @param isys_application $p_application
     * @return Renderable
     * @throws Exception
     */
    public function handle(isys_register $p_request, isys_application $p_application): Renderable
    {
        return $this->index($p_request);
    }

    /**
     * @param isys_register $p_request
     * @param isys_application $p_application
     * @param isys_component_tree $p_tree
     * @return Node
     * @throws Exception
     */
    public function tree(isys_register $p_request, isys_application $p_application, isys_component_tree $p_tree): Node
    {
        return TreeGenerator::getTree(TreeGenerator::TREE_MODE_DEFAULT, $p_request, $p_application, $p_tree);
    }

    /**
     * @route /shd_smview_connect/azure/index
     *
     * @param isys_register $request
     * @return Renderable
     * @throws Exception
     */
    public function index(isys_register $request): Renderable
    {
        return new IndexView($request);
    }

    /**
     * @route /shd_smview_connect/azure/discovery
     *
     * @param isys_register $request
     * @return Renderable
     * @throws Exception
     */
    public function discovery(isys_register $request): Renderable
    {
        /** @var AzureConfiguration $config */
        $config = isys_application::instance()->container->get('smview_connect.azure.config');

        if ($_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__SAVE')) {
            $config->save_config($_POST);
        }

        return new DiscoveryView($request);
    }

    /**
     * @route /shd_smview_connect/azure/import
     *
     * @param isys_register $request
     * @return Renderable
     * @throws Exception
     */
    public function import(isys_register $request): Renderable
    {
        $this->navbar->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__SAVE'));

        /** @var AzureConfiguration $config */
        $config = isys_application::instance()->container->get('smview_connect.azure.config');

        if ($_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__SAVE')) {
            $config->save_config($_POST);
        }

        return new ImportView($request);
    }
}
