<?php

namespace idoit\Module\Shdsmviewconnect\Controller;

use idoit\Controller\Base as BaseController;
use Exception;
use SHD\Smview\Connect\Service\ConnectLogger;
use SHD\Smview\Connect\Service\Ldap\Importer;
use idoit\Module\Shdsmviewconnect\View\Ldap\SettingsView;
use idoit\View\Renderable;
use isys_application;
use isys_exception_database;
use isys_module;
use isys_module_shd_smview_connect;
use isys_register;
use Monolog\Logger;
use SHD\Smview\DB\Schema;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class Ldap extends AddonController
{
    public function __construct(isys_module $p_module)
    {
        parent::__construct($p_module);

        $this->setup(isys_module_shd_smview_connect::CONNECT_MODULE_LDAP_PROFILE, isys_module_shd_smview_connect::CONNECT_MODULE_LDAP);
    }

    /**
     * @param isys_register $request
     * @param BaseController $controller
     * @return void
     * @throws Exception
     * @throws isys_exception_database
     */
    public function pre(isys_register $request, BaseController $controller)
    {
        parent::pre($request, $controller);

        $this->checkDbSchema(['ldap']);
    }

    /**
     * @param isys_register $p_request
     * @param isys_application $p_application
     * @return Renderable
     */
    public function handle(isys_register $p_request, isys_application $p_application)
    {
        return $this->settings($p_request);
    }

    public function settings(isys_register $request): Renderable
    {
        $this->navbar->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__SAVE'));

        $ldapConfig = isys_application::instance()->container->get('smview_connect.ldap.config');

        if ($_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__SAVE')) {
            $ldapConfig->save_config($_POST);
            $ldapConfig->export_config();
        }

        return new SettingsView($request);
    }

    public function dev(isys_register $request): Renderable
    {
        /** @var Importer $importer */
        //$importer = isys_application::instance()->container->get('smview_connect.ldap.profile_5.computer_importer');
        $importer = isys_application::instance()->container->get('smview_connect.ldap.profile_1.user_importer');

        $importer->import();

        return new SettingsView($request);
    }
}
