<?php

namespace idoit\Module\Shdsmviewconnect\Controller;

use Exception;
use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\Service\AbstractConfiguration;
use SHD\Smview\Connect\Service\Baramundi\BaramundiConfiguration;
use SHD\Smview\Connect\Service\CacheManager;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\ShdLicense;
use SHD\Smview\Connect\Service\Telekom\TelekomConfiguration;
use SHD\Smview\Connect\Service\TreeGenerator;
use idoit\Module\Shdsmviewconnect\View\Main\BaramundiView;
use idoit\Module\Shdsmviewconnect\View\Main\CacheOverviewView;
use idoit\Module\Shdsmviewconnect\View\Main\DevPropsView;
use idoit\Module\Shdsmviewconnect\View\Main\LicenseUploadView;
use idoit\Module\Shdsmviewconnect\View\Main\LicenseView;
use idoit\Module\Shdsmviewconnect\View\Main\TelekomCertificateUploadView;
use idoit\Module\Shdsmviewconnect\View\Main\TelekomTokenView;
use idoit\Module\Shdsmviewconnect\View\Main\TelekomView;
use idoit\Module\Shdsmviewconnect\View\Main\LicenseGeneratorView;
use idoit\Module\Shdsmviewconnect\View\Main\LoggingView;
use idoit\Module\Shdsmviewconnect\View\Main\SettingsView;
use idoit\View\Renderable;
use isys_application;
use isys_auth;
use isys_auth_auth;
use isys_component_dao;
use isys_component_template_navbar;
use isys_component_tree;
use isys_controller;
use isys_exception_database;
use isys_exception_general;
use isys_helper_link;
use isys_module;
use isys_module_shd_smview_connect;
use isys_register;
use idoit\Controller\Base as BaseController;
use idoit\Model\Dao\Base as BaseDao;
use idoit\Tree\Node;
use isys_tenantsettings;

/**
 * Main Controller
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Florian Weber <florian.weber@shd-online.de>
 * @copyright  2022 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class Main extends BaseController implements isys_controller
{
    /**
     * @var isys_component_template_navbar
     */
    private $navbar;

    public function __construct(isys_module $p_module)
    {
    }

    /**
     * @param isys_application $p_application
     * @return BaseDao|isys_component_dao
     * @throws Exception
     */
    public function dao(isys_application $p_application)
    {
        return BaseDao::instance($p_application->container->get('database'));
    }

    /**
     * @route   /shd_smview_connect
     * @route   /shd_smview_connect/main
     * @route   /shd_smview_connect/main/handle  But will result in error, because different second parameter.
     *
     * @param isys_register $p_request
     * @param isys_application $p_application
     * @return Renderable
     * @throws Exception
     */
    public function handle(isys_register $p_request, isys_application $p_application): Renderable
    {
        return $this->settings($p_request);
    }

    /**
     * @param isys_register $p_request
     * @param isys_application $p_application
     * @param isys_component_tree $p_tree
     * @return Node
     * @throws Exception
     */
    public function tree(isys_register $p_request, isys_application $p_application, isys_component_tree $p_tree): Node
    {
        return TreeGenerator::getTree(TreeGenerator::TREE_MODE_DEFAULT, $p_request, $p_application, $p_tree);
    }

    /**
     * Will be executed before every action
     *
     * @param isys_register $request
     * @param BaseController $controller
     * @return void
     */
    public function pre(isys_register $request, BaseController $controller)
    {
        $this->navbar = isys_component_template_navbar::getInstance();
        $canEdit = isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');

        if (
            $_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__EDIT') ||
            $_POST[C__CMDB__GET__EDITMODE] &&
            $_POST[C__CMDB__GET__EDITMODE] == defined_or_default('C__EDITMODE__ON')
        ) {
            $this->navbar
                ->set_selected($canEdit, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
                ->set_active($canEdit, defined_or_default('C__NAVBAR_BUTTON__SAVE'))
                ->set_active($canEdit, defined_or_default('C__NAVBAR_BUTTON__CANCEL'))
            ;
        } else {
            $this->navbar
                ->set_active($canEdit, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
                ->set_visible($canEdit, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
            ;
        }

        isys_application::instance()->template
            ->assign('ajax_url', isys_helper_link::create_url([C__GET__AJAX => 1, C__GET__AJAX_CALL => 'connect_connect']))
            ->assign('spinner_url', isys_module_shd_smview_connect::getAppDir() . '/images/ajax-loading.gif')
        ;
    }

    /**
     * @route /shd_smview_connect/main/settings
     *
     * @param isys_register $request
     * @return Renderable
     * @throws Exception
     */
    public function settings(isys_register $request): Renderable
    {
        $adapters = isys_module_shd_smview_connect::getAdapters();

        if ($_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__SAVE')) {
            AbstractConfiguration::save_config_by_mapping($_POST, AbstractConfiguration::IDOIT_FIELD_MAPPING);
            AbstractConfiguration::save_config_by_mapping($_POST, isys_module_shd_smview_connect::LOGGING_FIELD_MAPPING);

            foreach ($adapters as $key => $adapter) {
                if($_POST[$key] == "1" && !$adapter['enabled']) {
                    call_user_func($adapter['class'].'::enable');
                }

                if($_POST[$key] != "1" && $adapter['enabled']) {
                    call_user_func($adapter['class'].'::disable');
                }
            }
        }

        $adapters = isys_module_shd_smview_connect::getAdapters();

        //config page
        return new SettingsView($request, [
            'adapters' => $adapters,
        ]);
    }

    /**
     * @route /shd_smview_connect/main/cacheOverview
     *
     * @param isys_register $request
     * @return Renderable
     * @throws Exception
     */
    public function cacheOverview(isys_register $request): Renderable
    {
        if($request->get('POST')->get('invalidate') == 1) {
            /** @var CacheManager $cacheManager */
            $cacheManager = isys_application::instance()->container->get('smview_connect.cache');
            $cache = $cacheManager->retreive(
                $request->get('POST')->get('cache'),
                isys_module_shd_smview_connect::getParameter('smview_connect.api_cache_dir'),
            );
            $cacheManager->invalidate($cache);
        }

        $this->navbar
            ->set_active(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
        ;

        return new CacheOverviewView($request);
    }

    /**
     * @route /shd_smview_connect/main/adapterBaramundi
     *
     * @param isys_register $request
     * @return Renderable
     * @throws Exception
     */
    public function adapterBaramundi(isys_register $request): Renderable
    {
        $this->navbar->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__SAVE'));

        /** @var BaramundiConfiguration $baramundiConfig */
        $baramundiConfig = isys_application::instance()->container->get('smview_connect.baramundi.config');

        //wenn noch kein profil ausgewählt wurde dann automaitsch default holen
        $profile_id = isys_tenantsettings::get(isys_module_shd_smview_connect::CONNECT_MODULE_BARAMUNDI_PROFILE, null);
        if(!$profile_id) {
            $profiles = DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, isys_module_shd_smview_connect::CONNECT_MODULE_BARAMUNDI)) ?? [];

            if(count($profiles) > 0) {
                isys_tenantsettings::set(isys_module_shd_smview_connect::CONNECT_MODULE_BARAMUNDI_PROFILE, $profiles[0]['id']);
            }
        }

        if ($_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__SAVE')) {
            $baramundiConfig->save_config($_POST);
        }

        return new BaramundiView($request);
    }

    /**
     * @route /shd_smview_connect/main/logging
     *
     * @param isys_register $request
     * @return Renderable
     */
    public function logging(isys_register $request): Renderable
    {
        return new LoggingView($request);
    }

    /**
     * @route /shd_smview_connect/main/license
     *
     * @param isys_register $request
     * @return Renderable
     * @throws isys_exception_general
     */
    public function license(isys_register $request): Renderable
    {
        $this->navbar
            ->set_active(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
        ;

        return new LicenseView($request);
    }

    /**
     * @route /shd_smview_connect/main/licenseUpload
     *
     * @param isys_register $request
     * @return Renderable
     * @throws isys_exception_general
     * @throws Exception
     */
    public function licenseUpload(isys_register $request): Renderable
    {
        $canEdit = isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');

        $this->navbar
            ->set_active(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
        ;

        if($canEdit) {
            $this->navbar
                ->set_save_mode('formsubmit')
                ->append_button(
                    'LC__CMDB__SMVIEW_CONNECT__NAVBAR_BUTTON_UPLOAD',
                    'upload',
                    [
                        "icon"       => "icons/silk/page_save.png",
                        "js_onclick" => "trigger_submit();", //"document.isys_form.submit();",
                    ]
                )
            ;
        }

        $license = new ShdLicense();

        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            if (isset($_FILES['C__SMVIEW_CONNECT__LICENSE']) && $_FILES['C__SMVIEW_CONNECT__LICENSE']['size'] > 0) {
                if (!isys_auth_auth::instance()->is_allowed_to(isys_auth::CREATE, "MODULE/C__MODULE__SHD_SMVIEW_CONNECT")) {
                    // not authorized
                    throw new Exception('not authorized');
                }

                $filename = $_FILES['C__SMVIEW_CONNECT__LICENSE']['name'];
                if (strpos(strtolower($filename), '.sig') === null) {
                    // invalid file ending
                    throw new Exception('invalid file ending');
                }

                $license->handleLicenseUploadData($_FILES);
            }
        }

        return new LicenseUploadView($request);
    }

    /**
     * @route /shd_smview_connect/main/licenseGenerator
     *
     * @param isys_register $request
     * @return Renderable
     * @throws isys_exception_general
     */
    public function licenseGenerator(isys_register $request): Renderable
    {
        if ($_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__SAVE')) {
            $license = new ShdLicense();
            $path = $license->create_license($_POST);

            header("Content-Description: File Transfer");
            header("Content-Disposition: attachment; filename=license.sig");
            header("Content-Type: application/octet-stream");
            header("Content-Transfer-Encoding: binary");
            header("Content-length: " . filesize($path));
            header("Pragma: no-cache");
            header("Expires: 0");

            // read the file from disk
            readfile($path);
            die;
        }

        return new LicenseGeneratorView($request);
    }

    /**
     * @route /shd_smview_connect/main/dev
     *
     * @param isys_register $request
     * @return Renderable
     */
    public function dev(isys_register $request): Renderable
    {
        //        isys_application::instance()->container->get('smview_connect.baramundi.endpoint_importer')->import();
        isys_application::instance()->container->get('smview_connect.telekom.contract_importer')->import();

        return new LoggingView($request);
    }

    public function showProps(isys_register $request): Renderable
    {
        $class = $_GET['class'];
        $id = $_GET['objid'];

        return new DevPropsView($request, [
            'class' => $class,
            'id' => $id,
        ]);
    }
}
