<?php

namespace idoit\Module\Shdsmviewconnect\Controller;

use idoit\Controller\Base as BaseController;
use Exception;
use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\Service\ConnectLogger;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Telekom\ContractImporter;
use SHD\Smview\Connect\Service\Telekom\Importer;
use SHD\Smview\Connect\Service\Telekom\TelekomConfiguration;
use idoit\Module\Shdsmviewconnect\View\Telekom\DevDataView;
use idoit\Module\Shdsmviewconnect\View\Telekom\CertificateUploadView;
use idoit\Module\Shdsmviewconnect\View\Telekom\TokenView;
use idoit\Module\Shdsmviewconnect\View\Telekom\SettingsView;
use idoit\View\Renderable;
use isys_application;
use isys_auth;
use isys_auth_auth;
use isys_exception_database;
use isys_module;
use isys_module_shd_smview_connect;
use isys_register;
use isys_tenantsettings;
use Monolog\Logger;
use SHD\Smview\DB\Schema;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class Telekom extends AddonController
{
    public function __construct(isys_module $p_module)
    {
        parent::__construct($p_module);

        $this->setup(isys_module_shd_smview_connect::CONNECT_MODULE_TELEKOM_PROFILE, isys_module_shd_smview_connect::CONNECT_MODULE_TELEKOM);
    }

    public function pre(isys_register $request, BaseController $controller)
    {
        parent::pre($request, $controller);

        $this->checkDbSchema(['telekom']);
    }

    /**
     * @throws Exception
     */
    public function handle(isys_register $p_request, isys_application $p_application): Renderable
    {
        return $this->settings($p_request);
    }

    /**
     * @route /shd_smview_connect/telekom/settings
     *
     * @param isys_register $request
     * @return Renderable
     * @throws Exception
     */
    public function settings(isys_register $request): Renderable
    {
        $this->navbar->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__SAVE'));

        /** @var TelekomConfiguration $telekomConfig */
        $telekomConfig = isys_application::instance()->container->get('smview_connect.telekom.config');

        //wenn noch kein profil ausgewählt wurde dann automaitsch default holen
        $profile_id = isys_tenantsettings::get(isys_module_shd_smview_connect::CONNECT_MODULE_TELEKOM_PROFILE, null);
        if(!$profile_id) {
            $profiles = DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, isys_module_shd_smview_connect::CONNECT_MODULE_TELEKOM)) ?? [];

            if(count($profiles) > 0) {
                isys_tenantsettings::set(isys_module_shd_smview_connect::CONNECT_MODULE_TELEKOM_PROFILE, $profiles[0]['id']);
            }
        }

        if ($_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__SAVE')) {
            $telekomConfig->save_config($_POST);
        }

        return new SettingsView($request);
    }

    /**
     * @route /shd_smview_connect/telekom/certUpload
     *
     * @param isys_register $request
     * @return Renderable
     * @throws Exception
     */
    public function certUpload(isys_register $request): Renderable
    {
        $canEdit = isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');

        $this->navbar
            ->set_active(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
        ;

        if($canEdit) {
            $this->navbar
                ->set_save_mode('formsubmit')
                ->append_button(
                    'LC__CMDB__SMVIEW_CONNECT__NAVBAR_BUTTON_UPLOAD',
                    'upload',
                    [
                        "icon"       => "icons/silk/page_save.png",
                        "js_onclick" => "trigger_submit();", //"document.isys_form.submit();",
                    ]
                )
            ;
        }

        /** @var TelekomConfiguration $telekomConfig */
        $telekomConfig = isys_application::instance()->container->get('smview_connect.telekom.config');

        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            //cert upload
            $telekomConfig->saveClientCertificateData($_POST, $_FILES);
        }

        if($_GET['delete']) {
            $profile = DaoUtil::allRows(ProfileDAO::getInstance()->get_data($_GET['delete']))[0] ?? null;

            if(!$profile) {
                throw new \isys_exception_general(sprintf('profile with id "%s" not found', $_GET['delete']));
            }

            $telekomConfig->removeClientCertificateData($profile);
        }

        return new CertificateUploadView($request);
    }

    /**
     * @route /shd_smview_connect/telekom/token
     *
     * @param isys_register $request
     * @return Renderable
     * @throws isys_exception_database
     * @throws Exception
     */
    public function token(isys_register $request): Renderable
    {
        /** @var TelekomConfiguration $telekomConfig */
        $telekomConfig = isys_application::instance()->container->get('smview_connect.telekom.config');

        if(@isset($_GET['remove_token'])) {
            $profileId = $_GET['profile_id'] ?? null;

            $telekomConfig->removeToken($profileId);
        }

        $profiles = DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, isys_module_shd_smview_connect::CONNECT_MODULE_TELEKOM));

        foreach($profiles as $profile) {
            $profile['token'] = $telekomConfig->getToken($profile['id']);
        }

        $profiles = array_map(function ($p) use ($telekomConfig, $request) {
            $p['token'] = $telekomConfig->getToken($p['id']);
            $p['remove_token_url'] = substr($request->get('BASE'), 0, -1).$request->get('REQUEST').'?remove_token=1&profile_id='.$p['id'];

            return $p;
        }, DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, isys_module_shd_smview_connect::CONNECT_MODULE_TELEKOM)) ?? []);

        $view = new TokenView($request);
        $view->profiles = $profiles;

        return $view;
    }

    /**
     * @throws isys_exception_database
     * @throws Exception
     * @throws TransportExceptionInterface
     */
    public function devGetData(isys_register $request): Renderable
    {
        $canEdit = isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');

        $this->navbar
            ->set_active(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
        ;

        if($canEdit) {
            $this->navbar
                ->set_save_mode('formsubmit')
                ->append_button(
                    'LC__CMDB__SMVIEW_CONNECT__NAVBAR_BUTTON_FETCH',
                    'fetch',
                    [
                        "icon"       => "icons/silk/arrow_down.png",
                        "js_onclick" => "trigger_submit();", //"document.isys_form.submit();",
                    ]
                )
            ;
        }

        $view = new DevDataView($request);

        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            $profileId = @$_POST["C__SMVIEW_CONNECT__PROFILE"] ?? 0;

            if($profileId <= 0) {
                throw new Exception('no profile selected');
            }

            /** @var ContractImporter $importer */
            $importer = isys_application::instance()->container->get('smview_connect.telekom.contract_importer');
            $importer->setConfigFromProfile($profileId);

            if(!$importer->status($profileId)) {
                throw new Exception('status check failed');
            }

            $list = $importer->fetchContractsList($profileId);

            $view->data['list'] = var_export($list, true);
        }

        if($_SERVER['REQUEST_METHOD'] === 'GET' && @isset($_GET['telekom_href'])) {
            $profileId = $_GET['profile_id'];

            if($profileId <= 0) {
                throw new Exception('no profile selected');
            }

            /** @var ContractImporter $importer */
            $importer = isys_application::instance()->container->get('smview_connect.telekom.contract_importer');
            $importer->setConfigFromProfile($profileId);

            if(!$importer->status($profileId)) {
                throw new Exception('status check failed');
            }

            $response = $importer->exec($profileId, 'GET', $importer->getBaseUri().base64_decode($_GET['telekom_href']));
            $contract = json_decode($response->getContent());

            $view->data['list'] = var_export($contract, true);
        }

        return $view;
    }
}
