<?php

namespace idoit\Module\Shdsmviewconnect\Controller;

use idoit\Controller\Base as BaseController;
use idoit\Exception\Exception;
use SHD\Smview\Connect\Service\ConnectLogger;
use SHD\Smview\Connect\Service\Vsphere\Import\VmImporter;
use idoit\Module\Shdsmviewconnect\View\Vsphere\SettingsView;
use idoit\View\Renderable;
use isys_application;
use isys_module;
use isys_module_shd_smview_connect;
use isys_register;
use Monolog\Logger;
use SHD\Smview\DB\Schema;

/**
 * @author Florian Weber <florian.weber@shd-online.de>
 */
class Vsphere extends AddonController
{
    public function __construct(isys_module $p_module)
    {
        parent::__construct($p_module);

        $this->setup(isys_module_shd_smview_connect::CONNECT_MODULE_VSPHERE_PROFILE, isys_module_shd_smview_connect::CONNECT_MODULE_VSPHERE);
    }

    public function pre(isys_register $request, BaseController $controller)
    {
        parent::pre($request, $controller);

        $this->checkDbSchema(['vsphere']);
    }

    public function handle(isys_register $p_request, isys_application $p_application): Renderable
    {
        return $this->settings($p_request);
    }

    public function settings(isys_register $request): Renderable
    {
        $this->navbar->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__SAVE'));

        $vSphereConfig = isys_application::instance()->container->get('smview_connect.vsphere.config');

        if ($_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__SAVE')) {
            $vSphereConfig->save_config($_POST);
            $vSphereConfig->export_config();
        }

        return new SettingsView($request);
    }

    public function dev(isys_register $request): Renderable
    {
        /** @var VmImporter $importer */
        $importer = isys_application::instance()->container->get('smview_connect.vsphere.profile_4.vm_importer');
        $importer->import();

        return new SettingsView($request);
    }
}
