<?php

namespace idoit\Module\Shdsmviewconnect\View\Aeneis;

use Exception;
use SHD\Smview\Connect\Service\Aeneis\AeneisConfiguration;
use idoit\View\Base as ViewBase;
use idoit\Model\Dao\Base as DaoBase;
use idoit\View\Renderable;
use isys_application;
use isys_component_template as ComponentTemplate;
use isys_module as Module;
use isys_module_shd_smview_connect;
use isys_settings;

/**
 * Config View
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Steve Gröber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class ConfigView extends ViewBase
{

    /**
     * Get settings.
     *
     * @return array
     */
    public static function getSettings(): ?array
    {
        $s = isys_settings::get(isys_module_shd_smview_connect::CONNECT_MODULE_AENEIS_SETTINGS, '');
        return ! is_array($s) ? json_decode($s, true) : $s;
    }

    /**
     * Process configuration page.
     *
     * @param Module            $module
     * @param ComponentTemplate $template
     * @param DaoBase           $model
     *
     * @return Renderable
     * @throws Exception
     */
    public function process(Module $module, ComponentTemplate $template, DaoBase $model): Renderable
    {
        /** @var AeneisConfiguration */
        $config = isys_application::instance()->container->get('smview_connect.aeneis.config');
        $l_rules = $config->getRules();

        $l_rules['C__AENEIS__PASSWORD']['p_strValue'] = '******';
        $l_rules['C__AENEIS__GUID_IT_SERVICE_OBJECT_TYPE']['p_strTable'] = 'isys_obj_type';
        $l_rules['C__AENEIS__GUID_PROCESS_OBJECT_TYPE']['p_strTable'] = 'isys_obj_type';
        $l_rules['C__AENEIS__GUID_EMPLOYEE_OBJECT_TYPE']['p_strTable'] = 'isys_obj_type';

        /*
            Possible default values:

            $guid_process_directory = '94813513-f62a-4bde-acd6-1e57092fb889';
            $guid_process = '73e4b32acf5b3b94:1f0d7b8:fcf54cf6d6:-7f93';
            $guid_it_service_directory = 'aca72dec-ea0b-4e50-9bac-096b691d134a';
            $guid_it_service = '45534719-e882-4770-861b-cbe7703d45e7';
            $guid_employee_directory = '10fb3ab1-4090-48c9-b8e3-b05f6a9a513e';
            $guid_employee = '73e4b32acf5b3b94:1f0d7b8:fcf54cf6d6:-7f85';
         */

        $template->smarty_tom_add_rules("tom.content.bottom.content", $l_rules);
        $this->paths['contentbottomcontent'] = isys_module_shd_smview_connect::getPath().'templates/aeneis/config.tpl';
        return $this;
    }
}
