<?php

namespace idoit\Module\Shdsmviewconnect\View\Aeneis;

use Exception;
use SHD\Smview\Connect\Service\Aeneis\Connector;
use idoit\View\Base as ViewBase;
use idoit\Model\Dao\Base as DaoBase;
use idoit\View\Renderable;
use isys_application;
use isys_component_template as ComponentTemplate;
use isys_module as Module;
use isys_module_shd_smview_connect;

/**
 * Export View
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Steve Gröber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class ExportView extends ViewBase
{
    /**
     * @param Module            $module
     * @param ComponentTemplate $template
     * @param DaoBase           $model
     *
     * @return Renderable
     * @throws Exception
     */
    public function process(Module $module, ComponentTemplate $template, DaoBase $model): Renderable
    {
        $jsConfig = [
            'api_props' => ImportView::getApiConfig(),
            'image_url' => \isys_helper_link::get_base() . '/images/',
            'types'     => Connector::getTypes(),
            'language'  => $this->_getJsTranslations(),
            'mode'      => 'export',
        ];

        $template
            ->activate_editmode()
            ->assign('assets_dir', isys_module_shd_smview_connect::get_assets_www_dir())
            ->assign('js_config', json_encode($jsConfig));
        $this->paths['contentbottomcontent'] = isys_module_shd_smview_connect::getPath().'templates/aeneis/export.tpl';
        return $this;
    }

    /**
     * Get translated strings to use with javascript.
     *
     * @return array
     * @throws Exception
     */
    private function _getJsTranslations(): array
    {
        // language templates
        $language_templates = [
            'LC__UNIVERSAL__TYPE',
            'LC__UNIVERSAL__YES',
            'LC__UNIVERSAL__NO',
            'LC__UNIVERSAL__OBJECT_ID',
            'LC__UNIVERSAL__DESCRIPTION',
            'LC__CMDB__SMVIEW_CONNECT__AENEIS_GUID_IN_IDOIT',
            'LC__CMDB__SMVIEW_CONNECT__AENEIS_GUID',
            'LC__CMDB__SMVIEW_CONNECT__AENEIS_IN_AENEIS',
            'LC__CMDB__SMVIEW_CONNECT__AENEIS_OPEN_OBJECT_IN_CMDB',
            'LC__UNIVERSAL__EXPORT',
            'LC__CMDB__SMVIEW_CONNECT__AENEIS_SEARCH',
            'LC__CMDB__SMVIEW_CONNECT__AENEIS_ERROR_EXPORT',
        ];

        // translate
        $language = [];
        $lang = isys_application::instance()->container->get('language');
        foreach ($language_templates as $l) {
            $language[$l] = $lang->get($l);
        }
        return $language;
    }
}
