<?php

namespace idoit\Module\Shdsmviewconnect\View\Aeneis;

use Exception;
use SHD\Smview\Connect\Service\Aeneis\Connector;
use idoit\View\Base as ViewBase;
use idoit\Model\Dao\Base as DaoBase;
use idoit\View\Renderable;
use isys_application;
use isys_component_template as ComponentTemplate;
use isys_helper_link;
use isys_module as Module;
use isys_module_shd_smview_connect;

/**
 * Import View
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Steve Gröber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class ImportView extends ViewBase
{
    /**
     * @param  Module            $module
     * @param  ComponentTemplate $template
     * @param  DaoBase           $model
     * @return Renderable
     * @throws Exception
     */
    public function process(Module $module, ComponentTemplate $template, DaoBase $model): Renderable
    {
        $jsConfig = [
            'api_props' => self::getApiConfig(),
            'image_url' => isys_helper_link::get_base() . '/images/',
            'types'     => Connector::getTypes(),
            'language'  => $this->_getJsTranslations(),
            'mode'      => 'import',
        ];

        $template
            ->activate_editmode()
            ->assign('assets_dir', isys_module_shd_smview_connect::get_assets_www_dir())
            ->assign('js_config', json_encode($jsConfig));
        $this->paths['contentbottomcontent'] = isys_module_shd_smview_connect::getPath().'templates/aeneis/import.tpl';
        return $this;
    }

    /**
     * Get API configuration of this mandator.
     *
     * @return array
     */
    public static function getApiConfig(): array
    {
        $sysDb = isys_application::instance()->database_system;
        $session = isys_application::instance()->session;

        $mandatorId = (int)$session->get_mandator_id();
        $data = $sysDb->fetch_row_assoc($sysDb->query("SELECT isys_mandator__apikey apiKey FROM isys_mandator WHERE isys_mandator__id = $mandatorId;"));

        return [
            'json_rpc_url' => isys_helper_link::get_base() .'/src/jsonrpc.php',
            'api_key'      => $data['apiKey'],
            'session'      => $_COOKIE['PHPSESSID'],
            'lang'         => isys_application::instance()->language,
        ];
    }

    /**
     * Get translated strings to use with javascript.
     *
     * @return array
     * @throws Exception
     */
    private function _getJsTranslations(): array
    {
        // language templates
        $language_templates = [
            'LC__UNIVERSAL__TYPE',
            'LC__UNIVERSAL__YES',
            'LC__UNIVERSAL__NO',
            'LC__CMDB__SMVIEW_CONNECT__AENEIS_RESOLVE_LINKS',
            'LC__CMDB__SMVIEW_CONNECT__AENEIS_OPEN_OBJECT_IN_CMDB',
            'LC__CMDB__SMVIEW_CONNECT__AENEIS_OPEN_PROPERTY_DESCRIPTIONS',
            'LC__CMDB__SMVIEW_CONNECT__AENEIS_SEARCH',
            'LC__OVERVIEW__AENEIS_NAME',
            'LC__CMDB__SMVIEW_CONNECT__AENEIS_GUID',
            'LC__CMDB__SMVIEW_CONNECT__AENEIS_IN_CMDB',
            'LC__CMDB__SMVIEW_CONNECT__AENEIS_USED_OBJECTS',
            'LC__UNIVERSAL__IMPORT',
            'LC__CMDB__SMVIEW_CONNECT__AENEIS_ERROR_IMPORT',
        ];

        // translate
        $language = [];
        $lang = isys_application::instance()->container->get('language');
        foreach ($language_templates as $l) {
            $language[$l] = $lang->get($l);
        }
        return $language;
    }
}
