<?php

namespace idoit\Module\Shdsmviewconnect\View\Gcache;

use idoit\View\Base as ViewBase;
use idoit\Model\Dao\Base as DaoBase;
use idoit\View\Renderable;
use isys_application;
use isys_component_template as ComponentTemplate;
use isys_module as Module;
use isys_module_shd_smview_connect;
use isys_tenantsettings;
use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\Domain\Gcache\GcacheEntryMapping;
use SHD\Smview\Connect\Service\DB\DaoUtil;

class IndexView extends ViewBase
{
    public function process(Module $module, ComponentTemplate $template, DaoBase $model): Renderable
    {
        $isEditable = true;

        $lang = isys_application::instance()->language;

        $profile_id = isys_tenantsettings::get(isys_module_shd_smview_connect::CONNECT_MODULE_GCACHE_PROFILE, null);

        if(!(DaoUtil::allRows(ProfileDAO::getInstance()->get_data($profile_id))[0] ?? null)) {
            //es wurde eine profilid in der db/Cache gefunden die garnicht mehr exisitiert; wenn die im react ankommt dann lädt das nicht
            $profile_id = null;
        }

        $apiProps = [
            'json_rpc_url' => isys_module_shd_smview_connect::getAppDir() .'/src/jsonrpc.php',
            'api_key'      => isys_module_shd_smview_connect::getApiKey(),
            'session'      => $_COOKIE['PHPSESSID'],
            'lang'         => $lang,
        ];

        $mapping = [];
        foreach (get_object_vars(new GcacheEntryMapping()) as $key => $value) {
            $mapping[] = [
                'id' => null,
                'key' => $key,
                'value' => $value,
                'from_template' => true,
            ];
        }

        // create JS configuration
        $jsConfig = [
            'api_props'            => $apiProps,
            'image_url'            => isys_module_shd_smview_connect::getAppDir() . '/images/icons/silk/',
            'spinner_url'          => isys_module_shd_smview_connect::getAppDir() . '/images/ajax-loading.gif',
            'language'             => isys_module_shd_smview_connect::getJsTranslations(),
            'is_editable'          => $isEditable,
            'connect_module_name'  => isys_module_shd_smview_connect::CONNECT_MODULE_GCACHE,
            'profile_id_binding'   => isys_module_shd_smview_connect::CONNECT_MODULE_GCACHE_PROFILE_UI,
            'api_method_bases'     => [
            ],
            'mapping_template' => $mapping,
        ];

        $template
            ->assign('js_config', json_encode($jsConfig))
            ->assign('profile_ui', isys_module_shd_smview_connect::CONNECT_MODULE_GCACHE_PROFILE_UI)
            ->assign('profile_id', $profile_id)
            ->assign('assets_dir', isys_module_shd_smview_connect::get_assets_www_dir())
        ;

        $this->paths['contentbottomcontent'] = isys_module_shd_smview_connect::getPath().'templates/gcache/index.tpl';

        return $this;
    }
}
