<?php

namespace idoit\Module\Shdsmviewconnect\View\Main;

use idoit\View\Base as ViewBase;
use idoit\Model\Dao\Base as DaoBase;
use idoit\View\Renderable;
use isys_component_template as ComponentTemplate;
use isys_component_template_navbar;
use isys_module as Module;
use isys_module_shd_smview_connect;

/**
 * License Generator View
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Florian Weber <florian.weber@shd-online.de>
 * @copyright  2022 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class LicenseGeneratorView extends ViewBase
{
    /**
     * @param Module $module
     * @param ComponentTemplate $template
     * @param DaoBase $model
     * @return Renderable
     */
    public function process(Module $module, ComponentTemplate $template, DaoBase $model): Renderable
    {
        $rules = [];
        $rules['C__SMVIEW_CONNECT__VSPHERE_ENABLED']["p_arData"] = get_smarty_arr_YES_NO();
        $rules['C__SMVIEW_CONNECT__VSPHERE_VALID_UNTIL']['p_strPopupType'] = 'calendar';
        $rules['C__SMVIEW_CONNECT__LDAP_ENABLED']["p_arData"] = get_smarty_arr_YES_NO();
        $rules['C__SMVIEW_CONNECT__LDAP_VALID_UNTIL']['p_strPopupType'] = 'calendar';
        $rules['C__SMVIEW_CONNECT__TELEKOM_ENABLED']["p_arData"] = get_smarty_arr_YES_NO();
        $rules['C__SMVIEW_CONNECT__TELEKOM_VALID_UNTIL']['p_strPopupType'] = 'calendar';
        $rules['C__SMVIEW_CONNECT__BARAMUNDI_ENABLED']["p_arData"] = get_smarty_arr_YES_NO();
        $rules['C__SMVIEW_CONNECT__BARAMUNDI_VALID_UNTIL']['p_strPopupType'] = 'calendar';
        $rules['C__SMVIEW_CONNECT__AZURE_ENABLED']["p_arData"] = get_smarty_arr_YES_NO();
        $rules['C__SMVIEW_CONNECT__AZURE_VALID_UNTIL']['p_strPopupType'] = 'calendar';
        $rules['C__SMVIEW_CONNECT__AENEIS_ENABLED']["p_arData"] = get_smarty_arr_YES_NO();
        $rules['C__SMVIEW_CONNECT__AENEIS_VALID_UNTIL']['p_strPopupType'] = 'calendar';
        $template
            ->activate_editmode()
            ->smarty_tom_add_rules("tom.content.bottom.content", $rules);

        $navbar = isys_component_template_navbar::getInstance();
        $navbar->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__EDIT'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__SAVE'))
            ->set_visible(false, defined_or_default('C__NAVBAR_BUTTON__CANCEL'))
            ->append_button('LC__CMDB__SMVIEW_CONNECT__GENERATE_LICENSE', 'generate', [
                "icon"       => "icons/silk/page_save.png",
                "js_onclick" => "document.isys_form.navMode.value='" . C__NAVMODE__SAVE . "'; document.isys_form.submit();",
            ]);

        $this->paths['contentbottomcontent'] = isys_module_shd_smview_connect::getPath().'templates/main/license_generator.tpl';

        return $this;
    }
}
