<?php

namespace idoit\Module\Shdsmviewconnect\View\Main;

use SHD\Smview\Connect\Service\ShdLicense;
use idoit\View\Base as ViewBase;
use idoit\Model\Dao\Base as DaoBase;
use idoit\View\Renderable;
use isys_component_template as ComponentTemplate;
use isys_module as Module;
use isys_module_shd_smview_connect;

/**
 * License View
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Florian Weber <florian.weber@shd-online.de>
 * @copyright  2022 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class LicenseView extends ViewBase
{
    /**
     * @param Module $module
     * @param ComponentTemplate $template
     * @param DaoBase $model
     * @return Renderable
     */
    public function process(Module $module, ComponentTemplate $template, DaoBase $model): Renderable
    {
        $license = new ShdLicense();

        $adapter = [
            'telekom' => [
                'licensed' => \isys_module_shd_smview_connect_telekom::isLicensed(),
                'shd' => \isys_module_shd_smview_connect_telekom::isLicensedViaSHD(),
                'idoit' => \isys_module_shd_smview_connect_telekom::isLicensedViaIdoit(),
            ],
            'baramundi' => [
                'licensed' => \isys_module_shd_smview_connect_baramundi::isLicensed(),
                'shd' => \isys_module_shd_smview_connect_baramundi::isLicensedViaSHD(),
                'idoit' => \isys_module_shd_smview_connect_baramundi::isLicensedViaIdoit(),
            ],
            'azure' => [
                'licensed' => \isys_module_shd_smview_connect_azure::isLicensed(),
                'shd' => \isys_module_shd_smview_connect_azure::isLicensedViaSHD(),
                'idoit' => \isys_module_shd_smview_connect_azure::isLicensedViaIdoit(),
            ],
            'vsphere' => [
                'licensed' => \isys_module_shd_smview_connect_vsphere::isLicensed(),
                'shd' => \isys_module_shd_smview_connect_vsphere::isLicensedViaSHD(),
                'idoit' => \isys_module_shd_smview_connect_vsphere::isLicensedViaIdoit(),
            ],
            'ldap' => [
                'licensed' => \isys_module_shd_smview_connect_ldap::isLicensed(),
                'shd' => \isys_module_shd_smview_connect_ldap::isLicensedViaSHD(),
                'idoit' => \isys_module_shd_smview_connect_ldap::isLicensedViaIdoit(),
            ],
            'aeneis' => [
                'licensed' => \isys_module_shd_smview_connect_aeneis::isLicensed(),
                'shd' => \isys_module_shd_smview_connect_aeneis::isLicensedViaSHD(),
                'idoit' => \isys_module_shd_smview_connect_aeneis::isLicensedViaIdoit(),
            ],
        ];

        $template
            ->assign('has_license', $license->has_license())
            ->assign('license_data', $license->get_license_data())
            ->assign('adapter', $adapter)
        ;

        $this->paths['contentbottomcontent'] = isys_module_shd_smview_connect::getPath().'templates/main/license.tpl';

        return $this;
    }
}
