<?php

namespace idoit\Module\Shdsmviewconnect\View\Main;

use SHD\Smview\Connect\Service\ConnectLogger;
use SHD\Smview\Connect\Service\AbstractConfiguration;
use idoit\Module\Shdsmviewconnect\View\BaseView;
use idoit\Model\Dao\Base as DaoBase;
use idoit\View\Renderable;
use isys_application;
use isys_component_template as ComponentTemplate;
use isys_exception_general;
use isys_module as Module;
use isys_module_shd_smview_connect;
use Monolog\Logger;

/**
 * Settings View
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Florian Weber <florian.weber@shd-online.de>
 * @copyright  2022 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class SettingsView extends BaseView
{
    /**
     * @param Module $module
     * @param ComponentTemplate $template
     * @param DaoBase $model
     * @return Renderable
     * @throws isys_exception_general
     */
    public function process(Module $module, ComponentTemplate $template, DaoBase $model): Renderable
    {
        $language = isys_application::instance()->container->get('language');
        $rules = AbstractConfiguration::get_rules_by_mapping(AbstractConfiguration::IDOIT_FIELD_MAPPING);

        if (empty($rules['C__SMVIEW_CONNECT__IDOIT_APIKEY']['p_strValue'])) {
            $apiKey = isys_module_shd_smview_connect::getApiKey();
            $rules['C__SMVIEW_CONNECT__IDOIT_APIKEY']['p_strValue'] = $apiKey;
        }

        $loggingRules = AbstractConfiguration::get_rules_by_mapping(isys_module_shd_smview_connect::LOGGING_FIELD_MAPPING);
        $rules = array_merge($rules, $loggingRules);

        $logLevels = [
            Logger::DEBUG   => $language->get('LC__CMDB__SMVIEW_CONNECT__LOG_LEVEL_DEBUG'),
            Logger::INFO    => $language->get('LC__CMDB__SMVIEW_CONNECT__LOG_LEVEL_INFO'),
            Logger::WARNING => $language->get('LC__CMDB__SMVIEW_CONNECT__LOG_LEVEL_WARNING'),
            Logger::ERROR   => $language->get('LC__CMDB__SMVIEW_CONNECT__LOG_LEVEL_ERROR'),
        ];
        $rules['C__SMVIEW_CONNECT__LOG_LEVEL']['p_arData'] = $logLevels;

        foreach ($this->data['adapters'] as $key => $adapter) {
            $rules[$key]["p_arData"] = get_smarty_arr_YES_NO();
            $rules[$key]['p_strSelectedID'] = $adapter['enabled'] ?? 1;
        }

        $template
            ->assign('encType', 'multipart/form-data')
            ->assign('adapters', $this->data['adapters'])
            ->assign('www_dir', isys_module_shd_smview_connect::get_www_dir())
            ->smarty_tom_add_rules("tom.content.bottom.content", $rules)
        ;

        $this->paths['contentbottomcontent'] = isys_module_shd_smview_connect::getPath().'templates/main/settings.tpl';

        return $this;
    }
}
