<?php

namespace idoit\Module\Shdsmviewconnect\View\Telekom;

use Exception;
use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Telekom\Auth\TelekomAuthenticator;
use SHD\Smview\Connect\Service\Telekom\TelekomConfiguration;
use idoit\View\Base as ViewBase;
use idoit\Model\Dao\Base as DaoBase;
use idoit\View\Renderable;
use isys_application;
use isys_auth;
use isys_auth_auth;
use isys_component_template as ComponentTemplate;
use isys_module as Module;
use isys_module_shd_smview_connect;

/**
 * Telekom Cert Upload View
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Florian Weber <florian.weber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class CertificateUploadView extends ViewBase
{
    /**
     * @param Module $module
     * @param ComponentTemplate $template
     * @param DaoBase $model
     * @return Renderable
     * @throws Exception
     */
    public function process(Module $module, ComponentTemplate $template, DaoBase $model): Renderable
    {
        $canEdit = isys_auth_auth::instance()->is_allowed_to(isys_auth::EDIT, 'MODULE/C__MODULE__SHD_SMVIEW_CONNECT');

        if($canEdit) {
            $template->activate_editmode();
        }

        /** @var TelekomConfiguration $telekomConfig */
        $telekomConfig = isys_application::instance()->container->get('smview_connect.telekom.config');

        $rules = $telekomConfig->getRules();

        //profile certs
        $profiles = array_map(function ($p) {
            $p['crt_path'] = TelekomAuthenticator::getClientCertificatePath($p['id']);
            $p['crt'] = file_exists($p['crt_path']);
            $p['key_path'] = TelekomAuthenticator::getClientCertificatePath($p['id'], true);
            $p['key'] = file_exists($p['key_path']);

            return $p;
        }, DaoUtil::allRows(ProfileDAO::getInstance()->get_data(null, isys_module_shd_smview_connect::CONNECT_MODULE_TELEKOM)));

        $template
            ->assign('profiles', $profiles)
            ->assign('assets_dir', isys_module_shd_smview_connect::get_assets_www_dir())
            ->assign('image_url', isys_module_shd_smview_connect::getAppDir().'/images/icons/silk/')
            ->assign('encType', 'multipart/form-data')
            ->smarty_tom_add_rules("tom.content.bottom.content", $rules);

        $this->paths['contentbottomcontent'] = isys_module_shd_smview_connect::getPath().'templates/telekom/cert_upload.tpl';

        return $this;
    }
}
