<?php

namespace idoit\Module\Shdsmviewconnect\View\ValuePipeline;

use Exception;
use idoit\View\Base as ViewBase;
use idoit\Model\Dao\Base as DaoBase;
use idoit\View\Renderable;
use isys_application;
use isys_component_template as ComponentTemplate;
use isys_module as Module;
use isys_module_shd_smview_connect;

/**
 * ValuePipeline View
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Florian Weber <florian.weber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class FunctionsView extends ViewBase
{
    /**
     * @param Module $module
     * @param ComponentTemplate $template
     * @param DaoBase $model
     * @return Renderable
     * @throws Exception
     */
    public function process(Module $module, ComponentTemplate $template, DaoBase $model): Renderable
    {
        $isEditable = false;

        if (
            $_POST[C__GET__NAVMODE]
            && $_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__EDIT')
            || $_POST[C__CMDB__GET__EDITMODE]
            && $_POST[C__CMDB__GET__EDITMODE]== defined_or_default('C__EDITMODE__ON')
        ) {
            $isEditable = true;
        }

        if (
            $_POST[C__GET__NAVMODE]
            && $_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__SAVE')
        ) {
            // disable edit mode after save
            $isEditable = false;
        }

        $rules = [];

        $lang = isys_application::instance()->language;

        $apiProps = [
            'json_rpc_url' => isys_module_shd_smview_connect::getAppDir() .'/src/jsonrpc.php',
            'api_key'      => isys_module_shd_smview_connect::getApiKey(),
            'session'      => $_COOKIE['PHPSESSID'],
            'lang'         => $lang,
        ];

        // create JS configuration
        $jsConfig = [
            'api_props'            => $apiProps,
            'image_url'            => isys_module_shd_smview_connect::getAppDir() . '/images/icons/silk/',
            'spinner_url'          => isys_module_shd_smview_connect::getAppDir() . '/images/ajax-loading.gif',
            'language'             => isys_module_shd_smview_connect::getJsTranslations(),
            'is_editable'          => $isEditable,
            'api_method_bases'     => [
            ],
        ];

        $template
            ->assign('js_config', json_encode($jsConfig))
            ->assign('assets_dir', isys_module_shd_smview_connect::get_assets_www_dir())
            ->assign('encType', 'multipart/form-data')
            ->smarty_tom_add_rules("tom.content.bottom.content", $rules);

        $this->paths['contentbottomcontent'] = isys_module_shd_smview_connect::getPath().'templates/value_pipeline/functions.tpl';

        return $this;
    }
}
