<?php

namespace idoit\Module\Shdsmviewconnect\View\Vsphere;

use SHD\Smview\Connect\DAO\ProfileDAO;
use SHD\Smview\Connect\Service\DB\DaoUtil;
use SHD\Smview\Connect\Service\Vsphere\VsphereConfiguration;
use SHD\Smview\Connect\Service\Cron;
use idoit\View\Base as ViewBase;
use idoit\Model\Dao\Base as DaoBase;
use idoit\View\Renderable;
use isys_application;
use isys_component_template as ComponentTemplate;
use isys_exception_database;
use isys_module as Module;
use isys_module_shd_smview_connect;
use isys_tenantsettings;

/**
 * Vsphere View
 *
 * @package    i-doit
 * @subpackage smview_connect
 * @author     Florian Weber <florian.weber@shd-online.de>
 * @copyright  2022 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class SettingsView extends ViewBase
{
    /**
     * @param Module $module
     * @param ComponentTemplate $template
     * @param DaoBase $model
     * @return Renderable
     * @throws isys_exception_database
     */
    public function process(Module $module, ComponentTemplate $template, DaoBase $model): Renderable
    {
        $isEditable = false;

        if (
            $_POST[C__GET__NAVMODE]
            && $_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__EDIT')
            || $_POST[C__CMDB__GET__EDITMODE]
            && $_POST[C__CMDB__GET__EDITMODE] == defined_or_default('C__EDITMODE__ON')
        ) {
            $isEditable = true;
        }

        if (
            $_POST[C__GET__NAVMODE]
            && $_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__SAVE')
        ) {
            // disable edit mode after save
            $isEditable = false;
        }

        $lang = isys_application::instance()->language;

        /** @var VSphereConfiguration $vSphereConfig */
        $vSphereConfig = isys_application::instance()->container->get('smview_connect.vsphere.config');

        $profile_id = isys_tenantsettings::get(isys_module_shd_smview_connect::CONNECT_MODULE_VSPHERE_PROFILE, null);

        if(!(DaoUtil::allRows(ProfileDAO::getInstance()->get_data($profile_id))[0] ?? null)) {
            //es wurde eine profilid in der db/Cache gefunden die garnicht mehr exisitiert; wenn die im react ankommt dann lädt das nicht
            $profile_id = null;
        }

        $apiProps = [
            'json_rpc_url' => isys_module_shd_smview_connect::getAppDir() .'/src/jsonrpc.php',
            'api_key'      => isys_module_shd_smview_connect::getApiKey(),
            'session'      => $_COOKIE['PHPSESSID'],
            'lang'         => $lang,
        ];

        // create JS configuration
        $js_MappingEditorConfig = [
            'api_props'            => $apiProps,
            'image_url'            => isys_module_shd_smview_connect::getAppDir() . '/images/icons/silk/',
            'spinner_url'          => isys_module_shd_smview_connect::getAppDir() . '/images/ajax-loading.gif',
            'language'             => isys_module_shd_smview_connect::getJsTranslations(),
            'is_editable'          => $isEditable,
            'source_fields'        => $vSphereConfig->getAttributes($profile_id, true),
            'connect_module_name'  => isys_module_shd_smview_connect::CONNECT_MODULE_VSPHERE,
            'object_types'         => isys_module_shd_smview_connect::getObjectTypes(),
            'profile_id_binding'   => isys_module_shd_smview_connect::CONNECT_MODULE_VSPHERE_PROFILE_UI,
            'object_type_mapping_keys' => $vSphereConfig->getObjectTypeMappingKeys(),
            'settings_keys'        => $vSphereConfig->getSettingsKeys(),
            'api_method_bases'     => [
                //['section' => 'run', 'base' => 'connect.vsphere'],
                ['section' => 'status', 'base' => 'connect.vsphere'],
                ['section' => 'dryrun', 'base' => 'connect.vsphere'],
            ],
            'data_mapping_config'         => $vSphereConfig->getDataMappingConfig(),
            'dev_mode' => isys_module_shd_smview_connect::isDevMode(),
        ];

        $cron = isys_tenantsettings::get(VSphereConfiguration::VSPHERE_FIELD_MAPPING['vsphere.cron']['db_key'], Cron::getDefaultCronForNotConfigured());

        $rules = $vSphereConfig->getRules();

        $template
            ->assign('js_config', json_encode($js_MappingEditorConfig))
            ->assign('profile_ui', isys_module_shd_smview_connect::CONNECT_MODULE_VSPHERE_PROFILE_UI)
            ->assign('profile_id', $profile_id)
            ->assign('assets_dir', isys_module_shd_smview_connect::get_assets_www_dir())
            ->assign('translated_cron', Cron::getDateTimeFromCron($cron)->format('d.m.Y H:i:s'))
            ->smarty_tom_add_rules("tom.content.bottom.content", $rules);

        $this->paths['contentbottomcontent'] = isys_module_shd_smview_connect::getPath().'templates/vsphere/settings.tpl';

        return $this;
    }
}
