<?php

namespace SHD\Smview\Connect\Service;

use SHD\Smview\Connect\Service\Cache\CacheArray;
use PHPUnit\Framework\TestCase;

/**
 * @covers \SHD\Smview\Connect\Service\CacheManager
 */
class CacheManagerTest extends TestCase
{
    private CacheManager $cacheManager;

    protected function setUp(): void
    {
        /** @var CacheManager $cm */
        $cm = \isys_application::instance()->container->get('smview_connect.cache');
        $this->cacheManager = $cm;
    }

    /**
     * @depends testPersist
     */
    public function testGetCaches()
    {
        $caches = $this->cacheManager->getCaches($this->getCacheDir());

        $this->assertIsArray($caches);
        $this->assertCount(1, $caches);
    }

    public function testPersist()
    {
        $cache = new CacheArray('test', $this->getCacheDir());
        $cache->set(['foo' => 'bar']);

        $this->cacheManager->persist($cache);

        $this->assertTrue(true);
    }

    /**
     * @depends testPersist
     */
    public function testRetreive()
    {
        $cache = $this->cacheManager->retreive('test', $this->getCacheDir());

        $this->assertNotNull($cache);
        $this->assertEquals('bar', $cache->get()['foo']);

        $this->assertTrue($cache->isValid());

        $noCache = $this->cacheManager->retreive('false', $this->getCacheDir());
        $this->assertNull($noCache);
    }

    /**
     * @depends testPersist
     * @depends testRetreive
     */
    public function testInvalidate()
    {
        $cache = $this->cacheManager->retreive('test', $this->getCacheDir());

        $this->assertNotNull($cache);

        $this->assertTrue($cache->isValid());

        $this->cacheManager->invalidate($cache);
        $cache = $this->cacheManager->retreive('test', $this->getCacheDir());

        $this->assertFalse($cache->isValid());
    }

    private function getCacheDir(): string
    {
        $dir = __DIR__.'/../../store/tests/cache';
        mkdir($dir);

        return $dir;
    }
}
