<?php

namespace SHD\Smview\Connect\Service\DB;

use PHPUnit\Framework\TestCase;

/**
 * @covers \SHD\Smview\Connect\Service\DB\DaoUtil
 */
class DaoUtilTest extends TestCase
{
    public function testPluck()
    {
        $array = [
            [
                'foo' => 'bar',
                'baz' => 'bat',
            ],
            [
                'foo' => 'fizz',
                'baz' => 'buzz',
            ]
        ];

        $o = new \stdClass();
        $o->foo = 'pale';
        $o->baz = 'face';
        $array[] = $o;

        $a = DaoUtil::pluck($array, 'foo', 'bar');

        $this->assertIsArray($a);
        $this->assertEquals('bat', $a['baz']);

        $b = DaoUtil::pluck($array, 'foo', 'fizz');

        $this->assertIsArray($b);
        $this->assertEquals('buzz', $b['baz']);

        $c = DaoUtil::pluck($array, 'foo', 'pale');

        $this->assertIsObject($c);
        $this->assertEquals('face', $c->baz);

        $array[] = [
            'wrong' => 'format',
        ];

        $d = DaoUtil::pluck($array, 'wrong', 'format');

        $this->assertIsArray($d);
        $this->assertArrayNotHasKey('foo', $d);
        $this->assertEquals('format', $d['wrong']);
    }

    public function testWhere()
    {
        $array = [
            [
                'foo' => 'bar',
                'baz' => 'bat',
            ],
            [
                'foo' => 'fizz',
                'baz' => 'buzz',
            ],
            [
                'foo' => 'bar',
                'baz' => 'buzz',
            ],
        ];

        $o = new \stdClass();
        $o->foo = 'pale';
        $o->baz = 'face';
        $array[] = $o;

        $a = DaoUtil::where($array, 'foo', 'bar');

        $this->assertIsArray($a);
        $this->assertCount(2, $a);
    }

    public function testGuard()
    {
        $a = DaoUtil::guard(true, function () {
            return 'foo';
        });

        $this->assertEquals('foo', $a);

        $b = DaoUtil::guard(false, function () {
            return 'bar';
        });

        $this->assertNotEquals('bar', $b);
        $this->assertNull($b);

        $c = DaoUtil::guard(false, function () {
            return 'baz';
        }, 'bat');

        $this->assertNotEquals('baz', $c);
        $this->assertEquals('bat', $c);
    }
}
