<?php

namespace SHD\Smview\Connect\Service\Import;

use DateTime;
use Exception;
use SHD\Smview\Connect\DAO\CustomFunctionDAO;
use SHD\Smview\Connect\Domain\idoit\DynamicValueAssignment;
use PHPUnit\Framework\TestCase;

/**
 * @covers \SHD\Smview\Connect\Service\Import\ValuePipeline
 */
class ValuePipelineTest extends TestCase
{
    private ValuePipeline $pipeline;

    protected function setUp(): void
    {
        /** @var ValuePipeline $p */
        $p = \isys_application::instance()->container->get('smview_connect.value_pipeline');
        $this->pipeline = $p;
    }

    public function testTransform()
    {
        //builtin
        //date_time
        $pa1 = new DynamicValueAssignment();
        $pa1->meta1 = ValuePipeline::BUILTIN_DATE_TIME;

        $this->pipeline->load($pa1);

        $v1 = '1704381292';
        $v2 = '01.01.2024 16:08:23';
        $v3 = 'fail';

        $r1 = $this->pipeline->transform($v1, null);
        $this->assertEquals(new DateTime($v1), $r1);

        $r2 = $this->pipeline->transform($v2, null);
        $this->assertEquals(new DateTime($v2), $r2);

        $this->expectException(Exception::class);
        $this->pipeline->transform($v3, null);

        //noop
        $pa2 = new DynamicValueAssignment();
        $pa2->meta1 = ValuePipeline::BUILTIN_NOOP;

        $this->pipeline->load($pa2);

        $v1 = 'success';
        $v2 = ['success'];

        $r1 = $this->pipeline->transform($v1, null);
        $this->assertEquals($v1, $r1);

        $r2 = $this->pipeline->transform($v2, null);
        $this->assertEquals($v2, $r2);

        //array0
        $pa3 = new DynamicValueAssignment();
        $pa3->meta1 = ValuePipeline::BUILTIN_ARRAY_0;

        $this->pipeline->load($pa3);

        $v1 = ['foo', 'bar'];
        $v2 = ['bar'];
        $v3 = 'fizz';
        $v4 = ['this' => 'is ok'];

        $r1 = $this->pipeline->transform($v1, null);
        $this->assertEquals('foo', $r1);

        $r2 = $this->pipeline->transform($v2, null);
        $this->assertEquals('bar', $r2);

        $r3 = $this->pipeline->transform($v3, null);
        $this->assertEquals($v3, $r3);

        $r4 = $this->pipeline->transform($v4, null);
        $this->assertEquals($v4, $r4);

        //concat
        $pa4 = new DynamicValueAssignment();
        $pa4->meta1 = ValuePipeline::BUILTIN_ARRAY_0
            .'|'.ValuePipeline::BUILTIN_DATE_TIME
            .'|'.ValuePipeline::BUILTIN_NOOP;

        $this->pipeline->load($pa4);

        $v1 = ['1704381292', '1704382585'];

        $r1 = $this->pipeline->transform($v1, null);
        $this->assertEquals(new DateTime($v1[0]), $r1);

        //todo: test custom
    }
}
